using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Windows.Forms;
using RDPCrystalEDILibrary;
using RDPCrystalEDILibrary.Docs.x5010;
using RDPCrystalEDILibrary.PrinterForms;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to convert and 5010 837 Professional to a CMS 1500 Form
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class CMS1500_From_5010_837P : UserControl
    {
        private CMS1500Form cms1500 = new CMS1500Form();
        private CMSFormData cms;
        private EDIValidator ediValidator = new EDIValidator();

        public CMS1500_From_5010_837P()
        {
            InitializeComponent();
        }

        private void InitializeEDI()
        {
            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = "ValidationRules\\Rules_5010_837P_005010X222A1.Rules";

            txtEDIData.Text = File.ReadAllText("EDIFiles\\Sample_5010_837P_To_Convert_To_CMS1500.txt");

            //Set EDI file to validate
            ediValidator.EDIFile = "EDIFiles\\Sample_5010_837P_To_Convert_To_CMS1500.txt";
        }

        private void LoadEDI()
        {
            try
            {
                ediValidator.Validate();

                //The EDI Document will contain all the data of the 837 file
                if (ediValidator.EDILightWeightDocument != null)
                {
                    Typed5010Document doc = new Typed5010Document(ediValidator.EDILightWeightDocument);

                    //display a tree of the EDI data
                    ediDocumentViewer.LoadDocument(ediValidator.EDILightWeightDocument);

                    DocumentLoop interchange = doc.MainSection.GetLoop("INTERCHANGE HEADER");
                    ISA isa = interchange.GetSegment<ISA>();

                    DocumentLoop billingProviderLevel = doc.MainSection.GetLoop("INTERCHANGE HEADER/FUNCTIONAL GROUP/ST HEADER/2000A");

                    DocumentLoop billingProviderLoop = billingProviderLevel.GetLoop("2010AA");
                    NM1 billProv = billingProviderLoop.GetSegment<NM1>();
                    N3 street = billingProviderLoop.GetSegment<N3>();
                    N4 address = billingProviderLoop.GetSegment<N4>();

                    // Box 33 Set the Billing Provider Data
                    cms.Box33 = billProv.NameLastOrOrganizationName + " " + billProv.FirstName + " " + billProv.MiddleName + Environment.NewLine;
                    cms.Box33 = cms.Box33 + street.AddressInformation1 + " " + street.AddressInformation2 + Environment.NewLine;
                    cms.Box33 = cms.Box33 + address.CityName + " " + address.StateOrProvinceCode + " " + address.PostalCode;

                    //Box 33A Set the Billing Provider Data
                    cms.Box33A = billProv.IdentificationCode;

                    //Box 25 Federal ID Number
                    List<REF> refs = billingProviderLoop.GetSegments<REF>("REF");
                    foreach (REF id in refs)
                    {
                        if (id.ReferenceIdentificationQualifier == "SY") //Social Security Number
                        {
                            cms.Box25A = id.ReferenceIdentification;
                            cms.Box25B = true;
                        }
                        else if (id.ReferenceIdentificationQualifier == "EI") //EIN number
                        {
                            cms.Box25A = id.ReferenceIdentification;
                            cms.Box25C = true;
                        }
                    }

                    //Box 33B Provider ID
                    PRV prv = billingProviderLevel.GetSegment<PRV>();
                    if (prv != null)
                    {
                        cms.Box33B = prv.ReferenceIdentification;
                    }

                    //Box 33C Provider Contact - Phone Number
                    PER contact = billingProviderLoop.GetSegment<PER>();
                    if (contact != null)
                    {
                        cms.Box33C = contact.CommunicationNumber1; 
                    }

                    DocumentLoop subscriberLoop = doc.MainSection.GetLoop("INTERCHANGE HEADER/FUNCTIONAL GROUP/ST HEADER/2000A/2000B");
                    CreateCMS(subscriberLoop);

                    cms1500.Data.Add(cms);

                    printPreview.Document = cms1500.PrintDocument;
                }
            }
            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        private void CreateCMS(DocumentLoop subscriberLoop)
        {
            SBR sbr = subscriberLoop.GetSegment<SBR>();

            //Box 1 Plan Type
            switch (sbr.ClaimFilingIndicatorCode)
            {
                case "MA":
                case "MB":
                    {
                        cms.Box1A = true;
                        break;
                    }
                case "MC":
                    {
                        cms.Box1B = true;
                        break;
                    }
                case "CH":
                    {
                        cms.Box1C = true;
                        break;
                    }
                case "VA":
                    {
                        cms.Box1D = true;
                        break;
                    }
                default:
                    {
                        cms.Box1E = true;
                        break;
                    }
            }

            //Header 
            NM1 payerName = subscriberLoop.GetSegment<NM1>("2010BB/NM1");
            N3 payerStreet = subscriberLoop.GetSegment<N3>("2010BB/N3");
            N4 payerAddress = subscriberLoop.GetSegment<N4>("2010BB/N4");

            cms.BoxHeader = payerName.NameLastOrOrganizationName + Environment.NewLine + payerStreet.AddressInformation1 + Environment.NewLine;
            cms.BoxHeader = cms.BoxHeader + payerAddress.CityName + " " + payerAddress.StateOrProvinceCode + " " + payerAddress.PostalCode;

            if (sbr.IndividualRelationshipCode == "18")   //Patient and subscriber is the same person
            {
                #region Patient == Insured

                NM1 subName = subscriberLoop.GetSegment<NM1>("2010BA/NM1");

                //Insured's ID Number
                cms.Box1Aa = subName.IdentificationCode;

                //Patient Name
                cms.Box2 = subName.NameLastOrOrganizationName + " " + subName.FirstName + " " + subName.MiddleName;

                DMG demographics = subscriberLoop.GetSegment<DMG>("2010BA/DMG");

                //Patient Birth date
                DateTime birthDate = DateTime.ParseExact(demographics.DateTimePeriod, "yyyyMMdd", CultureInfo.InvariantCulture);

                cms.Box3A = birthDate;

                //Patient Sex
                switch (demographics.GenderCode)
                {
                    case "M":
                        {
                            cms.Box3B = true;

                            break;
                        }
                    case "F":
                        {
                            cms.Box3C = true;

                            break;
                        }
                }

                //Insured's name
                cms.Box4 = subName.NameLastOrOrganizationName + " " + subName.FirstName + " " + subName.MiddleName;

                //Patient's street address
                N3 subAddress = subscriberLoop.GetSegment<N3>("2010BA/N3");
                cms.Box5A = subAddress.AddressInformation1 + " " + subAddress.AddressInformation2;

                //Patient city, state and zipcode
                N4 subAddress2 = subscriberLoop.GetSegment<N4>("2010BA/N4");
                cms.Box5B = subAddress2.CityName;
                cms.Box5C = subAddress2.StateOrProvinceCode;
                cms.Box5D = subAddress2.PostalCode;

                //Patient's relationship to insured, Self because the patient is the subscriber
                cms.Box6A = true;

                //Insured Address
                cms.Box7A = subAddress.AddressInformation1 + " " + subAddress.AddressInformation2;

                //Insured city, state, zip
                cms.Box7B = subAddress2.CityName;
                cms.Box7C = subAddress2.StateOrProvinceCode;
                cms.Box7D = subAddress2.PostalCode;

                //Box 11aa Insured DOB
                cms.Box11A = DateTime.ParseExact(demographics.DateTimePeriod, "yyyyMMdd", CultureInfo.InvariantCulture);

                //Box 11 ab Patient Sex
                switch (demographics.GenderCode)
                {
                    case "M":
                        {
                            cms.Box11Ab = true;

                            break;
                        }
                    case "F":
                        {
                            cms.Box11Ac = true;

                            break;
                        }
                }

                ProcessClaim(subscriberLoop.GetLoop("2300"));

                #endregion
            }
            else
            {
                #region Patient != Insured

                NM1 subName = subscriberLoop.GetSegment<NM1>("2010BA/NM1");
            
                //Insured's ID Number
                cms.Box1Aa = subName.IdentificationCode;

                //Insured's name
                cms.Box4 = subName.NameLastOrOrganizationName + " " + subName.FirstName + " " + subName.MiddleName;

                //Insured address
                N3 subAddress = subscriberLoop.GetSegment<N3>("2010BA/N3");
                cms.Box7A = subAddress.AddressInformation1 + " " + subAddress.AddressInformation2;
                N4 subAddress2 = subscriberLoop.GetSegment<N4>("2010BA/N4");
                cms.Box7B = subAddress2.CityName;
                cms.Box7C = subAddress2.StateOrProvinceCode;
                cms.Box7D = subAddress2.PostalCode;

                DMG subDemographic = subscriberLoop.GetSegment<DMG>("2010BA/DMG");
                //Box 11aa Insured DOB
                cms.Box11A = DateTime.ParseExact(subDemographic.DateTimePeriod, "yyyyMMdd", CultureInfo.InvariantCulture);

                //Box 11 ab Patient Sex
                switch (subDemographic.GenderCode)
                {
                    case "M":
                        {
                            cms.Box11Ab = true;

                            break;
                        }
                    case "F":
                        {
                            cms.Box11Ac = true;

                            break;
                        }
                }

                //Get the patient information
                DocumentLoop patientLoop = subscriberLoop.GetLoop("2000C");

                //Patient Name
                NM1 patientName = patientLoop.GetSegment<NM1>("2010CA/NM1");
              
                cms.Box2 = patientName.NameLastOrOrganizationName + " " + patientName.FirstName + " " + patientName.MiddleName;

                //Patient Address
                N3 patAddress = patientLoop.GetSegment<N3>("2010CA/N3");
                cms.Box5A = patAddress.AddressInformation1 + " " + patAddress.AddressInformation2;
                N4 patAddress2 = patientLoop.GetSegment<N4>("2010CA/N4");
                cms.Box5B = patAddress2.CityName;
                cms.Box5C = patAddress2.StateOrProvinceCode;
                cms.Box5D = patAddress2.PostalCode;

                //Patient relationship to insured
                PAT pat = patientLoop.GetSegment<PAT>("PAT");
                switch (pat.IndividualRelationshipCode)
                {
                    case "01": //Spouse
                        {
                            cms.Box6B = true;
                            break;
                        }
                    case "19": //Child
                        {
                            cms.Box6C = true;
                            break;
                        }
                    default: //Other
                        {
                            cms.Box6D = true;
                            break;
                        }
                }

                DMG patDemographic = patientLoop.GetSegment<DMG>("2010CA/DMG");

                //Patient Birth date
                DateTime birthDate = DateTime.ParseExact(patDemographic.DateTimePeriod, "yyyyMMdd", CultureInfo.InvariantCulture);
                cms.Box3A = birthDate;

                //Patient Sex
                switch (patDemographic.GenderCode)
                {
                    case "M":
                        {
                            cms.Box3B = true;

                            break;
                        }
                    case "F":
                        {
                            cms.Box3C = true;

                            break;
                        }
                }

                ProcessClaim(patientLoop.GetLoop("2300"));

                #endregion
            }

            //Box 11
            cms.Box11 = sbr.ReferenceIdentification;

            //Box 11B - Other Ids
            List<REF> refs = subscriberLoop.GetSegments<REF>("2010BA/REF");
            foreach (REF r in refs)
            {
                if (r.ReferenceIdentificationQualifier == "Y4")
                {
                    cms.Box11B = r.ReferenceIdentification;
                }
            }

            //Box 11C - Plan or program name
            cms.Box11C = sbr.Name1;
        }

        private void ProcessClaim(DocumentLoop clmLoop)
        {
            CLM clm = clmLoop.GetSegment<CLM>();

            if (clm.RelatedCausesInformation.RelatedCausesCode1 == "EM")
            {
                cms.Box10Aa = true;
            }
            else
            {
                cms.Box10Ab = true;
            }

            if (clm.RelatedCausesInformation.RelatedCausesCode1 == "AA")
            {
                cms.Box10Ba = true;
                cms.Box10Bc = clm.RelatedCausesInformation.StateOrProvinceCode;
            }
            else
            {
                cms.Box10Bb = true;
            }

            if (clm.RelatedCausesInformation.RelatedCausesCode1 == "OA")
            {
                cms.Box10Ca = true;
            }
            else
            {
                cms.Box10Cb = true;
            }

            //Box 12 Release of Information
            if (clm.ReleaseOfInformationCode == "Y")
            {
                cms.Box12A = "SIGNATURE OF FILE";
            }

            //Box 13 Authorized person's signature
            if (clm.YesNoConditionOrResponseCode2 == "Y")
            {
                cms.Box13 = "SIGNATURE OF FILE";
            }

            //Box 11D
            //Presence of Loop 2320 indicates Y (yes) to the question.
            DocumentLoop otherInsuranceLoop = clmLoop.GetLoop("2300/2320");
            if (otherInsuranceLoop != null)
            {
                cms.Box11Da = true;
            }
            else
            {
                cms.Box11Db = true;
            }

            List<DTP> dates = clmLoop.GetSegments<DTP>("DTP");

            foreach (DTP date in dates)
            {
                if (date.DateTimeQualifier == "431") //Box 14 Date of current illness, injury or pregnancy (LMP)
                {
                    cms.Box14A = DateTime.ParseExact(date.DateTimePeriod, "yyyyMMdd", CultureInfo.InvariantCulture);
                    cms.Box14B = date.DateTimeQualifier;
                }
                else if (date.DateTimeQualifier == "297") //Box 16a - Disability/Unable to Work From
                {
                    cms.Box16A = DateTime.ParseExact(date.DateTimePeriod, "yyyyMMdd", CultureInfo.InvariantCulture);
                }
                else if (date.DateTimeQualifier == "296") //Box 16b -  Disability/Unable to Work To
                {
                    cms.Box16B = DateTime.ParseExact(date.DateTimePeriod, "yyyyMMdd", CultureInfo.InvariantCulture);
                }
                else if (date.DateTimeQualifier == "435") //Box 18a -  Admission/Hospitalized From
                {
                    cms.Box18A = DateTime.ParseExact(date.DateTimePeriod, "yyyyMMdd", CultureInfo.InvariantCulture);
                }
                else if (date.DateTimeQualifier == "096") //Box 18b -  Discharge/Hospitalized
                {
                    cms.Box18B = DateTime.ParseExact(date.DateTimePeriod, "yyyyMMdd", CultureInfo.InvariantCulture);
                }
            }

            //Box 17 Referring provider
            DocumentLoop referringProvider = clmLoop.GetLoop("2310A");
            if (referringProvider != null)
            {
                NM1 rName = referringProvider.GetSegment<NM1>();

                cms.Box17 = rName.NameLastOrOrganizationName + " " + rName.FirstName + " " + rName.MiddleName;
                cms.Box17B = rName.IdentificationCode;
            }

            List<REF> ids = clmLoop.GetSegments<REF>("REF");
            foreach (REF id in ids)
            {
                if (id.ReferenceIdentificationQualifier == "9F") //17 a Referral Number
                {
                    cms.Box17A = id.ReferenceIdentification;
                }
                else if (id.ReferenceIdentificationQualifier == "F8") //Box 22B  Control Number
                {
                    cms.Box22B = id.ReferenceIdentification;
                }
                else if (id.ReferenceIdentificationQualifier == "G1") //Box 23  Prior Authorizaiion Number
                {
                    cms.Box23 = id.ReferenceIdentification;
                }
            }

            //Box 19 additional claim note
            NTE claimNote = clmLoop.GetSegment<NTE>("NTE");
            if (claimNote != null)
            {
                if (claimNote.NoteReferenceCode == "ADD")
                {
                    cms.Box19 = claimNote.Description;
                }
            }

            //Box 26 Patient account number
            cms.Box26 = clm.ClaimSubmitterIdentifier;

            //Box 32 Provider Signature Indicator
            if (clm.YesNoConditionOrResponseCode1 == "Y")
            {
                cms.Box31A = "SIGNATURE ON FILE";
            }

            //Box 28 Claim Amount
            cms.Box28 = Decimal.Parse(clm.MonetaryAmount);

            //Box 27 Provider Signature Indicator
            if (clm.AssignmentOrPlanParticipationCode == "Y")
            {
                cms.Box27A = true;
            }
            else
            {
                cms.Box27B = true;
            }

            //Get the other subscriber information loop
            DocumentLoop otherSubscriberLoop = clmLoop.GetLoop("2320");

            if (otherSubscriberLoop != null)
            {
                //Box 9 Other subscriber
                NM1 otherSubName = otherSubscriberLoop.GetSegment<NM1>("2330A/NM1");
                cms.Box9 = otherSubName.NameLastOrOrganizationName + " " + otherSubName.FirstName + " " + otherSubName.MiddleName;

                //Box 9A Policy/Group Number 
                SBR otherSub = otherSubscriberLoop.GetSegment<SBR>();

                //Box 9B Plan/Program
                cms.Box9A = otherSub.ReferenceIdentification;
                cms.Box9D = otherSub.Name1;

                //Box 29 Amount Paid

                List<AMT> amts = otherSubscriberLoop.GetSegments<AMT>("AMT");
                foreach (AMT amt in amts)
                {
                    //Box 29 Monetary Amount
                    if (amt.AmountQualifierCode == "D")
                    {
                        cms.Box29 = Decimal.Parse(amt.MonetaryAmount);
                    }
                }

                //Box 11D. Presence of Loop 2320 indicates Y (yes) to the question.
                cms.Box11Da = true;
            }
            else
            {
                cms.Box11Db = true;
            }

            HI diagnosticCodes = clmLoop.GetSegment<HI>();

            //Box 21
            cms.Box21A = diagnosticCodes.HealthCareCodeInformation1.IndustryCode1;
            cms.Box21B = diagnosticCodes.HealthCareCodeInformation2.IndustryCode1;
            cms.Box21C = diagnosticCodes.HealthCareCodeInformation3.IndustryCode1;
            cms.Box21D = diagnosticCodes.HealthCareCodeInformation4.IndustryCode1;
            cms.Box21E = diagnosticCodes.HealthCareCodeInformation5.IndustryCode1;
            cms.Box21F = diagnosticCodes.HealthCareCodeInformation6.IndustryCode1;
            cms.Box21G = diagnosticCodes.HealthCareCodeInformation7.IndustryCode1;
            cms.Box21H = diagnosticCodes.HealthCareCodeInformation8.IndustryCode1;
            cms.Box21I = diagnosticCodes.HealthCareCodeInformation9.IndustryCode1;
            cms.Box21J = diagnosticCodes.HealthCareCodeInformation10.IndustryCode1;
            cms.Box21K = diagnosticCodes.HealthCareCodeInformation11.IndustryCode1;
            cms.Box21L = diagnosticCodes.HealthCareCodeInformation12.IndustryCode1;

            string renderingProviderID = "";
            string prvID = "";

            string otherIDQualifier = "";
            string otherID = "";

            DocumentLoop renderingProvider = clmLoop.GetLoop("2310B");
            if (renderingProvider != null)
            {
                NM1 nm = renderingProvider.GetSegment<NM1>();
                renderingProviderID = nm.IdentificationCode;

                PRV prv = renderingProvider.GetSegment<PRV>();
                if (prv != null)
                {
                    prvID = prv.ReferenceIdentification;
                }

                List<REF> otherIDs = renderingProvider.GetSegments<REF>("REF");
                foreach (REF id in otherIDs)
                {
                    if (id.ReferenceIdentificationQualifier == "G2" || id.ReferenceIdentificationQualifier == "0B" || id.ReferenceIdentificationQualifier == "IG")
                    {
                        otherID = id.ReferenceIdentification;
                        otherIDQualifier = id.ReferenceIdentificationQualifier;
                    }
                }
            }

            DocumentLoop serviceLocationLoop = clmLoop.GetLoop("2310C");
            if (serviceLocationLoop != null)
            {
                NM1 sfName = serviceLocationLoop.GetSegment<NM1>("NM1");
                N3 sfStreet = serviceLocationLoop.GetSegment<N3>("N3");
                N4 sfAddress = serviceLocationLoop.GetSegment<N4>("N4");

                cms.Box32 = sfName.NameLastOrOrganizationName + Environment.NewLine + sfStreet.AddressInformation1 + Environment.NewLine;
                cms.Box32 = cms.Box32 + sfAddress.CityName + " " + sfAddress.StateOrProvinceCode + " " + sfAddress.PostalCode;

                cms.Box32A = sfName.IdentificationCode;
            }

            List<DocumentLoop> serviceLines = clmLoop.GetLoops("2400");
            foreach (DocumentLoop sv in serviceLines)
            {
                ProcessServiceLine(clm, sv, renderingProviderID, prvID, otherIDQualifier, otherID);
            }
        }

        private void ProcessServiceLine(CLM clm, DocumentLoop slLoop, string renderingProviderID, string prvID, string otherIDQualifier, string otherID)
        {
            Box24 sv = new Box24();
            cms.Box24.Add(sv);

            //Get all dates in the service line
            List<DTP> dates = slLoop.GetSegments<DTP>("DTP");

            //Box 24a - From
            foreach (DTP date in dates)
            {
                if (date.DateTimeQualifier == "472")
                {
                    if (date.DateTimePeriodFormatQualifier == "D8")
                    {
                        sv.Box24Aa = DateTime.ParseExact(date.DateTimePeriod, "yyyyMMdd", CultureInfo.InvariantCulture);
                    }
                    else if (date.DateTimePeriodFormatQualifier == "RD8")
                    {
                        string[] dateParts = date.DateTimePeriod.Split('-');

                        sv.Box24Aa = DateTime.ParseExact(dateParts[0], "yyyyMMdd", CultureInfo.InvariantCulture);
                        sv.Box24Ab = DateTime.ParseExact(dateParts[1], "yyyyMMdd", CultureInfo.InvariantCulture);
                    }

                    break;
                }
            }

            SV1 sv1 = slLoop.GetSegment<SV1>();

            //Place of service code
            sv.Box24B = clm.HealthCareServiceLocation.FacilityCodeValue;

            //EMG
            sv.Box24C = sv1.YesNoConditionOrResponseCode1;

            //Box 24D - CPT Code
            sv.Box24D = sv1.CompositeMedicalProcedureIdentifier.ProductServiceID1;

            //Box 24D Modifiers
            sv.Box24Mod1 = sv1.CompositeMedicalProcedureIdentifier.ProcedureModifier1;
            sv.Box24Mod2 = sv1.CompositeMedicalProcedureIdentifier.ProcedureModifier2;
            sv.Box24Mod3 = sv1.CompositeMedicalProcedureIdentifier.ProcedureModifier3;
            sv.Box24Mod4 = sv1.CompositeMedicalProcedureIdentifier.ProcedureModifier4;

            sv.Box24E = sv1.CompositeDiagnosticCodePointer.DiagnosisCodePointer1 + " " + sv1.CompositeDiagnosticCodePointer.DiagnosisCodePointer2 + " " + sv1.CompositeDiagnosticCodePointer.DiagnosisCodePointer3 + " " + sv1.CompositeDiagnosticCodePointer.DiagnosisCodePointer4;

            sv.Box24F = Decimal.Parse(sv1.MonetaryAmount1);
            sv.Box24G = sv1.Quantity;

            //EPSDT Indicator
            sv.Box24H = sv1.YesNoConditionOrResponseCode2;

            //Check if the Rendering Provider was overriden
            DocumentLoop renderingProvider = slLoop.GetLoop("2420A");
            if (renderingProvider != null)
            {
                prvID = String.Empty;
                otherIDQualifier = String.Empty;
                otherID = String.Empty;

                NM1 rn = renderingProvider.GetSegment<NM1>();
                renderingProviderID = rn.IdentificationCode;

                PRV prv = renderingProvider.GetSegment<PRV>();
                if (prv != null)
                {
                    prvID = prv.ReferenceIdentification;
                }

                if (prvID == String.Empty)
                {
                    List<REF> otherIDs = renderingProvider.GetSegments<REF>("REF");
                    foreach (REF id in otherIDs)
                    {
                        if (id.ReferenceIdentificationQualifier == "G2" || id.ReferenceIdentificationQualifier == "0B" || id.ReferenceIdentificationQualifier == "IG")
                        {
                            sv.Box24IShaded = id.ReferenceIdentification;
                            sv.Box24JShaded = id.ReferenceIdentificationQualifier; ;

                            break;
                        }
                    }
                }
                else
                {
                    sv.Box24IShaded = otherIDQualifier;
                    sv.Box24JShaded = prvID;
                }
            }
            else
            {
                if (prvID == string.Empty)
                {
                    sv.Box24IShaded = otherIDQualifier;
                    sv.Box24JShaded = otherID;
                }
                else
                {
                    sv.Box24JShaded = prvID;
                }
            }

            //Rendering Provider ID
            sv.Box24J = renderingProviderID;
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            cms = new CMSFormData();

            InitializeEDI();

            LoadEDI();
        }

        private void btnPrint_Click(object sender, EventArgs e)
        {
            cms1500.Print();
        }

        private void lnk_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://www.rdpcrystal.com/cms1500form/");
        }
    }
}