﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Windows.Forms;
using RDPCrystalEDILibrary;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to create a 4010 856 Shipment Notice EDI document using the EDIDocument component
    /// and display the new document in the EDIDocumentViewer component
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Create_4010_856_File : UserControl
    {
        private EDIDocument sampleEDI856File;
        private ShipmentNotice shipmentNotice;

        public Create_4010_856_File()
        {
            InitializeComponent();

            CreateShipmentNotice();
        }

        #region Private Methods

        private void CreateShipmentNotice()
        {
            shipmentNotice = new ShipmentNotice();
            shipmentNotice.ShipmentID = "ww8VH00";
            shipmentNotice.NoticeType = NoticeType.PickAndPack;
            shipmentNotice.DateCreated = DateTime.Today;
            shipmentNotice.ShipmentDate = DateTime.Today;

            shipmentNotice.PackingCode = "CTN25";
            shipmentNotice.WeightingQualifier = "G";
            shipmentNotice.UnitOfMeasure = "LB";
            shipmentNotice.LadingQuantity = 1;
            shipmentNotice.Weight = 1;

            shipmentNotice.Carrier = new Carrier();
            shipmentNotice.Carrier.RoutingSequenceCode = "M";
            shipmentNotice.Carrier.RoutingSequenceID = "UPSN";
            shipmentNotice.Carrier.VendorNumber = "414";
            shipmentNotice.Carrier.ProNbrReference = "1141812E0116945162"; //Carrier's PRO Nbr Reference

            shipmentNotice.ShipToParty = new Party();
            shipmentNotice.ShipToParty.Name = "ABC Company";
            shipmentNotice.ShipToParty.ID = "117107dfd2112";
            shipmentNotice.ShipToParty.Address = "123 Maple Street";
            shipmentNotice.ShipToParty.City = "Queens";
            shipmentNotice.ShipToParty.State = "NY";
            shipmentNotice.ShipToParty.ZipCode = "11212";

            Order order = new Order();
            order.PONumber = "23434";

            Package package = new Package();
            package.PalletNumber = "114X811E0346945512";

            Item item = new Item();
            item.PartNumber = "PartNumber";
            item.UPCNumber = "021285110131";
            item.VenderCatalogNumber = "1200462";
            item.NumberOfUnitsShipped = 200;
            item.UnitOfMeasurement = "EA";

            package.Items.Add(item);
            order.Packages.Add(package);
            shipmentNotice.Orders.Add(order);
        }

        private void CreateEDIFile()
        {
            sampleEDI856File.SegmentSeparatorString = "\r\n";

            sampleEDI856File.FileBufferSize = 16;

            //Create an interchance loop. This loop will contain all other loops in the EDI structure
            Loop interchangeHeaderLoop = new Loop("Interchange Header", "Interchange Header");

            //Create an ISA segment in the interchange loop
            DataSegment isa = interchangeHeaderLoop.CreateSegment("ISA");

            //Add elements with data to the ISA segment
            isa.Elements.Add("00", " ", "00", " ", "ZZ", "InterchangeSenderID", "ZZ", "InterchangeReceiverID", "070303", "18:04", "U", "00401", "1", "1", "T", ":");

            //Now create the function header loop
            Loop functionalHeaderLoop = interchangeHeaderLoop.CreateLoop("GS Functional Header Loop", "Functional Header Loop");

            //Add a GS segment to it
            DataSegment gs = functionalHeaderLoop.CreateSegment("GS");

            //Set its elements and their values
            gs.Elements.Add("SH");
            gs.Elements.Add("ApplicationSenderCode");
            gs.Elements.Add("ApplicationReceiverCode");
            gs.Elements.Add("2005");
            gs.Elements.Add("132334");
            gs.Elements.Add("1");
            gs.Elements.Add("X");
            gs.Elements.Add("004010");

            //Create a transaction header loop
            Loop transactionLoop = functionalHeaderLoop.CreateLoop("Transaction Header", "ST Transaction Header");

            //Add an ST segment to the transaction header loop
            DataSegment st = transactionLoop.CreateSegment("ST");

            //Set its elements and their values
            st.Elements.Add("856");
            st.Elements.Add("3000012");

            int hierLevel = 1;

            //Add an BSN segment to the transaction header loop
            DataSegment bsn = transactionLoop.CreateSegment("BSN");
            bsn.Elements.Add("00"); //TRANSACTION SET PURPOSE CODE
            bsn.Elements.Add(shipmentNotice.ShipmentID); //SHIPMENT IDENTIFICATION
            bsn.Elements.Add(shipmentNotice.DateCreated.ToString("yyyyMMdd")); //DATE CREATED
            bsn.Elements.Add(shipmentNotice.DateCreated.ToString("HHmmss"));  //TIME CREATED
            bsn.Elements.Add(shipmentNotice.NoticeType == NoticeType.PickAndPack ? "0001" : "0002");  //PICK AND PACK (SHIPMENT, ORDER, PACK, ITEM)

            //SHIPMENT LOOP
            Loop shipmentLoop = transactionLoop.CreateLoop("LOOP HL", "Begin HL Shipment Loop");

            DataSegment hl = shipmentLoop.CreateSegment("HL");
            hl.Elements.Add(hierLevel++.ToString()); //Hierarchy level
            hl.Elements.Add("");
            hl.Elements.Add("S"); //S- Shipment loop

            //Shipment details
            DataSegment td1 = shipmentLoop.CreateSegment("TD1");
            td1.Elements.Add(shipmentNotice.PackingCode);//Packing Code
            td1.Elements.Add(shipmentNotice.LadingQuantity.ToString()); //lading quantity
            td1.Elements.Add("");
            td1.Elements.Add("");
            td1.Elements.Add("");
            td1.Elements.Add(shipmentNotice.WeightingQualifier); //Weighting qualifier
            td1.Elements.Add(shipmentNotice.Weight.ToString()); //Weight
            td1.Elements.Add(shipmentNotice.UnitOfMeasure); //unit of measure

            //Carrier details
            DataSegment td5 = shipmentLoop.CreateSegment("TD5");
            td5.Elements.Add(shipmentNotice.Carrier.RoutingSequenceCode); //Routing sequence qualifier
            td5.Elements.Add("2"); //ID code qualifier
            td5.Elements.Add(shipmentNotice.Carrier.RoutingSequenceID);  //Id code

            //Reference number:Vendor number
            DataSegment refn = shipmentLoop.CreateSegment("REF");
            refn.Elements.Add("IA"); //vendor number
            refn.Elements.Add(shipmentNotice.Carrier.VendorNumber); //Id code qualifier

            //Reference numbers:Carrier's PRO Nbr Reference
            DataSegment refn2 = shipmentLoop.CreateSegment("REF");
            refn.Elements.Add("CN"); //
            refn.Elements.Add(shipmentNotice.Carrier.ProNbrReference); //Carrier's PRO Nbr Reference

            //Shipping Date
            DataSegment dtp = shipmentLoop.CreateSegment("DTP");
            dtp.Elements.Add("011"); //qualifier
            dtp.Elements.Add(shipmentNotice.ShipmentDate.ToString("yyyyMMdd")); //shipment date

            //Shipment party 
            Loop shipmentParty = shipmentLoop.CreateLoop("LOOP NM1", "Shipment Party");

            //Shipment party
            DataSegment n1 = shipmentParty.CreateSegment("N1");
            n1.Elements.Add("ST");  //ST-Ship To
            n1.Elements.Add(shipmentNotice.ShipToParty.Name); //
            n1.Elements.Add("92"); //ID
            n1.Elements.Add(shipmentNotice.ShipToParty.ID); //ID Code

            //Shipment address 1
            DataSegment n3 = shipmentParty.CreateSegment("N3");
            n3.Elements.Add(shipmentNotice.ShipToParty.Address);

            //Shipment address 2
            DataSegment n4 = shipmentParty.CreateSegment("N4");
            n4.Elements.Add(shipmentNotice.ShipToParty.City);
            n4.Elements.Add(shipmentNotice.ShipToParty.State);
            n4.Elements.Add(shipmentNotice.ShipToParty.ZipCode);

            foreach (Order order in shipmentNotice.Orders)
            {
                int orderHierarchy = hierLevel++;

                //ORDER LOOP
                Loop orderLoop = shipmentLoop.CreateLoop("Order Loop", "Order Loop");

                DataSegment hl2 = orderLoop.CreateSegment("HL");
                hl2.Elements.Add(orderHierarchy.ToString()); //Hierarchy level
                hl2.Elements.Add("1");
                hl2.Elements.Add("O");  //O-Order Loop

                //Purchase Order reference
                DataSegment prf = orderLoop.CreateSegment("PRF");
                prf.Elements.Add(order.PONumber); //PO Number

                foreach (Package package in order.Packages)
                {
                    int packageHierarchy = hierLevel++;
                    //PACK LOOP
                    Loop packLoop = orderLoop.CreateLoop("Pack Loop", "Pack Loop");

                    DataSegment packHL = packLoop.CreateSegment("HL");
                    packHL.Elements.Add(packageHierarchy.ToString());
                    packHL.Elements.Add(orderHierarchy.ToString()); //parent HL is order loop
                    packHL.Elements.Add("P");  //P-Pack Loop

                    //Marks and Numbers
                    DataSegment man = packLoop.CreateSegment("MAN");
                    man.Elements.Add("W");
                    man.Elements.Add(package.PalletNumber);   //Pallet Number

                    foreach (Item item in package.Items)
                    {
                        //ITEM LOOP
                        Loop itemLoop = packLoop.CreateLoop("Items", "Items Loop");

                        DataSegment itemsHL = itemLoop.CreateSegment("HL");
                        itemsHL.Elements.Add(hierLevel++.ToString());
                        itemsHL.Elements.Add(packageHierarchy.ToString()); //parent HL is pack loop
                        itemsHL.Elements.Add("I");  //I-Item loop

                        //Line Item detail
                        DataSegment lin = itemLoop.CreateSegment("LIN");
                        lin.Elements.Add(""); //assigned it
                        lin.Elements.Add("CB"); //product/service id. CB- Customer part number
                        lin.Elements.Add(item.PartNumber);  //part number
                        lin.Elements.Add("UP");  //UPC number
                        lin.Elements.Add(item.UPCNumber);
                        lin.Elements.Add("VC");
                        lin.Elements.Add(item.VenderCatalogNumber);

                        //Line Item detail
                        //"SN1**1*EA~" +
                        DataSegment sn1 = itemLoop.CreateSegment("SN1");
                        sn1.Elements.Add(""); //assigned it
                        sn1.Elements.Add(item.NumberOfUnitsShipped.ToString()); // number of units shipped
                        sn1.Elements.Add(item.UnitOfMeasurement);  //units of measurement
                    }
                }
            }

            //Create the end of the transaction (SE segment). This must be done in a loop.
            Loop endOfTransactionLoop = transactionLoop.CreateLoop("End Of Transaction", "SE End Of Transaction");
            DataSegment se = endOfTransactionLoop.CreateSegment("SE");
            se.Elements.Add("0", "Segment Count");
            se.Elements.Add("1");

            //Create the end of the functional group (GE segment). This must be done in a loop.
            Loop endOfFunctionalGroupLoop = functionalHeaderLoop.CreateLoop("GE End Functional Group", "GE End Functional Group");
            DataSegment ge = endOfFunctionalGroupLoop.CreateSegment("GE");
            ge.Elements.Add("0");
            ge.Elements.Add("1");

            //Finally create the end of the interchange loop (IEA segment). This must be done in a loop.
            Loop endInterchangeLoop = interchangeHeaderLoop.CreateLoop("End Interchange", "IEA End Interchange");
            DataSegment iea = endInterchangeLoop.CreateSegment("IEA");
            iea.Elements.Add("0");
            iea.Elements.Add("1");

            //Add the interchange loop to the EDI document
            sampleEDI856File.Loops.Add(interchangeHeaderLoop);
        }

        #endregion

        private void btnGenerate_Click(object sender, EventArgs e)
        {
            try
            {
                sampleEDI856File = new EDIDocument();

                CreateEDIFile();

                rtbGeneratedFile.Text = sampleEDI856File.GenerateEDIData();

                ediDocumentViewer1.LoadDocument(sampleEDI856File);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "RDPCrystal EDI Library");
            }
        }

        private void lnk_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://www.rdpcrystal.com/products/edi-library/edi-document/");
        }
    }

    public enum NoticeType { PickAndPack, StandardCartonPack };

    class ShipmentNotice
    {
        private List<Order> orders = new List<Order>();

        public string ShipmentID
        {
            get;
            set;
        }

        public NoticeType NoticeType
        {
            get;
            set;
        }

        public List<Order> Orders
        {
            get { return orders; }
            set { orders = value; }
        }

        public DateTime DateCreated
        {
            get;
            set;
        }

        public DateTime ShipmentDate
        {
            get;
            set;
        }

        public Carrier Carrier
        {
            get;
            set;
        }

        public string WeightingQualifier
        {
            get;
            set;
        }

        public string PackingCode
        {
            get;
            set;
        }

        public string UnitOfMeasure
        {
            get;
            set;
        }

        public int LadingQuantity
        {
            get;
            set;
        }

        public double Weight
        {
            get;
            set;
        }

        public Party ShipToParty
        {
            get;
            set;
        }
    }

    class Party
    {
        public string Name
        {
            get;
            set;
        }

        public string ID
        {
            get;
            set;
        }

        public string Address
        {
            get;
            set;
        }

        public string City
        {
            get;
            set;
        }

        public string State
        {
            get;
            set;
        }

        public string ZipCode
        {
            get;
            set;
        }
    }

    class Package
    {
        private List<Item> items = new List<Item>();

        public List<Item> Items
        {
            get
            {
                return items;
            }
            set
            {
                items = value;
            }
        }

        public string PalletNumber
        {
            get;
            set;
        }
    }

    class Order
    {
        private List<Package> packages = new List<Package>();

        public string PONumber
        {
            get;
            set;
        }

        public List<Package> Packages
        {
            get { return packages; }
            set { packages = value; }
        }
    }

    class Item
    {
        public string PartNumber
        {
            get;
            set;
        }

        public string UPCNumber
        {
            get;
            set;
        }

        public string VenderCatalogNumber
        {
            get;
            set;
        }

        public int NumberOfUnitsShipped
        {
            get;
            set;
        }

        public string UnitOfMeasurement
        {
            get;
            set;
        }
    }

    class Carrier
    {
        public string VendorNumber
        {
            get;
            set;
        }

        public string ProNbrReference
        {
            get;
            set;
        }

        public string RoutingSequenceCode
        {
            get;
            set;
        }

        public string RoutingSequenceID
        {
            get;
            set;
        }
    }
}
