﻿using System;
using System.Diagnostics;
using System.Windows.Forms;
using RDPCrystalEDILibrary;
using RDPCrystalEDILibrary.Docs.x5010;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrate how to create a 5010 835 835 Health Care Claim Payment Advice EDI document using Typed5010Document
    /// and display the new document in the EDIDocumentViewer component
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Create_5010_835_File : UserControl
    {
        private EDIFileLoader ediFileLoader = new EDIFileLoader();

        public Create_5010_835_File()
        {
            InitializeComponent();
        }

        private void btnCreateEDIFile_Click(object sender, EventArgs e)
        {
            try
            {
                //Create a Typed 5010 Document
                Typed5010Document doc835 = new Typed5010Document();

                //Places the correct number of segments in X12 SE segment
                doc835.AutoPlaceCorrectNumOfSegments = true;
                doc835.AutoPlaceNumOfTransactions = true;
                doc835.AutoPlaceNumOfFunctionalGroups = true;

                //Make sure the ISA element values are the correct length
                doc835.PadISAElementValues = true;

                DocumentLoop interchangeHeader = doc835.MainSection.CreateLoop("Interchange Header");

                // ISA Header
                ISA isa = interchangeHeader.CreateSegment<ISA>();
                isa.AuthorizationInformationQualifier = "00";
                isa.AuthorizationInformation = "";
                isa.SecurityInformationQualifier = "00";
                isa.SecurityInformation = "";
                isa.InterchangeIDQualifier1 = "ZZ";
                isa.InterchangeSenderID = "SenderID";
                isa.InterchangeIDQualifier2 = "ZZ";
                isa.InterchangeReceiverID = "ReceiverID";
                isa.InterchangeDate = "030101";
                isa.InterchangeTime = "1253";
                isa.RepetitionSeparator = "^";
                isa.InterchangeControlVersionNumber = "00501";
                isa.InterchangeControlNumber = "000000905";
                isa.AcknowledgmentRequested = "1";
                isa.InterchangeUsageIndicator = "T";
                isa.ComponentElementSeparator = ":";

                //Create the function header section
                DocumentLoop functionalHeader = interchangeHeader.CreateLoop("Functional Header");
                //GS*HP*54704*0001124*20110726*23562041*344*X*005010X221A1~

                GS gs = new GS();
                functionalHeader.Add(gs);
                gs.FunctionalIdentifierCode = "HP";
                gs.ApplicationSenderCode = "Sender";
                gs.ApplicationReceiverCode = "Receiver";
                gs.Date = "20150303";
                gs.Time = "1424";
                gs.GroupControlNumber = "1234";
                gs.ResponsibleAgencyCode = "X";
                gs.VersionReleaseIndustryIdentifierCode = "005010X221A1";

                //Create a transaction header loop
                DocumentLoop transactionHeader = functionalHeader.CreateLoop("Transaction Header");

                //ST*837*0021*005010X222~
                ST st = transactionHeader.CreateSegment<ST>();
                st.TransactionSetIdentifierCode = "835";
                st.TransactionSetControlNumber = "0021";

                //Payment Remittance
                //BPR*H*500*C*NON************20061015~
                BPR bpr = transactionHeader.CreateSegment<BPR>();
                bpr.TransactionHandlingCode = "H";
                bpr.MonetaryAmount = "500";
                bpr.CreditDebitFlagCode = "C";
                bpr.PaymentMethodCode = "NON";
                bpr.Date = "20061015";

                // Trace Number
                //TRN*1**0000010305*1234567890~
                TRN trn = transactionHeader.CreateSegment<TRN>();
                trn.TraceTypeCode = "1";
                trn.ReferenceIdentification1 = "0000010305";
                trn.OriginatingCompanyIdentifier = "1234567890";

                // Production Date
                // DTM*405*20110706~
                DTM dtm = transactionHeader.CreateSegment<DTM>();
                dtm.DateTimeQualifier = "405";
                dtm.Date = "20110706";

                //PAYER Loop1000A
                AddPayerName(transactionHeader);

                //PayTo Loop1000B
                AddPayToProvider(transactionHeader);

                //ClaimPayment
                AddClaimPay(transactionHeader);

                // Provider Adjustment
                DocumentLoop loop2100 = transactionHeader.CreateLoop("2100");
                //PLB*NPI No*20111231*50>111*6173.4~
                //PLB*NPI No*20111231**6173.4~
                PLB plb = loop2100.CreateSegment<PLB>();
                plb.ReferenceIdentification = "NPI No";
                plb.Date = "20111231";
                plb.AdjustmentIdentifier1.AdjustmentReasonCode = "50";
                plb.AdjustmentIdentifier1.ReferenceIdentification = "111";

                plb.MonetaryAmount1 = "6173.4";

                //Create the end of the transaction (SE segment). This must be done in a loop.
                DocumentLoop endOfTransactionLoop = transactionHeader.CreateLoop("End Of Transaction");

                //SE*42*37~
                SE se = endOfTransactionLoop.CreateSegment<SE>();
                se.TransactionSetControlNumber = "0021";

                //Create the end of the functional group (GE segment). This must be done in a loop.
                DocumentLoop endOfFunctionalGroupLoop = functionalHeader.CreateLoop("End Functional Group");
                GE ge = endOfFunctionalGroupLoop.CreateSegment<GE>();
                ge.GroupControlNumber = "1234";

                //Finally create the end of the interchange loop (IEA segment). This must be done in a loop.
                DocumentLoop endInterchangeSection = interchangeHeader.CreateLoop("End Interchange");
                IEA iea = endInterchangeSection.CreateSegment<IEA>();
                iea.InterchangeControlNumber = "000000905";

                txtFile.Text = doc835.ToEDIString(new Delimiters());

                //Display the new EDI Document
                ediFileLoader.EDIDataString = txtFile.Text;
                ediDocumentViewer1.LoadDocument(ediFileLoader.Load());
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "RDPCrystal EDI Library");
            }
        }

        //Claim Payments Loop2000/Loop2100/Loop2110
        private static void AddClaimPay(DocumentLoop transactionHeader)
        {
            DocumentLoop loop2000 = transactionHeader.CreateLoop("2000");

            // Header Number
            //LX*1~
            LX lx = loop2000.CreateSegment<LX>();
            lx.AssignedNumber = "1";

            // Provider Summary
            //TS3*89*11*20201231*3*745~
            TS3 ts3 = loop2000.CreateSegment<TS3>();
            ts3.ReferenceIdentification = "89";
            ts3.FacilityCodeValue = "11";
            ts3.Date = "20061231";
            ts3.Quantity1 = "3";
            ts3.MonetaryAmount1 = "745";

            //Claim Header Loop2100
            DocumentLoop loop2100 = transactionHeader.CreateLoop("2100");
            //CLP*8000002735678*1*500*175**HM*0000000033219*11~

            CLP clp = loop2100.CreateSegment<CLP>();
            clp.ClaimSubmitterIdentifier = "8000002735678";
            clp.ClaimStatusCode = "1";
            clp.MonetaryAmount1 = "500";
            clp.MonetaryAmount2 = "325";
            clp.MonetaryAmount3 = "175";
            clp.ClaimFilingIndicatorCode = "HM";
            clp.ReferenceIdentification = "0000000033219";
            clp.FacilityCodeValue = "11";

            // Patient
            //NM1*QC*1*GARCIA*HOSEA****MI*0000672-02~
            NM1 PTnm1 = loop2100.CreateSegment<NM1>();
            PTnm1.EntityIdentifierCode = "QC";
            PTnm1.EntityTypeQualifier = "1";
            PTnm1.NameLastOrOrganizationName = "GARCIA";
            PTnm1.FirstName = "HOSEA";
            PTnm1.MiddleName = "";
            PTnm1.IdentificationCodeQualifier = "MI";
            PTnm1.IdentificationCode = "0000672-02";

            // Subscriber
            //NM1*IL*1*SANCHEZ-GARCIA*TRACY*Q***MI*0000672-01~
            NM1 SUBnm1 = loop2100.CreateSegment<NM1>();
            SUBnm1.EntityIdentifierCode = "IL";
            SUBnm1.EntityTypeQualifier = "1";
            SUBnm1.NameLastOrOrganizationName = "SANCHEZ-GARCIA";
            SUBnm1.FirstName = "TRACY";
            SUBnm1.MiddleName = "Q";
            SUBnm1.IdentificationCodeQualifier = "MI";
            SUBnm1.IdentificationCode = "0000672-01";

            // Provider
            // NM1*82*1*DOCTOR*STRANGE****XX*NPI No~
            NM1 DRnm1 = loop2100.CreateSegment<NM1>();
            DRnm1.EntityIdentifierCode = "82";
            DRnm1.EntityTypeQualifier = "1";
            DRnm1.NameLastOrOrganizationName = "STRANGE";
            DRnm1.FirstName = "DOCTOR";
            DRnm1.MiddleName = "X";
            DRnm1.IdentificationCodeQualifier = "XX";
            DRnm1.IdentificationCode = "NPI No";

            // Claim Suppliment Info
            //  AMT*AU*500~
            AMT amt = loop2100.CreateSegment<AMT>();
            amt.AmountQualifierCode = "AU";
            amt.MonetaryAmount = "500";

            //Claim Detail Loop2110
            AddServiceLine(transactionHeader);
        }

        private static void AddServiceLine(DocumentLoop transactionHeader)
        {
            DocumentLoop loop2110 = transactionHeader.CreateLoop("2110");

            // Claim Detail
            // SVC*AD:D5214*500*325**1~
            SVC svc = loop2110.CreateSegment<SVC>();
            svc.CompositeMedicalProcedureIdentifier1.ProductServiceID1 = "D5214";
            svc.CompositeMedicalProcedureIdentifier1.ProductServiceIDQualifier = "AD";
            svc.MonetaryAmount1 = "500";
            svc.MonetaryAmount2 = "325";
            svc.Quantity1 = "1";

            // DTM*472*20191206~
            //service date "472"
            DTM svcDT = loop2110.CreateSegment<DTM>();
            svcDT.DateTimeQualifier = "472";
            svcDT.Date = "20110706";

            // Service Adjustment
            //CAS*PR*3*175~
            CAS cas2 = loop2110.CreateSegment<CAS>();
            cas2.ClaimAdjustmentGroupCode = "PR";
            cas2.ClaimAdjustmentReasonCode1 = "3";
            cas2.MonetaryAmount1 = "175";

            // Sender Ref Line No
            //REF*6R*800000273567801~
            REF ref3 = loop2110.CreateSegment<REF>();
            ref3.ReferenceIdentificationQualifier = "6R";
            ref3.ReferenceIdentification = "800000273567801";

            // Service Line Adjustment
            //AMT*B6*500~
            AMT amt2 = loop2110.CreateSegment<AMT>();
            amt2.AmountQualifierCode = "B6";
            amt2.MonetaryAmount = "500";
        }

        private static void AddPayerName(DocumentLoop transactionHeader)
        {
            //PAYER
            DocumentLoop loop1000A = transactionHeader.CreateLoop("1000A");

            // N1*PR*INDEPENDENCE BLUE CROSS~
            N1 n1 = loop1000A.CreateSegment<N1>();
            n1.EntityIdentifierCode1 = "PR";
            n1.Name1 = "INDEPENDENCE BLUE CROSS";

            // N3*1901 MARKET STREET~
            N3 n3 = loop1000A.CreateSegment<N3>();
            n3.AddressInformation1 = "1901 MARKET STREET";

            // N4*PHILADELPHIA*PHILADELPHIA*19103~
            N4 n4 = loop1000A.CreateSegment<N4>();
            n4.CityName = "PHILADELPHIA";
            n4.StateOrProvinceCode = "PA";
            n4.PostalCode = "19103";

            // PER*CX*IBC Service Desk*TE*111111111~
            PER per1 = loop1000A.CreateSegment<PER>();
            per1.ContactFunctionCode = "CX";
            per1.Name = "IBC Service Desk";
            per1.CommunicationNumberQualifier1 = "TE";
            per1.CommunicationNumber1 = "111111111";

            // PER*BL*John Smith*TE*111111111~
            PER per2 = loop1000A.CreateSegment<PER>();
            per2.ContactFunctionCode = "BL";
            per2.Name = "John Smith";
            per2.CommunicationNumberQualifier1 = "TE";
            per2.CommunicationNumber1 = "111111111";
        }

        private void AddPayToProvider(DocumentLoop transactionHeader)
        {
            //PAY TO PROVIDER
            DocumentLoop loop1000B = transactionHeader.CreateLoop("1000B");
            // N1*PE*CHESTNUT HILL REHAB HOSPITAL*XX*NPI No~

            N1 n1PE = loop1000B.CreateSegment<N1>();
            n1PE.EntityIdentifierCode1 = "PE";
            n1PE.Name1 = "CHESTNUT HILL REHAB HOSPITAL";
            n1PE.IdentificationCodeQualifier = "XX";
            n1PE.IdentificationCode = "NPI No";

            // N3*ATTENTION JOE SAMPLE*8835 MARKET AVENUE~
            N3 n3PE = loop1000B.CreateSegment<N3>();
            n3PE.AddressInformation1 = "ATTENTION JOE SAMPLE";
            n3PE.AddressInformation2 = "8835 MARKET AVENUE";

            // N4*PHILADELPHIA*PA*191182718~
            N4 n4PE = loop1000B.CreateSegment<N4>();
            n4PE.CityName = "PHILADELPHIA";
            n4PE.StateOrProvinceCode = "PA";
            n4PE.PostalCode = "191182718";

            // REF*PQ*Lic Number~
            REF ref1 = loop1000B.CreateSegment<REF>();
            ref1.ReferenceIdentificationQualifier = "PQ";
            ref1.ReferenceIdentification = "Lic Number";

            //  REF*TJ*Tax ID~
            REF ref2 = loop1000B.CreateSegment<REF>();
            ref2.ReferenceIdentificationQualifier = "TJ";
            ref2.ReferenceIdentification = "Tax ID";
        }

        private void lnk_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://www.rdpcrystal.com/products/edi-library/typed-document/");
        }
    }
}
