﻿using System;
using System.Diagnostics;
using System.Windows.Forms;
using RDPCrystalEDILibrary;
using RDPCrystalEDILibrary.Docs.x5010;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrate how to create a 5010 837 Professional EDI document using Typed5010Document
    /// and display the new document in the EDIDocumentViewer component
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Create_5010_837P_File : UserControl
    {
        private EDIFileLoader ediFileLoader = new EDIFileLoader();

        public Create_5010_837P_File()
        {
            InitializeComponent();
        }

        private void btnCreateEDIFile_Click(object sender, EventArgs e)
        {
            try
            {
                //Create a Typed 5010 Document
                Typed5010Document doc837P = new Typed5010Document();

                //Places the correct number of segments in X12 SE segment
                doc837P.AutoPlaceCorrectNumOfSegments = true;
                doc837P.AutoPlaceNumOfTransactions = true;
                doc837P.AutoPlaceNumOfFunctionalGroups = true;
                //Make sure the ISA element values are the correct length
                doc837P.PadISAElementValues = true;

                DocumentLoop interchangeHeader = doc837P.MainSection.CreateLoop("Interchange Header");

                ISA isa = interchangeHeader.CreateSegment<ISA>();
                isa.AuthorizationInformationQualifier = "00";
                isa.AuthorizationInformation = "";
                isa.SecurityInformationQualifier = "00";
                isa.SecurityInformation = "";
                isa.InterchangeIDQualifier1 = "ZZ";
                isa.InterchangeSenderID = "SenderID";
                isa.InterchangeIDQualifier2 = "ZZ";
                isa.InterchangeReceiverID = "ReceiverID";
                isa.InterchangeDate = "030101";
                isa.InterchangeTime = "1253";
                isa.RepetitionSeparator = "^";
                isa.InterchangeControlVersionNumber = "00501";
                isa.InterchangeControlNumber = "000000905";
                isa.AcknowledgmentRequested = "1";
                isa.InterchangeUsageIndicator = "T";
                isa.ComponentElementSeparator = ":";

                //Create the function header section
                DocumentLoop functionalHeader = interchangeHeader.CreateLoop("Functional Header");

                GS gs = new GS();
                functionalHeader.Add(gs);
                gs.FunctionalIdentifierCode = "HC";
                gs.ApplicationSenderCode = "ApplicationSenderCode";
                gs.ApplicationReceiverCode = "ApplicationReceiverCode";
                gs.Date = "20150303";
                gs.Time = "1424";
                gs.GroupControlNumber = "1234";
                gs.ResponsibleAgencyCode = "X";
                gs.VersionReleaseIndustryIdentifierCode = "005010X222";

                //Create a transaction header loop
                DocumentLoop transactionHeader = functionalHeader.CreateLoop("Transaction Header");
                //ST*837*0021*005010X222~
                ST st = transactionHeader.CreateSegment<ST>();
                st.TransactionSetIdentifierCode = "837";
                st.TransactionSetControlNumber = "0021";
                st.ImplementationConventionReference = "005010X222";

                //BHT*0019*00*244579*20061015*1023*CH~
                BHT bht = transactionHeader.CreateSegment<BHT>();
                bht.HierarchicalStructureCode = "0019";
                bht.TransactionSetPurposeCode = "10";
                bht.ReferenceIdentification = "244579";
                bht.Date = "20061015";
                bht.Time = "1023";
                bht.TransactionTypeCode = "CH";

                AddSubmitter(transactionHeader);

                AddReceiver(transactionHeader);

                AddBillingProvider(transactionHeader);

                AddSubscriber(transactionHeader);

                //Create the end of the transaction (SE segment). This must be done in a loop.
                DocumentLoop endOfTransactionLoop = transactionHeader.CreateLoop("End Of Transaction");

                //SE*42*37~
                SE se = endOfTransactionLoop.CreateSegment<SE>();
                se.TransactionSetControlNumber = "42";

                //Create the end of the functional group (GE segment). This must be done in a loop.
                DocumentLoop endOfFunctionalGroupLoop = functionalHeader.CreateLoop("End Functional Group");
                GE ge = endOfFunctionalGroupLoop.CreateSegment<GE>();
                ge.GroupControlNumber = "11234";

                //Finally create the end of the interchange loop (IEA segment). This must be done in a loop.
                DocumentLoop endInterchangeSection = interchangeHeader.CreateLoop("End Interchange");
                IEA iea = endInterchangeSection.CreateSegment<IEA>();
                iea.InterchangeControlNumber = "123";

                txtFile.Text = doc837P.ToEDIString(new Delimiters());

                //Display the new EDI Document
                ediFileLoader.EDIDataString = txtFile.Text;
                ediDocumentViewer1.LoadDocument(ediFileLoader.Load());
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "RDPCrystal EDI Library");
            }
        }

        private void AddBillingProvider(DocumentLoop transactionHeader)
        {
            DocumentLoop loop2000A = transactionHeader.CreateLoop("2000A BILLING PROVIDER HIERARCHICAL LEVEL");

            //HL*1**20*1~
            HL hl2000C = loop2000A.CreateSegment<HL>();
            hl2000C.HierarchicalIDNumber = "1";
            hl2000C.HierarchicalLevelCode = "20";
            hl2000C.HierarchicalChildCode = "1";

            //PRV*BI*PXC*203BF0100Y~
            PRV prv = loop2000A.CreateSegment<PRV>();  //BILLING PROVIDER SPECIALTY INFORMATION
            prv.ProviderCode = "BI";
            prv.ReferenceIdentification = "PXC";
            prv.ReferenceIdentification = "203BF0100Y";

            DocumentLoop loop2010AA = loop2000A.CreateLoop("2010AA BILLING PROVIDER NAME");

            //NM1*85*2*BEN KILDARE SERVICE*****XX*9876543210~
            NM1 nm1 = loop2010AA.CreateSegment<NM1>(); //BILLING PROVIDER NAME
            nm1.EntityIdentifierCode = "85";
            nm1.EntityTypeQualifier = "2";
            nm1.NameLastOrOrganizationName = "BEN KILDARE SERVICE";
            nm1.IdentificationCodeQualifier = "XX";
            nm1.IdentificationCode = "9876543210";

            //N3*234 SEAWAY ST~
            N3 n3 = loop2010AA.CreateSegment<N3>();  //BILLING PROVIDER ADDRESS
            n3.AddressInformation1 = "234 SEAWAY ST";

            //N4*MIAMI*FL*33111~
            N4 n4 = loop2010AA.CreateSegment<N4>();  //BILLING PROVIDER LOCATION
            n4.CityName = "MIAMI";
            n4.StateOrProvinceCode = "FL";
            n4.PostalCode = "33111";

            //REF*EI*587654321~
            REF reff = loop2010AA.CreateSegment<REF>();  //BILLING PROVIDER TAX IDENTIFICATION
            reff.ReferenceIdentificationQualifier = "EI";
            reff.ReferenceIdentification = "587654321";

            AddPayToProvider(loop2000A);
        }

        private void AddSubscriber(DocumentLoop transactionHeader)
        {
            DocumentLoop loop2000B = transactionHeader.CreateLoop("2000B SUBSCRIBER HIERARCHICAL LEVEL");

            //HL*2*1*22*1~
            HL hl2000C = loop2000B.CreateSegment<HL>();
            hl2000C.HierarchicalIDNumber = "2";
            hl2000C.HierarchicalParentIDNumber = "1";
            hl2000C.HierarchicalLevelCode = "22";
            hl2000C.HierarchicalChildCode = "1";

            //SBR*P**2222-SJ******CI~
            SBR sbr = loop2000B.CreateSegment<SBR>();   //SUBSCRIBER INFORMATION
            sbr.PayerResponsibilitySequenceNumberCode = "P";
            sbr.ReferenceIdentification = "2222-SJ";
            sbr.ClaimFilingIndicatorCode = "CI";

            AddSubscriberName(loop2000B);

            AddPayerName(loop2000B);

            AddPatient(loop2000B);
        }

        private void AddPatient(DocumentLoop loop2000B)
        {
            DocumentLoop loop2000C = loop2000B.CreateLoop("2000C PATIENT HIERARCHICAL LEVEL");

            //HL*3*2*23*0~
            HL hl2000C = loop2000C.CreateSegment<HL>();
            hl2000C.HierarchicalIDNumber = "3";
            hl2000C.HierarchicalParentIDNumber = "2";
            hl2000C.HierarchicalLevelCode = "23";
            hl2000C.HierarchicalChildCode = "0";

            //PAT*19~
            PAT pat = loop2000C.CreateSegment<PAT>();   //PAT PATIENT INFORMATION
            pat.IndividualRelationshipCode = "19";

            DocumentLoop loop2010CA = loop2000C.CreateLoop("2010CA PATIENT NAME");

            //NM1*QC*1*SMITH*TED~
            NM1 nm1 = loop2010CA.CreateSegment<NM1>(); //PATIENT NAME
            nm1.EntityIdentifierCode = "QC";
            nm1.EntityTypeQualifier = "1";
            nm1.NameLastOrOrganizationName = "SMITH";
            nm1.FirstName = "TED";

            //N3*236 N MAIN ST~
            N3 n3 = loop2010CA.CreateSegment<N3>();  //PATIENT ADDRESS
            n3.AddressInformation1 = "236 N MAIN ST";

            //N4*MIAMI*FL*33413~
            N4 n4 = loop2010CA.CreateSegment<N4>();  //PATIENT CITY/STATE/ZIP
            n4.CityName = "MIAMI";
            n4.StateOrProvinceCode = "FL";
            n4.PostalCode = "33413";

            //DMG*D8*19730501*M~
            DMG dmg = loop2010CA.CreateSegment<DMG>(); //PATIENT DEMOGRAPHIC INFORMATION
            dmg.DateTimePeriodFormatQualifier = "D8";
            dmg.DateTimePeriod = "19730501";
            dmg.GenderCode = "M";

            AddClaim(loop2000C);
        }

        private static void AddClaim(DocumentLoop loop2000C)
        {
            DocumentLoop loop2300 = loop2000C.CreateLoop("2300 CLAIM INFORMATION");

            //CLM*26463774*100***11:B:1*Y*A*Y*I~
            CLM clm = loop2300.CreateSegment<CLM>();   // CLM CLAIM LEVEL INFORMATION
            clm.ClaimSubmitterIdentifier = "26463774";
            clm.MonetaryAmount = "100";

            clm.HealthCareServiceLocation.FacilityCodeValue = "11";  //Composite data
            clm.HealthCareServiceLocation.FacilityCodeQualifier = "B";
            clm.HealthCareServiceLocation.ClaimFrequencyTypeCode = "1";

            clm.YesNoConditionOrResponseCode1 = "Y";
            clm.AssignmentOrPlanParticipationCode = "A";
            clm.YesNoConditionOrResponseCode2 = "Y";
            clm.ReleaseOfInformationCode = "I";

            //REF*D9*17312345600006351~
            REF reff = loop2300.CreateSegment<REF>(); // CLAIM IDENTIFICATION NUMBER FOR CLEARING HOUSES (Added by C.H.)
            reff.ReferenceIdentificationQualifier = "D9";
            reff.ReferenceIdentification = "17312345600006351";

            //HI*BK:0340*BF:V7389~
            HI hi = loop2300.CreateSegment<HI>();  //HEALTH CARE DIAGNOSIS CODES
            hi.HealthCareCodeInformation1.CodeListQualifierCode = "BK";
            hi.HealthCareCodeInformation1.IndustryCode1 = "0340";

            hi.HealthCareCodeInformation2.CodeListQualifierCode = "BF";
            hi.HealthCareCodeInformation2.IndustryCode1 = "V7389";

            AddServiceLine(loop2300);
        }

        private static void AddServiceLine(DocumentLoop loop2300)
        {
            for (int i = 1; i < 3; i++)
            {
                DocumentLoop loop2400 = loop2300.CreateLoop("2400 SERVICE LINE NUMBER");

                //LX*1~
                LX lx1 = loop2400.CreateSegment<LX>();  //SERVICE LINE COUNTER
                lx1.AssignedNumber = i.ToString();

                //SV1*HC:99213*40*UN*1***1~
                SV1 sv1 = loop2400.CreateSegment<SV1>();  // SV1 PROFESSIONAL SERVICE
                sv1.CompositeMedicalProcedureIdentifier.ProductServiceIDQualifier = "HC";
                sv1.CompositeMedicalProcedureIdentifier.ProductServiceID1 = "99213";

                sv1.MonetaryAmount1 = "40";
                sv1.UnitOrBasisForMeasurementCode = "UN";
                sv1.Quantity = "1";
                sv1.CompositeDiagnosticCodePointer.DiagnosisCodePointer1 = "1";

                //DTP*472*D8*20061003~
                DTP serviceDate = loop2400.CreateSegment<DTP>();  //DTP DATE - SERVICE DATE(S)
                serviceDate.DateTimeQualifier = "472";
                serviceDate.DateTimePeriodFormatQualifier = "D8";
                serviceDate.DateTimePeriod = "20061003";
            }
        }

        private static void AddSubscriberName(DocumentLoop loop2000B)
        {
            DocumentLoop loop2010BA = loop2000B.CreateLoop("2010BA SUBSCRIBER NAME");

            //NM1*IL*1*SMITH*JANE****MI*JS00111223333~
            NM1 nm1 = loop2010BA.CreateSegment<NM1>(); //SUBSCRIBER NAME
            nm1.EntityIdentifierCode = "IL";
            nm1.EntityTypeQualifier = "1";
            nm1.NameLastOrOrganizationName = "Smith";
            nm1.FirstName = "Jane";
            nm1.IdentificationCodeQualifier = "MI";
            nm1.IdentificationCode = "JS00111223333";

            //DMG*D8*19430501*F~
            DMG dmg = loop2010BA.CreateSegment<DMG>(); //SUBSCRIBER DEMOGRAPHIC INFORMATION
            dmg.DateTimePeriodFormatQualifier = "D8";
            dmg.DateTimePeriod = "19430501";
            dmg.GenderCode = "F";
        }

        private static void AddPayerName(DocumentLoop loop2000B)
        {
            DocumentLoop loop2010BB = loop2000B.CreateLoop("2010BB PAYER NAME");

            //NM1*PR*2*KEY INSURANCE COMPANY*****PI*999996666~
            NM1 nm1 = loop2010BB.CreateSegment<NM1>(); //PAYER NAME
            nm1.EntityIdentifierCode = "PR";
            nm1.EntityTypeQualifier = "2";
            nm1.NameLastOrOrganizationName = "KEY INSURANCE COMPANY";
            nm1.IdentificationCodeQualifier = "PI";
            nm1.IdentificationCode = "999996666";

            // REF*G2*KA6663~
            REF reff = loop2010BB.CreateSegment<REF>();  //PAYER SECONDARY ID
            reff.ReferenceIdentificationQualifier = "G2";
            reff.ReferenceIdentification = "KA6663";
        }

        private void AddPayToProvider(DocumentLoop billingProviderHLLevel)
        {
            DocumentLoop loop2010AB = billingProviderHLLevel.CreateLoop("2010AB PAY-TO ADDRESS NAME");

            //NM1*87*2~
            NM1 nm1 = loop2010AB.CreateSegment<NM1>(); //PAY-TO PROVIDER NAME
            nm1.EntityIdentifierCode = "87";
            nm1.EntityTypeQualifier = "2";

            //N3*2345 OCEAN BLVD~
            N3 n3 = loop2010AB.CreateSegment<N3>();  // PAY-TO PROVIDER ADDRESS
            n3.AddressInformation1 = "2345 OCEAN BLVD";

            //N4*MIAMI*FL*33111~
            N4 n4 = loop2010AB.CreateSegment<N4>();  // PAY-TO PROVIDER CITY
            n4.CityName = "MIAMI";
            n4.StateOrProvinceCode = "FL";
            n4.PostalCode = "33111";
        }

        private void AddSubmitter(DocumentLoop transactionHeader)
        {
            DocumentLoop submitter = transactionHeader.CreateLoop("1000A SUBMITTER NAME");

            //NM1*41*2*PREMIER BILLING SERVICE*****46*TGJ23~
            NM1 nm1 = submitter.CreateSegment<NM1>();
            nm1.EntityIdentifierCode = "41";
            nm1.EntityTypeQualifier = "2";
            nm1.NameLastOrOrganizationName = "PREMIER BILLING SERVICE";
            nm1.IdentificationCodeQualifier = "46";
            nm1.IdentificationCode = "TGJ23";

            //PER*IC*JERRY*TE*3055552222*EX*231~
            PER per = submitter.CreateSegment<PER>();
            per.ContactFunctionCode = "IC";
            per.ContactName = "JERRY";
            per.CommunicationNumberQualifier1 = "TE";
            per.CommunicationNumber1 = "3055552222";
            per.CommunicationNumberQualifier2 = "EX";
            per.CommunicationNumber3 = "231";
        }

        private void AddReceiver(DocumentLoop transactionHeader)
        {
            DocumentLoop receiver = transactionHeader.CreateLoop("1000B RECEIVER NAME");

            //NM1*40*2*KEY INSURANCE COMPANY*****46*66783JJT~
            NM1 nm1 = receiver.CreateSegment<NM1>();
            nm1.EntityIdentifierCode = "40";
            nm1.EntityTypeQualifier = "2";
            nm1.NameLastOrOrganizationName = "KEY INSURANCE COMPANY";
            nm1.IdentificationCodeQualifier = "46";
            nm1.IdentificationCode = "66783JJT";
        }

        private void lnk_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://www.rdpcrystal.com/products/edi-library/typed-document/");
        }
    }
}
