﻿using System;
using System.Windows.Forms;
using RDPCrystalEDILibrary;
using RDPCrystalEDILibrary.Docs.x5010;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrate for to create a 5010 837 Professional EDI document using Typed5010Document.
    /// Show the mapping from the HCFA-1500 Forms text fields to the 837 elements in the comments.
    /// Display the new document in the EDIDocumentViewer component.
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Create_5010_837P_From_HCFA_Fields : UserControl
    {
        private EDIFileLoader ediFileLoader = new EDIFileLoader();

        public Create_5010_837P_From_HCFA_Fields()
        {
            InitializeComponent();
        }

        private void btnCreateEDIFile_Click(object sender, EventArgs e)
        {
            try
            {
                //Create a Typed 5010 Document
                Typed5010Document doc837P = new Typed5010Document();

                //Places the correct number of segments in X12 SE segment
                doc837P.AutoPlaceCorrectNumOfSegments = true;
                doc837P.AutoPlaceNumOfTransactions = true;
                doc837P.AutoPlaceNumOfFunctionalGroups = true;

                //Make sure the ISA element values are the correct length
                doc837P.PadISAElementValues = true;

                //Create the Interchange header loop
                DocumentLoop interchangeHeader = doc837P.MainSection.CreateLoop("Interchange Header");

                ISA isa = interchangeHeader.CreateSegment<ISA>();
                isa.AuthorizationInformationQualifier = "00";
                isa.AuthorizationInformation = "";
                isa.SecurityInformationQualifier = "00";
                isa.SecurityInformation = "";
                isa.InterchangeIDQualifier1 = "ZZ";
                isa.InterchangeSenderID = "SenderID";
                isa.InterchangeIDQualifier2 = "ZZ";
                isa.InterchangeReceiverID = "ReceiverID";
                isa.InterchangeDate = DateTime.Today.ToString("yyMMdd");
                isa.InterchangeTime = DateTime.Now.ToString("hhmm");
                isa.RepetitionSeparator = "^";
                isa.InterchangeControlVersionNumber = "00501";
                isa.InterchangeControlNumber = "000000001";
                isa.AcknowledgmentRequested = "1";
                isa.InterchangeUsageIndicator = "T";
                isa.ComponentElementSeparator = ":";

                //Create the function group loop
                DocumentLoop functionalHeader = interchangeHeader.CreateLoop("Functional Header");

                GS gs = functionalHeader.CreateSegment<GS>();
                gs.FunctionalIdentifierCode = "HC";
                gs.ApplicationSenderCode = "ApplicationSenderCode";
                gs.ApplicationReceiverCode = "ApplicationReceiverCode";
                gs.Date = DateTime.Today.ToString("yyyyMMdd");
                gs.Time = DateTime.Now.ToString("hhmm");
                gs.GroupControlNumber = "1234";
                gs.ResponsibleAgencyCode = "X";
                gs.VersionReleaseIndustryIdentifierCode = "005010X222";

                //Create a transaction header loop
                DocumentLoop transactionHeader = functionalHeader.CreateLoop("Transaction Header");

                //ST*837*0021*005010X222~
                ST st = transactionHeader.CreateSegment<ST>();
                st.TransactionSetIdentifierCode = "837";
                st.TransactionSetControlNumber = "0021";
                st.ImplementationConventionReference = "005010X222";

                //BHT*0019*00*244579*20061015*1023*CH~
                BHT bht = transactionHeader.CreateSegment<BHT>();
                bht.HierarchicalStructureCode = "0019";
                bht.TransactionSetPurposeCode = "00";
                bht.ReferenceIdentification = "244579";
                bht.Date = DateTime.Today.ToString("yyyyMMdd");
                bht.Time = DateTime.Now.ToString("hhmm");
                bht.TransactionTypeCode = "CH";

                AddSubmitter(transactionHeader);
                AddReceiver(transactionHeader);
                AddBillingProvider(transactionHeader);
                AddSubscriber(transactionHeader);

                //Create the end of the transaction (SE segment). This must be done in a loop.
                DocumentLoop endOfTransactionLoop = transactionHeader.CreateLoop("End Of Transaction");

                //SE*42*37~
                SE se = endOfTransactionLoop.CreateSegment<SE>();
                se.TransactionSetControlNumber = "0021"; //same value as ST02

                //Create the end of the functional group (GE segment). This must be done in a loop.
                DocumentLoop endOfFunctionalGroupLoop = functionalHeader.CreateLoop("End Functional Group");
                GE ge = endOfFunctionalGroupLoop.CreateSegment<GE>();
                ge.GroupControlNumber = "1234"; //same value as GS06

                //Finally create the end of the interchange loop (IEA segment). This must be done in a loop.
                DocumentLoop endInterchangeSection = interchangeHeader.CreateLoop("End Interchange");
                IEA iea = endInterchangeSection.CreateSegment<IEA>();
                iea.InterchangeControlNumber = "000000001"; //same value is ISA13

                txtFile.Text = doc837P.ToEDIString(new Delimiters());

                //Display the new segments
                ediFileLoader.EDIDataString = txtFile.Text;
                ediDocumentViewer1.LoadDocument(ediFileLoader.Load());
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "RDPCrystal EDI Library");
            }
        }

        private void AddBillingProvider(DocumentLoop transactionHeader)
        {
            DocumentLoop loop2000A = transactionHeader.CreateLoop("2000A BILLING PROVIDER HIERARCHICAL LEVEL");

            //HL*1**20*1~
            HL hl2000C = loop2000A.CreateSegment<HL>();
            hl2000C.HierarchicalIDNumber = "1";
            hl2000C.HierarchicalLevelCode = "20";
            hl2000C.HierarchicalChildCode = "1";

            //HCFA FIELD 33b
            //PRV*BI*PXC*203BF0100Y~
            PRV prv = loop2000A.CreateSegment<PRV>();  //BILLING PROVIDER SPECIALTY INFORMATION
            prv.ProviderCode = "BI";
            prv.ReferenceIdentification = "PXC";
            prv.ReferenceIdentification = "203BF0100Y"; //HCFA FIELD 33b

            DocumentLoop loop2010AA = loop2000A.CreateLoop("2010AA BILLING PROVIDER NAME");

            //HCFA FIELD 33 - Billing Provider Info
            //NM1*85*2*BEN KILDARE SERVICE*****XX*9876543210~
            NM1 nm1 = loop2010AA.CreateSegment<NM1>(); //BILLING PROVIDER NAME
            nm1.EntityIdentifierCode = "85";
            nm1.EntityTypeQualifier = "2";
            nm1.NameLastOrOrganizationName = "BEN KILDARE SERVICE";
            nm1.IdentificationCodeQualifier = "XX";
            nm1.IdentificationCode = "9876543210"; //HCFA FIELD 33a

            //N3*234 SEAWAY ST~
            N3 n3 = loop2010AA.CreateSegment<N3>();  //BILLING PROVIDER ADDRESS
            n3.AddressInformation1 = "234 SEAWAY ST";

            //N4*MIAMI*FL*33111~
            N4 n4 = loop2010AA.CreateSegment<N4>();  //BILLING PROVIDER LOCATION
            n4.CityName = "MIAMI";
            n4.StateOrProvinceCode = "FL";
            n4.PostalCode = "33111";

            REF ref1 = loop2010AA.CreateSegment<REF>();
            ref1.ReferenceIdentificationQualifier = "SS"; //or "EIN" 
            ref1.ReferenceIdentification = "12345"; //HCFA FIELD 25

            //Billing Provider Phone
            PER phone = loop2010AA.CreateSegment<PER>();
            phone.ContactFunctionCode = "IC";
            phone.CommunicationNumberQualifier1 = "TE"; //telephone
            phone.CommunicationNumber1 = "2123222222";

            AddPayToProvider(loop2000A);
        }

        private static void AddPayerName(DocumentLoop loop2000B)
        {
            //CARRIER BLOCK ON THE VERY TOP OF THE HCFA-1500 FORM
            DocumentLoop loop2010BB = loop2000B.CreateLoop("2010BB PAYER NAME");

            //NM1*PR*2*KEY INSURANCE COMPANY*****PI*999996666~
            NM1 nm1 = loop2010BB.CreateSegment<NM1>(); //Carrier Name
            nm1.EntityIdentifierCode = "PR";
            nm1.EntityTypeQualifier = "2";
            nm1.NameLastOrOrganizationName = "KEY INSURANCE COMPANY";
            nm1.IdentificationCodeQualifier = "PI";
            nm1.IdentificationCode = "Payer Id";

            N3 n3 = loop2010BB.CreateSegment<N3>(); //Carrier Address
            n3.AddressInformation1 = "123 Main Street";
            N4 n4 = loop2010BB.CreateSegment<N4>();
            n4.CityName = "New York";
            n4.StateOrProvinceCode = "NY";
            n4.PostalCode = "11223";
        }

        private void AddSubscriber(DocumentLoop transactionHeader)
        {
            DocumentLoop loop2000B = transactionHeader.CreateLoop("2000B SUBSCRIBER HIERARCHICAL LEVEL");

            //HL*2*1*22*1~
            HL hl2000C = loop2000B.CreateSegment<HL>();
            hl2000C.HierarchicalIDNumber = "2";
            hl2000C.HierarchicalParentIDNumber = "1";
            hl2000C.HierarchicalLevelCode = "22";
            hl2000C.HierarchicalChildCode = "1";

            //SBR*P**2222-SJ******CI~
            SBR sbr = loop2000B.CreateSegment<SBR>();   //SUBSCRIBER INFORMATION
            sbr.PayerResponsibilitySequenceNumberCode = "P";
            sbr.ReferenceIdentification = "Policy Id";  //HCFA FIELD 11
            sbr.Name = "Program Name"; //HCFA FIELD 11c
            sbr.ClaimFilingIndicatorCode = "CI";  //HCFA FIELD 1, 'CI' means commercial insurance
            sbr.IndividualRelationshipCode = "18"; //HCFA FIELD 6 '18'  means the patient is the subscriber

            AddSubscriberName(loop2000B);

            AddPayerName(loop2000B);

            AddClaim(loop2000B);
        }

        private static void AddClaim(DocumentLoop loop2000C)
        {
            DocumentLoop loop2300 = loop2000C.CreateLoop("2300 CLAIM INFORMATION");

            //CLM*26463774*100***11:B:1*Y*A*Y*I~
            CLM clm = loop2300.CreateSegment<CLM>();
            clm.ClaimSubmitterIdentifier = "Patient Account Number"; //HCFA FIELD 26 Patient Account Number
            clm.MonetaryAmount = "100"; //HCFA FIELD 28 - Total Charge

            clm.HealthCareServiceLocation.FacilityCodeValue = "11";  //Composite data
            clm.HealthCareServiceLocation.FacilityCodeQualifier = "B";
            clm.HealthCareServiceLocation.ClaimFrequencyTypeCode = "1";

            clm.YesNoConditionOrResponseCode1 = "Y"; //HCFA FIELD 31 - Provider or Supplier Signature Indicator
            clm.AssignmentOrPlanParticipationCode = "A";
            clm.YesNoConditionOrResponseCode2 = "Y"; //HCFA FIELD 13
            clm.ReleaseOfInformationCode = "Y";  //HCFA FIELD 12
            clm.ProviderAgreementCode = "A"; //HCFA FIELD 27 Accept Assignment

            clm.RelatedCausesInformation.RelatedCausesCode1 = "OA"; //HCFA FIELD 10a
            clm.RelatedCausesInformation.RelatedCausesCode2 = "OA"; //HCFA FIELD 10b
            clm.RelatedCausesInformation.StateOrProvinceCode = "NY"; //HCFA FIELD 10b - state
            clm.RelatedCausesInformation.RelatedCausesCode3 = ""; //HCFA FIELD 10c

            clm.HealthCareServiceLocation.FacilityCodeValue = "12"; //HCFA FIELD 24b
            clm.HealthCareServiceLocation.FacilityCodeQualifier = "B";
            clm.HealthCareServiceLocation.ClaimFrequencyTypeCode = "C"; //HCFA FIELD 22a Resubmission Code

            HI healthcareCode = loop2300.CreateSegment<HI>();
            healthcareCode.HealthCareCodeInformation1.CodeListQualifierCode = "ABK"; //HCFA FIELD 10d, ICD-10 codes

            //HCFA FIELD 15 ONSET OF CURRENT ILLNESS OR SYMPTOM
            DTP date = loop2300.CreateSegment<DTP>();
            date.DateTimeQualifier = "431";
            date.DateTimePeriodFormatQualifier = "D8";
            date.DateTimePeriod = "20050108"; //HCFA FIELD 14

            //HCFA FIELD 14 could be any other date, for example, date last seen (below)
            DTP date2 = loop2300.CreateSegment<DTP>();
            date2.DateTimeQualifier = "304";
            date2.DateTimePeriodFormatQualifier = "D8";
            date2.DateTimePeriod = "20050108"; //HCFA FIELD 15

            //HCFA FIELD 16 - Disability From Date
            DTP date3 = loop2300.CreateSegment<DTP>();
            date3.DateTimeQualifier = "314";
            date3.DateTimePeriodFormatQualifier = "D8";
            date3.DateTimePeriod = "20050108"; //HCFA FIELD 16 - From

            //HCFA FIELD 16 - Work Return Date
            DTP date4 = loop2300.CreateSegment<DTP>();
            date4.DateTimeQualifier = "296";
            date4.DateTimePeriodFormatQualifier = "D8";
            date4.DateTimePeriod = "20050108"; //HCFA FIELD 16 - To

            //HCFA FIELD 18 - Admission Date
            DTP date5 = loop2300.CreateSegment<DTP>();
            date5.DateTimeQualifier = "435";
            date5.DateTimePeriodFormatQualifier = "D8";
            date5.DateTimePeriod = "20050108"; //HCFA FIELD 18 - From

            //HCFA FIELD 18 - Discharge Date
            DTP date6 = loop2300.CreateSegment<DTP>();
            date6.DateTimeQualifier = "096";
            date6.DateTimePeriodFormatQualifier = "D8";
            date6.DateTimePeriod = "20050108"; //HCFA FIELD 18 - To

            //HCFA FIELD 29- Amount Paid
            AMT amt = loop2300.CreateSegment<AMT>();
            amt.AmountQualifierCode = "F5";
            amt.MonetaryAmount = "50"; //HCFA FIELD 29 - Amount Paid

            //HCFA FIELD 23 - Prior Authorization Number
            REF reff = loop2300.CreateSegment<REF>();
            reff.ReferenceIdentificationQualifier = "G1";
            reff.ReferenceIdentification = "Prior Auth Number"; //HCFA FIELD 23

            //HCFA FIELD 19 - Clain Note
            NTE note = loop2300.CreateSegment<NTE>();
            note.NoteReferenceCode = "ADD";
            note.Description = "test"; //HCFA FIELD 19

            //HCFA FIELD 21 HEALTH CARE DIAGNOSIS CODES
            HI hi = loop2300.CreateSegment<HI>();  
            hi.HealthCareCodeInformation1.CodeListQualifierCode = "ABF";
            hi.HealthCareCodeInformation1.IndustryCode1 = "0340"; //HCFA FIELD 21a

            hi.HealthCareCodeInformation2.CodeListQualifierCode = "ABF";
            hi.HealthCareCodeInformation2.IndustryCode1 = "V7389"; //HCFA FIELD 21b

            AddReferringProviderName(loop2300);

            AddServiceLine(loop2300);
        }

        private static void AddReferringProviderName(DocumentLoop loop)
        {
            DocumentLoop loop2310A = loop.CreateLoop("2310A Referring Provider");

            NM1 nm1 = loop2310A.CreateSegment<NM1>(); //Referring Provider Name
            nm1.EntityIdentifierCode = "DN";
            nm1.EntityTypeQualifier = "1";
            nm1.NameLastOrOrganizationName = "Provider";  //HCFA FIELD 17
            nm1.FirstName = "Referring";  //HCFA FIELD 17
            nm1.IdentificationCodeQualifier = "XX";
            nm1.IdentificationCode = "1111"; //HCFA FIELD 17b

            REF secondId = loop2310A.CreateSegment<REF>();
            secondId.ReferenceIdentificationQualifier = "0B";
            secondId.ReferenceIdentification = "Provider ID"; //HCFA FIELD 17a
        }

        private static void AddServiceLine(DocumentLoop loop2300)
        {
            for (int i = 1; i < 3; i++)
            {
                DocumentLoop loop2400 = loop2300.CreateLoop("2400 SERVICE LINE NUMBER");

                //SERVICE LINE COUNTER
                LX lx1 = loop2400.CreateSegment<LX>();  
                lx1.AssignedNumber = i.ToString();

                //SV1*HC:99213*40*UN*1***1~
                SV1 sv1 = loop2400.CreateSegment<SV1>();  // SV1 PROFESSIONAL SERVICE

                //HCFA FIELD 24d
                sv1.CompositeMedicalProcedureIdentifier.ProductServiceIDQualifier = "HC";
                sv1.CompositeMedicalProcedureIdentifier.ProductServiceID1 = "99213"; //HCFA FIELD 24d Procedures

                sv1.MonetaryAmount1 = "40";  //HCFA FIELD 24f
                sv1.UnitOrBasisForMeasurementCode = "UN";
                sv1.Quantity = "1"; //HCFA FIELD 24g
                sv1.CompositeDiagnosticCodePointer.DiagnosisCodePointer1 = "1";  //HCFA FIELD 24e
                sv1.FacilityCodeValue = "11"; //HCFA FIELD 24b Place of Service Code
                sv1.YesNoConditionOrResponseCode1 = "Y"; //HCFA FIELD 24c Emergency Indicator
                sv1.YesNoConditionOrResponseCode2 = "Y"; //HCFA FIELD 24h EPSDT
                sv1.YesNoConditionOrResponseCode3 = "Y"; //HCFA FIELD 24h Family Plan

                //HCFA FIELD 24a - Service Date
                DTP serviceDate = loop2400.CreateSegment<DTP>(); 
                serviceDate.DateTimeQualifier = "472";
                serviceDate.DateTimePeriodFormatQualifier = "D8";
                serviceDate.DateTimePeriod = "20061003";  //HCFA FIELD 24a Service Date

                //HCFA FIELD 20 - Purchased Service Charge
                PS1 ps1 = loop2400.CreateSegment<PS1>();
                ps1.ReferenceIdentification = "test"; //This must be the identifier from the Purchased Service Provider Loop (Loop ID-2420B).
                ps1.MonetaryAmount = "100";

                AddRenderingProvider(loop2400);
            }
        }

        private static void AddRenderingProvider(DocumentLoop loop2400)
        {
            DocumentLoop loop2420A = loop2400.CreateLoop("2420A RENDERING PROVIDER");

            //NM1*IL*1*SMITH*JANE****MI*JS00111223333~
            NM1 nm1 = loop2420A.CreateSegment<NM1>(); //Rendering Provider Name
            nm1.EntityIdentifierCode = "82";
            nm1.EntityTypeQualifier = "1";
            nm1.NameLastOrOrganizationName = "Smith";  
            nm1.FirstName = "Jane"; 
            nm1.IdentificationCodeQualifier = "XX";
            nm1.IdentificationCode = "JS00111223333";  //HCFA FIELD 24J

            REF secondId = loop2420A.CreateSegment<REF>();
            secondId.ReferenceIdentificationQualifier = "1G";  //HCFA FIELD 24J Shaded Line
            secondId.ReferenceIdentification = "112122343";    //HCFA FIELD 24J
        }

        private static void AddSubscriberName(DocumentLoop loop2000B)
        {
            DocumentLoop loop2010BA = loop2000B.CreateLoop("2010BA SUBSCRIBER NAME");

            //NM1*IL*1*SMITH*JANE****MI*JS00111223333~
            NM1 nm1 = loop2010BA.CreateSegment<NM1>(); //Patient/Subscriber Name
            nm1.EntityIdentifierCode = "IL";
            nm1.EntityTypeQualifier = "1";
            nm1.NameLastOrOrganizationName = "Smith";  //HCFA FIELD 2, HCFA FIELD 4, We are assuming the patient and the subscriber is the same person
            nm1.FirstName = "Jane";  //HCFA FIELD 2
            nm1.IdentificationCodeQualifier = "MI";
            nm1.IdentificationCode = "Subscriber Id"; //HCFA FIELD 1a

            //N3*236 N MAIN ST~
            N3 n3 = loop2010BA.CreateSegment<N3>();  //PATIENT ADDRESS
            n3.AddressInformation1 = "236 N MAIN ST"; //HCFA FIELD 5, HCFA FIELD 7 - Street

            //N4*MIAMI*FL*33413~
            N4 n4 = loop2010BA.CreateSegment<N4>();  //HCFA FIELD 5, HCFA FIELD 7 - CITY/STATE/ZIP
            n4.CityName = "MIAMI";   
            n4.StateOrProvinceCode = "FL";
            n4.PostalCode = "33413";

            //DMG*D8*19730501*M~
            DMG dmg = loop2010BA.CreateSegment<DMG>(); //PATIENT DEMOGRAPHIC INFORMATION
            dmg.DateTimePeriodFormatQualifier = "D8";
            dmg.DateTimePeriod = "19730501"; //HCFA FIELD 3, HCFA FIELD 11a - Patient/Subscriber Birthdate
            dmg.GenderCode = "M"; //HCFA FIELD 3, HCFA FIELD 11a - Patient/Subscriber Sex

            REF secondId = loop2010BA.CreateSegment<REF>(); //PATIENT DEMOGRAPHIC INFORMATION
            secondId.ReferenceIdentificationQualifier = "SY";
            secondId.ReferenceIdentification = "112-12-2343"; //HCFA FIELD 11b
        }

        private void AddPayToProvider(DocumentLoop billingProviderHLLevel)
        {
            DocumentLoop loop2010AB = billingProviderHLLevel.CreateLoop("2010AB PAY-TO ADDRESS NAME");

            //NM1*87*2~
            NM1 nm1 = loop2010AB.CreateSegment<NM1>(); //PAY-TO PROVIDER NAME
            nm1.EntityIdentifierCode = "87";
            nm1.EntityTypeQualifier = "2";

            //N3*2345 OCEAN BLVD~
            N3 n3 = loop2010AB.CreateSegment<N3>();  // PAY-TO PROVIDER ADDRESS
            n3.AddressInformation1 = "2345 OCEAN BLVD";

            //N4*MIAMI*FL*33111~
            N4 n4 = loop2010AB.CreateSegment<N4>();  // PAY-TO PROVIDER CITY
            n4.CityName = "MIAMI";
            n4.StateOrProvinceCode = "FL";
            n4.PostalCode = "33111";
        }

        private void AddSubmitter(DocumentLoop transactionHeader)
        {
            //This loop is the submitter of the EDI claim
            DocumentLoop submitter = transactionHeader.CreateLoop("1000A SUBMITTER NAME");

            //NM1*41*2*PREMIER BILLING SERVICE*****46*TGJ23~
            NM1 nm1 = submitter.CreateSegment<NM1>();
            nm1.EntityIdentifierCode = "41";
            nm1.EntityTypeQualifier = "2";
            nm1.NameLastOrOrganizationName = "PREMIER BILLING SERVICE";
            nm1.IdentificationCodeQualifier = "46";
            nm1.IdentificationCode = "Submitter Id";

            //PER*IC*JERRY*TE*3055552222~
            PER per = submitter.CreateSegment<PER>();
            per.ContactFunctionCode = "IC";
            per.ContactName = "Sam Pang";
            per.CommunicationNumberQualifier1 = "TE";
            per.CommunicationNumber1 = "3055552222";
        }

        private void AddReceiver(DocumentLoop transactionHeader)
        {
            //This loop is the company you are sending the claim to
            DocumentLoop receiver = transactionHeader.CreateLoop("1000B RECEIVER NAME");

            //NM1*40*2*KEY INSURANCE COMPANY*****46*66783JJT~
            NM1 nm1 = receiver.CreateSegment<NM1>();
            nm1.EntityIdentifierCode = "40";
            nm1.EntityTypeQualifier = "2";
            nm1.NameLastOrOrganizationName = "KEY INSURANCE COMPANY";
            nm1.IdentificationCodeQualifier = "46";
            nm1.IdentificationCode = "Receiver Id";
        }
    }
}
