using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;
using RDPCrystalEDILibrary;
using RDPCrystalEDILibrary.Docs.x4010;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to parse and load a 5010 835 EDI document using the EDIValidator component.
    /// EDI data will then be dislayed in the UI
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Load_4010_810_File : UserControl
    {
        private EDIValidator ediValidator = new EDIValidator();

        public Load_4010_810_File()
        {
            InitializeComponent();

            txtFile.Text = File.ReadAllText("EDIFiles\\Sample_4010_810.txt");
        }

        private void InitializeEDI()
        {
            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = "ValidationRules\\Rules_4010_810_004010.Rules";

            ediValidator.EDIFile = "EDIFiles\\Sample_4010_810.txt";
        }

        private void LoadEDI()
        {
            try
            {
                ediValidator.Validate();

                //The EDI Document will contain all the data of the 835 file
                if (ediValidator.EDILightWeightDocument != null)
                {
                    Typed4010Document doc = new Typed4010Document(ediValidator.EDILightWeightDocument);

                    DocumentLoop transaction = doc.MainSection.GetLoop("INTERCHANGE HEADER/FUNCTIONAL GROUP/ST HEADER");

                    //Get Payer
                    DocumentLoop payer = transaction.GetLoop("PAYER PARTY");
                    N1 payerName = payer.GetSegment<N1>("N1");
                    txtPayerName.Text = payerName.Name1;

                    N3 payerStreet = payer.GetSegment<N3>("N3");
                    txtPayerAdd1.Text = payerStreet.AddressInformation1;

                    N4 payerAddress = payer.GetSegment<N4>("N4");
                    txtPayerAdd2.Text = payerAddress.CityName + " " + payerAddress.StateOrProvinceCode + " " + payerAddress.PostalCode;

                    //Get Payee Party
                    DocumentLoop payee = transaction.GetLoop("PAYEE PARTY");
                    N1 payeeName = payee.GetSegment<N1>("N1");
                    txtPayeeName.Text = payeeName.Name1;

                    N3 payeeStreet = payee.GetSegment<N3>("N3");
                    txtPayeeAdd1.Text = payerStreet.AddressInformation1;

                    N4 payeeAddress = payee.GetSegment<N4>("N4");
                    txtPayeeAdd2.Text = payeeAddress.CityName + " " + payeeAddress.StateOrProvinceCode + " " + payeeAddress.PostalCode;

                    //Get Buying Party
                    N1 buyingName = transaction.GetSegment<N1>("BUYING PARTY/N1");
                    txtBuyingName.Text = buyingName.Name1;

                    N3 buyingStreet = transaction.GetSegment<N3>("BUYING PARTY/N3");
                    txtBuyingAdd1.Text = buyingStreet.AddressInformation1;

                    N4 buyingAddress = transaction.GetSegment<N4>("BUYING PARTY/N4");
                    txtBuyingAdd2.Text = buyingAddress.CityName + " " + buyingAddress.StateOrProvinceCode + " " + buyingAddress.PostalCode;

                    //Get the Selling Party
                    N1 sellingName = transaction.GetSegment<N1>("SELLING PARTY/N1");
                    txtSellingName.Text = sellingName.Name1;

                    N3 sellingStreet = transaction.GetSegment<N3>("SELLING PARTY/N3");
                    txtSellingAdd1.Text = sellingStreet.AddressInformation1;

                    N4 sellingAddress = transaction.GetSegment<N4>("SELLING PARTY/N4");
                    txtSellingAdd2.Text = sellingAddress.CityName + " " + sellingAddress.StateOrProvinceCode + " " + sellingAddress.PostalCode;

                    //Get the Ship From Party
                    N1 shipFromName = transaction.GetSegment<N1>("SHIP FROM PARTY/N1");
                    txtShipFromName.Text = shipFromName.Name1;

                    N3 shipFromStreet = transaction.GetSegment<N3>("SHIP FROM PARTY/N3");
                    txtShipFromAdd1.Text = shipFromStreet.AddressInformation1;

                    N4 shipFromAddress = transaction.GetSegment<N4>("SHIP FROM PARTY/N4");
                    txtShipFromAdd2.Text = shipFromAddress.CityName + " " + shipFromAddress.StateOrProvinceCode + " " + shipFromAddress.PostalCode;

                    //Get the Ship To Party
                    DocumentLoop shipTo = transaction.GetLoop("SHIP TO PARTY");
                    N1 shipToName = shipTo.GetSegment<N1>("N1");
                    txtShipToName.Text = shipToName.Name1;

                    N3 shipToStreet = shipTo.GetSegment<N3>("N3");
                    txtShipToAdd1.Text = shipToStreet.AddressInformation1;

                    N4 shipToAddress = shipTo.GetSegment<N4>("N4");
                    txtShipToAdd2.Text = shipToAddress.CityName + " " + shipToAddress.StateOrProvinceCode + " " + shipToAddress.PostalCode;

                    List<ItemData> items = new List<ItemData>();

                    foreach (DocumentLoop loop in transaction.GetLoops("LOOP IT1"))
                    {
                        IT1 item = loop.GetSegment<IT1>();

                        if (item != null)
                        {
                            ItemData itemData = new ItemData();
                            itemData.AssignedID = item.AssignedIdentification;
                            itemData.QuantityInvoiced = item.QuantityInvoiced;
                            itemData.Unit = item.UnitOrBasisForMeasurementCode;
                            itemData.UnitPrice = item.UnitPrice;
                            itemData.BuyerItemNumber = item.ProductServiceID1;
                            itemData.SellerItemNumber = item.ProductServiceID2;
                            itemData.ItemType = item.ProductServiceID3;
                            itemData.InternalNumber = item.ProductServiceID4;

                            foreach (QTY qty in loop.GetSegments<QTY>("QTY"))
                            {
                                if (qty.QuantityQualifier == "T5")
                                {
                                    itemData.TotalNumberOfUnits = qty.CompositeUnitOfMeasure.Multiplier;
                                }
                                else if (qty.QuantityQualifier == "39")
                                {
                                    itemData.ShippedQuantity = qty.CompositeUnitOfMeasure.Multiplier;
                                }
                            }

                            items.Add(itemData);
                        }
                    }

                    dataGrid.DataSource = items;

                    ediDocumentViewer.LoadDocument(ediValidator.EDILightWeightDocument);
                }
            }
            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        public class ItemData
        {
            public string AssignedID { get; set; }

            public string TotalNumberOfUnits { get; set; }

            public string ShippedQuantity { get; set; }

            public string QuantityInvoiced { get; set; }

            public string Unit { get; set; }

            public string UnitPrice { get; set; }

            public string BuyerItemNumber { get; set; }

            public string SellerItemNumber { get; set; }

            public string ItemType { get; set; }

            public string InternalNumber { get; set; }
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            InitializeEDI();

            LoadEDI();
        }
    }
}