using System;
using System.Collections.Generic;
using System.Data;
using System.Windows.Forms;
using RDPCrystalEDILibrary;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstratse how to parse and load a 4010 835 EDI document using the EDIValidator component.
    /// EDI data will then be dislayed in the UI
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Load_4010_835_File : UserControl
    {
        private DataSet mainDataSet = new DataSet("Claims Information");
        private DataTable claimPaymentInfoTable = new DataTable("Claim Payment Information");
        private DataTable servicePaymentInfoTable = new DataTable("Service Payment Information");
        private DataTable interchangeHeaderTable = new DataTable("Interchange Header");
        private DataTable functionalGroupTable = new DataTable("Functional Group");

        private EDIValidator ediValidator = new EDIValidator();

        public Load_4010_835_File()
        {
            InitializeComponent();

            ediValidator.ProgressChanged += Validation_ProgressChanged;

            InitializeTables();
        }

        private void InitializeTables()
        {
            interchangeHeaderTable.Columns.Add("Header Item", typeof(string));
            interchangeHeaderTable.Columns.Add("Header Value", typeof(string));

            functionalGroupTable.Columns.Add("Header Item", typeof(string));
            functionalGroupTable.Columns.Add("Header Value", typeof(string));

            DataColumn rowNum = claimPaymentInfoTable.Columns.Add("Claim #", typeof(int));
            claimPaymentInfoTable.Columns.Add("Patient Name");
            claimPaymentInfoTable.Columns.Add("Insured Name");

            claimPaymentInfoTable.PrimaryKey = new DataColumn[] { rowNum };

            DataColumn rowNum1 = servicePaymentInfoTable.Columns.Add("RowNum", typeof(int));
            rowNum1.ColumnMapping = MappingType.Hidden;
            servicePaymentInfoTable.Columns.Add("Product/Service ID");
            servicePaymentInfoTable.Columns.Add("Amount");
            servicePaymentInfoTable.Columns.Add("Quantity");

            mainDataSet.Tables.Add(claimPaymentInfoTable);
            mainDataSet.Tables.Add(servicePaymentInfoTable);

            mainDataSet.Relations.Add("Claim Payment Information", rowNum, rowNum1);

            dataGrid.DataSource = mainDataSet;
        }

        private void InitializeEDI()
        {
            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = "ValidationRules\\Rules_4010_835_004010X091A1.Rules";

            //Set the EDI data to load
            ediValidator.EDIDataString = SampleEDI835Data;
        }

        private void LoadEDI()
        {
            try
            {
                ediValidator.Validate();

                //The EDI Document will contain all the data of the 835 file
                if (ediValidator.EDILightWeightDocument != null)
                {
                    EDILightWeightDocument file835 = ediValidator.EDILightWeightDocument;

                    LightWeightLoop interchangeHeaderLoop = file835.Loops[0].GetLoop("INTERCHANGE HEADER");

                    DisplayInterchangeHeader(interchangeHeaderLoop.GetSegment("ISA"));

                    LightWeightLoop functionalHeaderLoop = interchangeHeaderLoop.GetLoop("FUNCTIONAL GROUP");

                    DisplayFunctionalHeader(functionalHeaderLoop.GetSegment("GS"));

                    //Get the ST loops.  The sample EDI file contains 2 ST-SE groups however we will only display one
                    List<LightWeightLoop> stHeaderLoops = functionalHeaderLoop.GetLoops("ST HEADER");

                    int numberOfST_SE_groups = 1;
                    foreach (LightWeightLoop stHeaderLoop in stHeaderLoops)
                    {
                        //In this demo we are only displaying data in the first ST-SE group
                        //However you can display as many ST-SE groups you desire

                        if (numberOfST_SE_groups == 1)
                        {
                            //Get the payer identification loop. In an 835 file the payer identification loop is 1000A
                            LightWeightLoop payerIdentificationLoop = stHeaderLoop.GetLoop("1000A");
                            DisplayPayerIdenfificationInformation(payerIdentificationLoop);

                            // Get the receiver loop. In an 837 file the receiver loop is 1000B
                            LightWeightLoop payeeIdentificationLoop = stHeaderLoop.GetLoop("1000B");
                            DisplayPayeeIdenfificationInformation(payeeIdentificationLoop);

                            // Get the 2000A BILLING/PAY-TO PROVIDER HIERARCHICAL LEVEL
                            //In an 837 file this loop is 2000A
                            LightWeightLoop headerNumberLoop = stHeaderLoop.GetLoop("2000");
                            List<LightWeightLoop> claimPaymentInformationLoops = headerNumberLoop.GetLoops("2100");

                            DisplayAllClaimPayments(claimPaymentInformationLoops);

                            numberOfST_SE_groups++;
                        }
                    }

                    ediDocumentViewer.LoadDocument(ediValidator.EDILightWeightDocument);
                }
            }
            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        private void DisplayInterchangeHeader(LightWeightSegment isa)
        {
            CreateInterchangeHeaderItem("Authorization Information Qualifier", isa.Elements[0].DataValue);
            CreateInterchangeHeaderItem("Authorization Information", isa.Elements[1].DataValue);
            CreateInterchangeHeaderItem("Security Information Qualifier", isa.Elements[2].DataValue);
            CreateInterchangeHeaderItem("Security Information", isa.Elements[3].DataValue);
            CreateInterchangeHeaderItem("Interchange ID Qualifier1", isa.Elements[4].DataValue);
            CreateInterchangeHeaderItem("Interchange Sender ID", isa.Elements[5].DataValue);
            CreateInterchangeHeaderItem("Interchange ID Qualifier2", isa.Elements[6].DataValue);
            CreateInterchangeHeaderItem("Interchange Receiver ID", isa.Elements[7].DataValue);
            CreateInterchangeHeaderItem("Interchange Date", isa.Elements[8].DataValue);
            CreateInterchangeHeaderItem("Interchange Time", isa.Elements[9].DataValue);
            CreateInterchangeHeaderItem("Repetition Separator", isa.Elements[10].DataValue);
            CreateInterchangeHeaderItem("Interchange Control Version Number", isa.Elements[11].DataValue);
            CreateInterchangeHeaderItem("Interchange Control Number", isa.Elements[12].DataValue);
            CreateInterchangeHeaderItem("Acknowledgment Requested", isa.Elements[13].DataValue);
            CreateInterchangeHeaderItem("Interchange Usage Indicator", isa.Elements[14].DataValue);
            CreateInterchangeHeaderItem("Component Element Separator", isa.Elements[15].DataValue);

            dgInterchangeHeader.DataSource = interchangeHeaderTable;
        }

        private void DisplayFunctionalHeader(LightWeightSegment gs)
        {
            CreateFunctionalHeaderItem("Functional Identifier Code", gs.Elements[0].DataValue);
            CreateFunctionalHeaderItem("Application Senders Code", gs.Elements[1].DataValue);
            CreateFunctionalHeaderItem("Application Receivers Code", gs.Elements[2].DataValue);
            CreateFunctionalHeaderItem("Date", gs.Elements[3].DataValue);
            CreateFunctionalHeaderItem("Time", gs.Elements[4].DataValue);
            CreateFunctionalHeaderItem("Group Control Number", gs.Elements[5].DataValue);
            CreateFunctionalHeaderItem("Responsible Agency Code", gs.Elements[6].DataValue);
            CreateFunctionalHeaderItem("Version / Release / Industry Identifier Code", gs.Elements[7].DataValue);

            dgFunctionalHeader.DataSource = functionalGroupTable;
        }

        private void CreateInterchangeHeaderItem(string headerItem, string headerValue)
        {
            DataRow row = interchangeHeaderTable.NewRow();
            row["Header Item"] = headerItem;
            row["Header Value"] = headerValue;

            interchangeHeaderTable.Rows.Add(row);
        }

        private void CreateFunctionalHeaderItem(string headerItem, string headerValue)
        {
            DataRow row = functionalGroupTable.NewRow();
            row["Header Item"] = headerItem;
            row["Header Value"] = headerValue;

            functionalGroupTable.Rows.Add(row);
        }

        private void DisplayAllClaimPayments(List<LightWeightLoop> claimPayments)
        {
            int rowNumber = 1;
            foreach (LightWeightLoop loop in claimPayments)
            {
                List<LightWeightSegment> parties = loop.GetSegments("NM1");

                string patientName = parties[0].Elements[2].DataValue + " " + parties[0].Elements[3].DataValue;
                string insuredName = parties[1].Elements[2].DataValue + " " + parties[1].Elements[3].DataValue;

                DataRow subInfoRow = claimPaymentInfoTable.NewRow();
                subInfoRow[0] = rowNumber;
                subInfoRow[1] = patientName;
                subInfoRow[2] = insuredName;

                claimPaymentInfoTable.Rows.Add(subInfoRow);

                List<LightWeightLoop> paymentInformationLoop = loop.GetLoops("2110");

                foreach (LightWeightLoop paymentInformation in paymentInformationLoop)
                {
                    LightWeightSegment svc = paymentInformation.GetSegment("SVC");

                    DataRow claimInfoRow = servicePaymentInfoTable.NewRow();
                    claimInfoRow[0] = rowNumber;
                    claimInfoRow[1] = svc.Elements[0].Elements[1].DataValue;
                    claimInfoRow[2] = svc.Elements[1].DataValue;
                    claimInfoRow[3] = svc.Elements[4].DataValue;

                    servicePaymentInfoTable.Rows.Add(claimInfoRow);
                }

                rowNumber++;
            }

            dataGrid.NavigateTo(0, "Claim Payment Information");
        }

        private void DisplayPayerIdenfificationInformation(LightWeightLoop loop)
        {
            //Get the name
            LightWeightSegment n1 = loop.GetSegment("N1");
            txtPayerName.Text = n1.Elements[1].DataValue;

            //Get the address 1
            LightWeightSegment n3 = loop.GetSegment("N3");
            if (n3 != null)
            {
                txtPayerAdd1.Text = n3.Elements[0].DataValue;
            }

            //Get the address 1
            LightWeightSegment n4 = loop.GetSegment("N4");
            if (n4 != null)
            {
                txtPayerAdd2.Text = n4.Elements[0].DataValue + " " + n4.Elements[0].DataValue + " " + n4.Elements[0].DataValue;
            }
        }

        private void DisplayPayeeIdenfificationInformation(LightWeightLoop loop)
        {
            //Get the name
            LightWeightSegment n1 = loop.GetSegment("N1");
            txtPayeeName.Text = n1.Elements[1].DataValue;

            //Get the address 1
            LightWeightSegment n3 = loop.GetSegment("N3");
            if (n3 != null)
            {
                txtPayeeAdd1.Text = n3.Elements[0].DataValue;
            }

            //Get the address 1
            LightWeightSegment n4 = loop.GetSegment("N4");
            if (n4 != null)
            {
                txtPayeeAdd2.Text = n4.Elements[0].DataValue + " " + n4.Elements[0].DataValue + " " + n4.Elements[0].DataValue;
            }
        }

        private void Validation_ProgressChanged(int progress)
        {
            BeginInvoke(new Action(() =>
            {
                if ((pbValidator.Value < pbValidator.Maximum))
                    pbValidator.Value = progress;
            }));
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            //Reset the progress bar & tables
            pbValidator.Value = 0;

            servicePaymentInfoTable.Rows.Clear();
            claimPaymentInfoTable.Rows.Clear();

            InitializeEDI();

            LoadEDI();
        }

        public const string SampleEDI835Data = @"ISA*00*          *00*          *ZZ*ASHTB          *ZZ*01017          *040315*1005*U*00401*004075123*0*P*:~" +
"GS*HP*ASHTB*01017*20040315*1005*1*X*004010X091A1~" +
"ST*835*07504123~" +
"BPR*H*5.75*C*NON************20040315~" +
"TRN*1*A04B001017.07504*1346000128~" +
"DTM*405*20040308~" +
"N1*PR*LAT COUNTY ADAMH BD~" +
"N3*4817 STATE ROAD SUITE 203~" +
"N4*LAT*OH*44004~" +
"N1*PE*LAKE AREA RECOVERY CENTER *FI*346608640~" +
"N3*2801 C. COURT~" +
"N4*PINSBERG*OH*44004~" +
"REF*PQ*1017~" +
"LX*1~" +
"CLP*444444*1*56.70*56.52*0*MC*0000000655555555*53~" +
"NM1*QC*1*FUDD*ELMER*S***MI*1333333~" +
"NM1*82*2*WECOVERWY SVCS*****FI*346608640~" +
"REF*F8*A76B04054~" +
"SVC*HC:H0005:HF:H9*56.70*56.52**6~" +
"DTM*472*20040205~" +
"CAS*CO*42*0.18*0~" +
"REF*6R*444444~" +
"CLP*999999*4*25.95*0*25.95*13*0000000555555555*11~" +
"NM1*QC*1*SAM*YOSEMITE*A***MI*3333333~" +
"NM1*82*2*ACME AGENCY*****FI*310626223~" +
"REF*F8*H57B10401~" +
"SVC*ZZ:M2200:HE*25.95*0**1~" +
"DTM*472*20021224~" +
"CAS*CR*18*25.95*0~" +
"CAS*CO*42*0*0~" +
"REF*6R*999999~" +
"CLP*888888*4*162.13*0*162.13*MC*0000000456789123*11~" +
"NM1*QC*1*SQUAREPANTS*BOB* ***MI*2222222~" +
"NM1*82*2*BIKINI AGENCY*****FI*310626223~" +
"REF*F8*H57B10401~" +
"SVC*ZZ:M151000:F0*162.13*0**1.9~" +
"DTM*472*20020920~" +
"CAS*CO*29*162.13*0*42*0*0~" +
"REF*6R*888888~" +
"CLP*111111*2*56.52*18.88*0*13*0000000644444444*53~" +
"NM1*QC*1*LEGHORN*FOGHORN*P***MI*7777777~" +
"NM1*82*2*CHICKENHAWK SVCS*****FI*346608640~" +
"REF*F8*A76B04054~" +
"SVC*HC:H0005:HF:H9*56.52*18.88**6~" +
"DTM*472*20031209~" +
"CAS*CO*42*0*0~" +
"CAS*OA*23*37.64*0~" +
"REF*6R*111111~" +
"CLP*121212*4*56.52*0*0*13*0000000646464640*53~" +
"NM1*QC*1*EXPLORER*DORA****MI*1717171~" +
"NM1*82*2*SWIPER AGENCY*****FI*346608640~" +
"REF*F8*A76B04054~" +
"SVC*HC:H0005:HF:H9*56.52*0**6~" +
"DTM*472*20031202~" +
"CAS*CO*42*0*0~" +
"CAS*OA*23*57.6*0*23*-1.08*0~" +
"REF*6R*121212~" +
"CLP*333333*1*74.61*59.69*14.92*13*0000000688888888*55~" +
"NM1*QC*1*BEAR*YOGI* ***MI*2222222~" +
"NM1*82*2*JELLYSTONE SVCS*****FI*346608640~" +
"REF*F8*A76B04054~" +
"SVC*ZZ:A0230:HF*74.61*59.69**1~" +
"DTM*472*20040203~" +
"CAS*PR*2*14.92*0~" +
"CAS*CO*42*0*0~" +
"REF*6R*333333~" +
"CLP*777777*25*136.9*0*0*13*0000000622222222*53~" +
"NM1*QC*1*BIRD*TWEETY*M***MI*4444444~" +
"NM1*82*2*GRANNY AGENCY*****FI*340716747~" +
"REF*F8*A76B03293~" +
"SVC*HC:H0015:HF:99:H9*136.9*0**1~" +
"DTM*472*20030911~" +
"CAS*PI*104*136.72*0~" +
"CAS*CO*42*0.18*0~" +
"REF*6R*777777~" +
"CLP*123456*22*-42.58*-42.58*0*13*0000000657575757*11~" +
"NM1*QC*1*SIMPSON*HOMER* ***MI*8787888~" +
"NM1*82*2*DOH GROUP*****FI*310626223~" +
"REF*F8*A57B04033~" +
"SVC*HC:H0036:GT:UK*-42.58*-42.58**-2~" +
"DTM*472*20040102~" +
"CAS*CR*141*0*0*42*0*0*22*0*0~" +
"CAS*OA*141*0*0~" +
"REF*6R*123456~" +
"CLP*090909*22*-86.76*-86.76*0*MC*0000000648484848*53~" +
"NM1*QC*1*DUCK*DAFFY*W***MI*1245849~" +
"NM1*82*2*ABTHSOLUTE HELP*****FI*346608640~" +
"REF*F8*A76B04054~" +
"SVC*HC:H0004:HF:H9*-86.76*-86.76**-4~" +
"DTM*472*20040210~" +
"CAS*CR*22*0*0*42*0*0~" +
"CAS*OA*22*0*0~" +
"REF*6R*090909~" +
"LQ*HE*MA92~" +
"SE*93*07504123~" +
"ST*835*07504123~" +
"BPR*H*5.75*C*NON************20040315~" +
"TRN*1*A04B001017.07504*1346000128~" +
"DTM*405*20040308~" +
"N1*PR*ASHTABULA COUNTY ADAMH BD~" +
"N3*4817 STATE ROAD SUITE 203~" +
"N4*ASHTABULA*OH*44004~" +
"N1*PE*LAKE AREA RECOVERY CENTER *FI*346608640~" +
"N3*2801 C. COURT~" +
"N4*ASHTABULA*OH*44004~" +
"REF*PQ*1017~" +
"LX*1~" +
"CLP*444444*1*56.70*56.52*0*MC*0000000655555555*53~" +
"NM1*QC*1*FUDD*ELMER*S***MI*1333333~" +
"NM1*82*2*WECOVERWY SVCS*****FI*346608640~" +
"REF*F8*A76B04054~" +
"SVC*HC:H0005:HF:H9*56.70*56.52**6~" +
"DTM*472*20040205~" +
"CAS*CO*42*0.18*0~" +
"REF*6R*444444~" +
"CLP*999999*4*25.95*0*25.95*13*0000000555555555*11~" +
"NM1*QC*1*SAM*YOSEMITE*A***MI*3333333~" +
"NM1*82*2*ACME AGENCY*****FI*310626223~" +
"REF*F8*H57B10401~" +
"SVC*ZZ:M2200:HE*25.95*0**1~" +
"DTM*472*20021224~" +
"CAS*CR*18*25.95*0~" +
"CAS*CO*42*0*0~" +
"REF*6R*999999~" +
"CLP*888888*4*162.13*0*162.13*MC*0000000456789123*11~" +
"NM1*QC*1*SQUAREPANTS*BOB* ***MI*2222222~" +
"NM1*82*2*BIKINI AGENCY*****FI*310626223~" +
"REF*F8*H57B10401~" +
"SVC*ZZ:M151000:F0*162.13*0**1.9~" +
"DTM*472*20020920~" +
"CAS*CO*29*162.13*0*42*0*0~" +
"REF*6R*888888~" +
"CLP*111111*2*56.52*18.88*0*13*0000000644444444*53~" +
"NM1*QC*1*LEGHORN*FOGHORN*P***MI*7777777~" +
"NM1*82*2*CHICKENHAWK SVCS*****FI*346608640~" +
"REF*F8*A76B04054~" +
"SVC*HC:H0005:HF:H9*56.52*18.88**6~" +
"DTM*472*20031209~" +
"CAS*CO*42*0*0~" +
"CAS*OA*23*37.64*0~" +
"REF*6R*111111~" +
"CLP*121212*4*56.52*0*0*13*0000000646464640*53~" +
"NM1*QC*1*EXPLORER*DORA****MI*1717171~" +
"NM1*82*2*SWIPER AGENCY*****FI*346608640~" +
"REF*F8*A76B04054~" +
"SVC*HC:H0005:HF:H9*56.52*0**6~" +
"DTM*472*20031202~" +
"CAS*CO*42*0*0~" +
"CAS*OA*23*57.6*0*23*-1.08*0~" +
"REF*6R*121212~" +
"CLP*333333*1*74.61*59.69*14.92*13*0000000688888888*55~" +
"NM1*QC*1*BEAR*YOGI* ***MI*2222222~" +
"NM1*82*2*JELLYSTONE SVCS*****FI*346608640~" +
"REF*F8*A76B04054~" +
"SVC*ZZ:A0230:HF*74.61*59.69**1~" +
"DTM*472*20040203~" +
"CAS*PR*2*14.92*0~" +
"CAS*CO*42*0*0~" +
"REF*6R*333333~" +
"CLP*777777*25*136.9*0*0*13*0000000622222222*53~" +
"NM1*QC*1*BIRD*TWEETY*M***MI*4444444~" +
"NM1*82*2*GRANNY AGENCY*****FI*340716747~" +
"REF*F8*A76B03293~" +
"SVC*HC:H0015:HF:99:H9*136.9*0**1~" +
"DTM*472*20030911~" +
"CAS*PI*104*136.72*0~" +
"CAS*CO*42*0.18*0~" +
"REF*6R*777777~" +
"CLP*123456*22*-42.58*-42.58*0*13*0000000657575757*11~" +
"NM1*QC*1*SIMPSON*HOMER* ***MI*8787888~" +
"NM1*82*2*DOH GROUP*****FI*310626223~" +
"REF*F8*A57B04033~" +
"SVC*HC:H0036:GT:UK*-42.58*-42.58**-2~" +
"DTM*472*20040102~" +
"CAS*CR*141*0*0*42*0*0*22*0*0~" +
"CAS*OA*141*0*0~" +
"REF*6R*123456~" +
"CLP*090909*22*-86.76*-86.76*0*MC*0000000648484848*53~" +
"NM1*QC*1*DUCK*DAFFY*W***MI*1245849~" +
"NM1*82*2*ABTHSOLUTE HELP*****FI*346608640~" +
"REF*F8*A76B04054~" +
"SVC*HC:H0004:HF:H9*-86.76*-86.76**-4~" +
"DTM*472*20040210~" +
"CAS*CR*22*0*0*42*0*0~" +
"CAS*OA*22*0*0~" +
"REF*6R*090909~" +
"LQ*HE*MA92~" +
"SE*93*07504123~" +
"GE*1*1~" +
"IEA*1*004075123~";
    }
}