using System;
using System.Collections.Generic;
using System.Data;
using System.Windows.Forms;
using RDPCrystalEDILibrary;
using RDPCrystalEDILibrary.Docs.x4010;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to load an EDI document and use TypedDocuments to get it's data
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Load_4010_837P_With_TypedDocuments : UserControl
    {
        private DataSet mainDataSet = new DataSet("Claims Information");
        private DataTable subscriberTable = new DataTable("Subcriber Information");
        private DataTable claimsTable = new DataTable("Claims");
        private DataTable serviceLineTable = new DataTable("Service Lines");
        private DataTable interchangeHeaderTable = new DataTable("Interchange Header");
        private DataTable functionalGroupTable = new DataTable("Functional Group");

        private EDIValidator ediValidator = new EDIValidator();

        public Load_4010_837P_With_TypedDocuments()
        {
            InitializeComponent();

            InitializeTables();

            ediValidator.ProgressChanged += Validation_ProgressChanged;
        }

        private void InitializeTables()
        {
            interchangeHeaderTable.Columns.Add("Header Item", typeof(string));
            interchangeHeaderTable.Columns.Add("Header Value", typeof(string));

            functionalGroupTable.Columns.Add("Header Item", typeof(string));
            functionalGroupTable.Columns.Add("Header Value", typeof(string));

            DataColumn rowNum = subscriberTable.Columns.Add("Claim #", typeof(int));
            subscriberTable.Columns.Add("Subscriber Name");
            subscriberTable.Columns.Add("Insurance Company");
            subscriberTable.Columns.Add("Relation Type");
            subscriberTable.Columns.Add("Insurance Type");

            subscriberTable.PrimaryKey = new DataColumn[] { rowNum };

            DataColumn rowNum1 = claimsTable.Columns.Add("RowNum", typeof(int));
            rowNum1.ColumnMapping = MappingType.Hidden;
            claimsTable.Columns.Add("Claim ID");
            claimsTable.Columns.Add("Amount");
            claimsTable.Columns.Add("Diagnosis 1");

            DataColumn rowNum2 = serviceLineTable.Columns.Add("RowNum", typeof(int));
            rowNum2.ColumnMapping = MappingType.Hidden;
            serviceLineTable.Columns.Add("Procedure");
            serviceLineTable.Columns.Add("Charge");

            mainDataSet.Tables.Add(subscriberTable);
            mainDataSet.Tables.Add(claimsTable);
            mainDataSet.Tables.Add(serviceLineTable);

            mainDataSet.Relations.Add("Claim Information", rowNum, rowNum1);
            mainDataSet.Relations.Add("Service Line Info", rowNum1, serviceLineTable.Columns[0]);

            dataGrid.DataSource = mainDataSet;
        }

        private void InitializeEDI()
        {
            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = "ValidationRules\\Rules_4010_837P_004010X098A1.Rules";

            //Set the EDI data to load
            ediValidator.EDIDataString = SampleEDI837Data;
        }

        private void LoadEDI()
        {
            try
            {
                ediValidator.Validate();

                //The EDI Document will contain all the data of the 837 file
                if (ediValidator.EDILightWeightDocument != null)
                {
                    Typed4010Document doc = new Typed4010Document(ediValidator.EDILightWeightDocument);

                    DocumentLoop interchange = doc.MainSection.GetLoop("INTERCHANGE HEADER");

                    DisplayInterchangeHeader(interchange);

                    DisplayFunctionalHeader(interchange.GetLoop("FUNCTIONAL GROUP"));

                    ISA header = interchange.GetSegment<ISA>();

                    DocumentLoop stHeaderLoop = interchange.GetLoop("FUNCTIONAL GROUP/ST HEADER");

                    //Get the submitter loop. In an 837 file the submitter loop is 1000A
                    DocumentLoop submitterSection = stHeaderLoop.GetLoop("1000A");
                    DisplaySubmitterInformation(submitterSection);

                    //Get the receiver loop. In an 837 file the receiver loop is 1000B
                    DocumentLoop receiverSection = stHeaderLoop.GetLoop("1000B");
                    DisplayReceiverInformation(receiverSection);

                    //Get the 2000A BILLING/PAY-TO PROVIDER HIERARCHICAL LEVEL
                    //In an 837 file this loop is 2000A
                    DocumentLoop bpHierLevel = stHeaderLoop.GetLoop("2000A");
                    DocumentLoop billingProviderLoop = bpHierLevel.GetLoop("2010AA");
                    DisplayBillingProviderInformation(billingProviderLoop);

                    //Get all the subscribers and claims information
                    List<DocumentLoop> subscriberAndClaimsSections = bpHierLevel.GetLoops("2000B");
                    DisplayAllClaims(subscriberAndClaimsSections);

                    ediDocumentViewer.LoadDocument(ediValidator.EDILightWeightDocument);
                }
            }

            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        private void DisplayInterchangeHeader(DocumentLoop header)
        {
            ISA isa = header.GetSegment<ISA>();
            CreateInterchangeHeaderItem("Authorization Information Qualifier", isa.AuthorizationInformationQualifier);
            CreateInterchangeHeaderItem("Authorization Information", isa.AuthorizationInformation);
            CreateInterchangeHeaderItem("Security Information Qualifier", isa.SecurityInformationQualifier);
            CreateInterchangeHeaderItem("Security Information", isa.SecurityInformation);
            CreateInterchangeHeaderItem("Interchange ID Qualifier1", isa.InterchangeIDQualifier1);
            CreateInterchangeHeaderItem("Interchange Sender ID", isa.InterchangeSenderID);
            CreateInterchangeHeaderItem("Interchange ID Qualifier2", isa.InterchangeIDQualifier2);
            CreateInterchangeHeaderItem("Interchange Receiver ID", isa.InterchangeReceiverID);
            CreateInterchangeHeaderItem("Interchange Date", isa.InterchangeDate);
            CreateInterchangeHeaderItem("Interchange Time", isa.InterchangeTime);
            CreateInterchangeHeaderItem("Interchange Control Version Number", isa.InterchangeControlVersionNumber);
            CreateInterchangeHeaderItem("Interchange Control Number", isa.InterchangeControlNumber);
            CreateInterchangeHeaderItem("Acknowledgment Requested", isa.AcknowledgmentRequested);
            CreateInterchangeHeaderItem("Interchange Usage Indicator", isa.InterchangeUsageIndicator);
            CreateInterchangeHeaderItem("Component Element Separator", isa.ComponentElementSeparator);

            dgInterchangeHeader.DataSource = interchangeHeaderTable;
        }

        private void DisplayFunctionalHeader(DocumentLoop header)
        {
            GS gs = header.GetSegment<GS>();
            CreateFunctionalHeaderItem("Functional Identifier Code", gs.FunctionalIdentifierCode);
            CreateFunctionalHeaderItem("Application Senders Code", gs.ApplicationReceiverCode);
            CreateFunctionalHeaderItem("Application Receivers Code", gs.ApplicationReceiverCode);
            CreateFunctionalHeaderItem("Date", gs.Date);
            CreateFunctionalHeaderItem("Time", gs.Time);
            CreateFunctionalHeaderItem("Group Control Number", gs.GroupControlNumber);
            CreateFunctionalHeaderItem("Responsible Agency Code", gs.ResponsibleAgencyCode);
            CreateFunctionalHeaderItem("Version / Release / Industry Identifier Code", gs.VersionReleaseIndustryIdentifierCode);

            dgFunctionalHeader.DataSource = functionalGroupTable;
        }

        private void CreateInterchangeHeaderItem(string headerItem, string headerValue)
        {
            DataRow row = interchangeHeaderTable.NewRow();
            row["Header Item"] = headerItem;
            row["Header Value"] = headerValue;

            interchangeHeaderTable.Rows.Add(row);
        }

        private void CreateFunctionalHeaderItem(string headerItem, string headerValue)
        {
            DataRow row = functionalGroupTable.NewRow();
            row["Header Item"] = headerItem;
            row["Header Value"] = headerValue;

            functionalGroupTable.Rows.Add(row);
        }

        private void DisplayAllClaims(List<DocumentLoop> claims)
        {
            int rowNumber = 1;
            foreach (DocumentLoop claimSection in claims)
            {
                SBR subscriber = claimSection.GetSegment<SBR>();
                DocumentLoop subscriberInfo = claimSection.GetLoop("2010BA");
                NM1 subName = subscriberInfo.GetSegment<NM1>();

                DocumentLoop insuranceInfoSection = claimSection.GetLoop("2010BB");
                NM1 insuranceName = insuranceInfoSection.GetSegment<NM1>();

                DataRow subInfoRow = subscriberTable.NewRow();
                subInfoRow[0] = rowNumber;
                subInfoRow[1] = subName.NameLastOrOrganizationName + " " + subName.NameFirst;
                subInfoRow[2] = insuranceName.NameLastOrOrganizationName;
                subInfoRow[3] = subscriber.IndividualRelationshipCode;
                subInfoRow[4] = subscriber.ClaimFilingIndicatorCode;

                subscriberTable.Rows.Add(subInfoRow);

                List<DocumentLoop> claimLoopSections = claimSection.GetLoops("2300");

                foreach (DocumentLoop cl in claimLoopSections)
                {
                    CLM clm = cl.GetSegment<CLM>();

                    DataRow claimInfoRow = claimsTable.NewRow();
                    claimInfoRow[0] = rowNumber;
                    claimInfoRow[1] = clm.ClaimSubmitterIdentifier;
                    claimInfoRow[2] = clm.MonetaryAmount;

                    claimsTable.Rows.Add(claimInfoRow);

                    List<DocumentLoop> serviceLineSections = cl.GetLoops("2400");
                    foreach (DocumentLoop sl in serviceLineSections)
                    {
                        //Get the service line segment
                        SV1 sv1 = sl.GetSegment<SV1>();

                        DataRow serviceLineInfoRow = serviceLineTable.NewRow();
                        serviceLineInfoRow[0] = rowNumber;
                        serviceLineInfoRow[1] = sv1.CompositeMedicalProcedureIdentifier.ProductServiceID;
                        serviceLineInfoRow[2] = sv1.MonetaryAmount;

                        serviceLineTable.Rows.Add(serviceLineInfoRow);
                    }
                }

                rowNumber++;
            }

            dataGrid.NavigateTo(0, "Subcriber Information");
        }

        private void DisplayBillingProviderInformation(DocumentLoop billingProviderNameSection)
        {
            //Get the Billing Provider's name
            NM1 nm1 = billingProviderNameSection.GetSegment<NM1>();
            txtBillingProviderName.Text = nm1.NameLastOrOrganizationName + nm1.NameFirst;

            //Get the street address
            N3 n3 = billingProviderNameSection.GetSegment<N3>();
            txtProviderStreet.Text = n3.AddressInformation1;

            //Get the city state and zip code
            N4 n4 = billingProviderNameSection.GetSegment<N4>();
            txtProviderAddress.Text = n4.CityName + "," + n4.StateOrProvinceCode + "," + n4.PostalCode;
        }

        private void DisplaySubmitterInformation(DocumentLoop submitterSection)
        {
            //Get the submitter's name
            NM1 nm1 = submitterSection.GetSegment<NM1>();
            txtSubmitterName.Text = nm1.NameLastOrOrganizationName + nm1.NameFirst;

            //Get the submitter's contact information
            PER per = submitterSection.GetSegment<PER>();
            if (per != null)
            {
                txtSubmitterPhone.Text = per.CommunicationNumber1;
            }
        }

        private void DisplayReceiverInformation(DocumentLoop receiverSection)
        {
            //Get the receiver's name
            NM1 nm1 = receiverSection.GetSegment<NM1>();
            txtReceiverName.Text = nm1.NameLastOrOrganizationName + nm1.NameFirst;
        }

        private void Validation_ProgressChanged(int progress)
        {
            BeginInvoke(new Action(() =>
            {
                if ((pbValidator.Value < pbValidator.Maximum))
                    pbValidator.Value = progress;
            }));
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            pbValidator.Value = 0;

            claimsTable.Rows.Clear();
            serviceLineTable.Rows.Clear();
            subscriberTable.Rows.Clear();

            InitializeEDI();

            LoadEDI();
        }

        public const string SampleEDI837Data = "ISA*00*          *00*          *ZZ*SMITH          *ZZ*BCBSGA         *07011A*1350*U*00401*000000011*1*P*>~GS*HC*SMITH*BCBSGA*20200118*1350*11*Y*004010X098A1~GS*HC*SMITH*BCBSGA*20200118*1350*11*X*004010X098A1~ST*837*000000001~BHT*0019*00*00000*20200118*1350*CH~REF*87*004010X098A1~NM1*41*2*ABC LABORATORY*****46*SMITH~PER*IC*JANE DOE*TE*2127351234~NM1*40*2*A COMPANY*****46*00601~HL*1**20*1~NM1*85*2*ABC LABORATORY*****24*123456789~N3*112 ROAD WAY~N4*A City*NY*11232~REF*AS*00228498B~REF*G2*00228498L~HL*2*1*22*0~SBR*P*18*******CI~NM1*IL*1*SMITH*JOHN****MI*505XXXX~N3*2345 NORTH RD~N4*SPRINGFIELD*NY*11111~DMG*D8*19950101*F~NM1*PR*2*BLUE CROSS*****PI*XX123A~N3*PO BOX 78765~N4*SOUTH BEACH*FL*23466~CLM*1-1178*126.84***81>>1*Y*A*Y*Y*B*AA>>>>>>~HI*BK>6268*BF>78930~NM1*DN*1*SMITH*BOBBY****24*123456789~REF*1G*T32418~NM1*82*2*SMITH*****24*123456789~REF*G2*00228498B~NM1*LI*2*SMITH LANEY*****24*123456789~N3*1111 DRICE PLACE~N4*SOMEWHERE*NY*11119~LX*1~SV1*HC>87086*31.96*UN*1***1~DTP*472*D8*20200118~LX*1~SV1*HC>87086*31.96*UN*1***1~DTP*472*D8*20200118~LX*1~SV1*HC>87086*31.96*UN*1***1~DTP*472*D8*20200118~LX*1~SV1*HC>87086*31.96*UN*1***1~DTP*472*D8*20200118~HL*3*1*22*0~SBR*P*18*******CI~NM1*IL*1*SMITH*SARAH****MI*543XXXX~N3*1111 NORTH LEG RD~N4*ABC PLACE*NY*11223~DMG*D8*19950101*F~NM1*PR*2*AMERIGROUP/MDGA MANAGED CARE*****PI*12CCCA~N3*PO BOX 44344~N4*BRIGHTON*NY*23466~CLM*1-1179*40.1***81>>1*Y*A*Y*Y*B~HI*BK>5990~NM1*DN*1*SMITH*WILLIAM****24*7876666666~REF*1G*T12345~NM1*82*2*SMITH*****24*111212222~REF*G2*00228498B~NM1*LI*2*SMITH LAB*****24*123456789~N3*1111 DRICE PLACE~N4*PLACE CITY*NY*11234~LX*1~SV1*HC>80053*40.1*UN*1***1~DTP*472*D8*20200118~HL*4*1*22*0~SBR*P*18*******CI~NM1*IL*1*SMITH*TEST****MI*505XXXX~N3*1111 A ROAD~N4*PLACE CITY*NY*11234~DMG*D8*19950101*F~NM1*PR*2*ETNA*****PI*9876~N3*PO BOX 61010~N4*VIRGINIA BEACH*VA*23466~CLM*1-1180*27.26***81>>1*Y*A*Y*Y*B~HI*BK>3829~NM1*DN*1*SMITH*WILLIAM****24*123456789~REF*1G*T12345~NM1*82*2*SMITH*****24*123456789~REF*G2*00227448X~NM1*LI*2*SMITH LAB*****24*123456789~N3*1527 NORTH LEG~N4*PLACE CITY*NY*11234~LX*1~SV1*HC>85025*27.26*UN*1***1~DTP*472*D8*20200118~HL*5*1*22*0~SBR*P*18*******CI~NM1*IL*1*LAB*TEST****MI*111XXXX~N3*1111 A ROAD~N4*PLACE CITY*NY*11234~DMG*D8*19880101*M~NM1*PR*2*PRONTO INSURANCE 101*****PI*12345~N3*PO BOX 61010~N4*VIRGINIA BEACH*VA*23466~CLM*1-1181*15.98***81>>1*Y*A*Y*Y*B~HI*BK>05671~NM1*DN*1*SMITH*WILLIAM****24*123456789~REF*1G*T32418~NM1*82*2*SMITH*****24*123456789~REF*G2*00227448X~NM1*LI*2*SMITH LAB*****24*123456789~N3*1111 DRICE PLACE~N4*PLACE CITY*NY*11234~LX*1~SV1*HC>85651*15.98*UN*1***1~DTP*472*D8*20200118~HL*6*1*22*0~SBR*P*18*******CI~NM1*IL*1*LAB*TEST****MI*111XXXX~N3*1111 A ROAD~N4*PLACE CITY*NY*11234~DMG*D8*19880101*M~NM1*PR*2*FAST SERVICE INSURANCE*****PI*12345~N3*PO BOX 61010~N4*VIRGINIA BEACH*VA*23466~CLM*1-1182*40.1***81>>1*Y*A*Y*Y*B~HI*BK>05671~NM1*DN*1*SMITH*WILLIAM****24*123456789~REF*1G*T32418~NM1*82*2*SMITH*****24*123456789~REF*G2*00227448X~NM1*LI*2*SMITH LAB*****24*123456789~N3*1111 DRICE PLACE~N4*PLACE CITY*NY*11234~LX*1~SV1*HC>80053*40.1*UN*1***1~DTP*472*D8*20200118~HL*7*1*22*0~SBR*P*18*******CI~NM1*IL*1*BORAT*LAB****MI*222XXXX~N3*1402 WALTON WAY~N4*AUGUSTA*GA*30901~DMG*D8*19750101*M~NM1*PR*2*AMERIGROUP/MDGA MANAGED CARE*****PI*12345~N3*PO BOX 61010~N4*VIRGINIA BEACH*VA*23466~CLM*1-1183*27.26***81>>1*Y*A*Y*Y*B~HI*BK>20891~NM1*DN*1*SMITH*WILLIAM****24*123456789~REF*1G*T32418~NM1*82*2*SMITH*****24*123456789~REF*G2*00227448X~NM1*LI*2*SMITH LAB*****24*123456789~N3*1111 DRICE PLACE~N4*PLACE CITY*NY*11234~LX*1~SV1*HC>85025*27.26*UN*1***1~DTP*472*D8*20200118~HL*8*1*22*0~SBR*P*18*******CI~NM1*IL*1*BORAT*LAB****MI*222XXXX~N3*1402 WALTON WAY~N4*AUGUSTA*GA*30901~DMG*D8*19750101*M~NM1*PR*2*AMERIGROUP/MDGA MANAGED CARE*****PI*12345~N3*PO BOX 61010~N4*VIRGINIA BEACH*VA*23466~CLM*1-1184*32.9***81>>1*Y*A*Y*Y*B~HI*BK>20891~NM1*DN*1*SMITH*WILLIAM****24*123456789~REF*1G*T32418~NM1*82*2*SMITH*****24*123456789~REF*G2*00227448X~NM1*LI*2*SMITH LAB*****24*123456789~N3*1111 DRICE PLACE~N4*PLACE CITY*NY*11234~LX*1~SV1*HC>80048*32.9*UN*1***1~DTP*472*D8*20200118~HL*9*1*22*0~SBR*P*18*******CI~NM1*IL*1*BORAT*LAB****MI*222XXXX~N3*1402 WALTON WAY~N4*AUGUSTA*GA*30901~DMG*D8*19750101*M~NM1*PR*2*AMERIGROUP/MDGA MANAGED CARE*****PI*12345~N3*PO BOX 61010~N4*VIRGINIA BEACH*VA*23466~CLM*1-1185*74.25***81>>1*Y*A*Y*Y*B~HI*BK>20891~NM1*DN*1*SMITH*WILLIAM****24*123456789~REF*1G*T32418~NM1*82*2*SMITH*****24*123456789~REF*G2*00227448X~NM1*LI*2*SMITH LAB*****24*123456789~N3*1111 DRICE PLACE~N4*PLACE CITY*NY*11234~LX*1~SV1*HC>85060*74.25*UN*1***1~DTP*472*D8*20200118~SE*190*000000001~GE*1*11~IEA*1*000000011~";
    }
}