using System;
using System.Collections.Generic;
using System.Data;
using System.Windows.Forms;
using RDPCrystalEDILibrary;
using RDPCrystalEDILibrary.Docs.x5010;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to parse and load a 5010 834 EDI document using the EDIValidator component.
    /// EDI data will then be dislayed in the UI
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Load_5010_834_File : UserControl
    {
        private DataTable memberTable = new DataTable("Subcriber Information");
        private DataTable interchangeHeaderTable = new DataTable("Interchange Header");
        private DataTable functionalGroupTable = new DataTable("Functional Group");

        private EDIValidator ediValidator = new EDIValidator();

        public Load_5010_834_File()
        {
            InitializeComponent();

            ediValidator.ProgressChanged += Validation_ProgressChanged;

            InitializeTables();
        }

        private void InitializeTables()
        {
            interchangeHeaderTable.Columns.Add("Header Item", typeof(string));
            interchangeHeaderTable.Columns.Add("Header Value", typeof(string));

            functionalGroupTable.Columns.Add("Header Item", typeof(string));
            functionalGroupTable.Columns.Add("Header Value", typeof(string));

            memberTable.Columns.Add("Member Name");
            memberTable.Columns.Add("Subscriber ID");
            memberTable.Columns.Add("Subscriber");
            memberTable.Columns.Add("Relationship");
            memberTable.Columns.Add("Health Coverage");
            memberTable.Columns.Add("Policy Begin Date");
            memberTable.Columns.Add("Policy End Date");

            dataGrid.DataSource = memberTable;
        }

        private void InitializeEDI()
        {
            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = "ValidationRules\\Rules_5010_834_005010X220A1.Rules";

            //Set the EDI data to load
            ediValidator.EDIFile = "EDIFiles\\Sample_5010_834_2.txt";
        }

        private void LoadEDI()
        {
            try
            {
                ediValidator.Validate();

                //The EDI Document will contain all the data of the 834 file
                if (ediValidator.EDILightWeightDocument != null)
                {
                    //Show the document on the screen
                    ediDocumentViewer.LoadDocument(ediValidator.EDILightWeightDocument);

                    //Convert to a typed document
                    Typed5010Document doc = new Typed5010Document(ediValidator.EDILightWeightDocument);

                    DocumentLoop interchange = doc.MainSection.GetLoop("INTERCHANGE HEADER");

                    //Display ISA and GS header segments
                    DisplayInterchangeHeader(interchange);

                    DocumentLoop functionalHeaderLoop = interchange.GetLoop("FUNCTIONAL GROUP");
                    DisplayFunctionalHeader(functionalHeaderLoop);

                    //Get the ST transaction loop
                    DocumentLoop stHeaderLoop = functionalHeaderLoop.GetLoop("ST HEADER");

                    //Display Sponser Info
                    DocumentLoop sponserSection = stHeaderLoop.GetLoop("1000A");
                    DisplaySponserInformation(sponserSection);

                    //Display Payer Info
                    DocumentLoop payerSection = stHeaderLoop.GetLoop("1000B");
                    DisplayPayerInformation(payerSection);

                    //Display member insurance info
                    List<DocumentLoop> memberLevels = stHeaderLoop.GetLoops("2000");

                    DisplayMemberLevels(memberLevels);
                }
            }
            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        private void DisplayInterchangeHeader(DocumentLoop header)
        {
            ISA isa = header.GetSegment<ISA>();
            CreateInterchangeHeaderItem("Authorization Information Qualifier", isa.AuthorizationInformationQualifier);
            CreateInterchangeHeaderItem("Authorization Information", isa.AuthorizationInformation);
            CreateInterchangeHeaderItem("Security Information Qualifier", isa.SecurityInformationQualifier);
            CreateInterchangeHeaderItem("Security Information", isa.SecurityInformation);
            CreateInterchangeHeaderItem("Interchange ID Qualifier1", isa.InterchangeIDQualifier1);
            CreateInterchangeHeaderItem("Interchange Sender ID", isa.InterchangeSenderID);
            CreateInterchangeHeaderItem("Interchange ID Qualifier2", isa.InterchangeIDQualifier2);
            CreateInterchangeHeaderItem("Interchange Receiver ID", isa.InterchangeReceiverID);
            CreateInterchangeHeaderItem("Interchange Date", isa.InterchangeDate);
            CreateInterchangeHeaderItem("Interchange Time", isa.InterchangeTime);
            CreateInterchangeHeaderItem("Repetition Separator", isa.RepetitionSeparator);
            CreateInterchangeHeaderItem("Interchange Control Version Number", isa.InterchangeControlVersionNumber);
            CreateInterchangeHeaderItem("Interchange Control Number", isa.InterchangeControlNumber);
            CreateInterchangeHeaderItem("Acknowledgment Requested", isa.AcknowledgmentRequested);
            CreateInterchangeHeaderItem("Interchange Usage Indicator", isa.InterchangeUsageIndicator);
            CreateInterchangeHeaderItem("Component Element Separator", isa.ComponentElementSeparator);

            dgInterchangeHeader.DataSource = interchangeHeaderTable;
        }

        private void DisplayFunctionalHeader(DocumentLoop header)
        {
            GS gs = header.GetSegment<GS>();
            CreateFunctionalHeaderItem("Functional Identifier Code", gs.FunctionalIdentifierCode);
            CreateFunctionalHeaderItem("Application Senders Code", gs.ApplicationReceiverCode);
            CreateFunctionalHeaderItem("Application Receivers Code", gs.ApplicationReceiverCode);
            CreateFunctionalHeaderItem("Date", gs.Date);
            CreateFunctionalHeaderItem("Time", gs.Time);
            CreateFunctionalHeaderItem("Group Control Number", gs.GroupControlNumber);
            CreateFunctionalHeaderItem("Responsible Agency Code", gs.ResponsibleAgencyCode);
            CreateFunctionalHeaderItem("Version / Release / Industry Identifier Code", gs.VersionReleaseIndustryIdentifierCode);

            dgFunctionalHeader.DataSource = functionalGroupTable;
        }

        private void DisplaySponserInformation(DocumentLoop submitterSection)
        {
            //Get the sponser name
            N1 n1 = submitterSection.GetSegment<N1>();
            txtSponser.Text = n1.Name;
            txtSponserID.Text = n1.IdentificationCode;
        }

        private void DisplayPayerInformation(DocumentLoop receiverSection)
        {
            //Get the payer name
            N1 n1 = receiverSection.GetSegment<N1>();
            txtPayerName.Text = n1.Name;
            txtPayerID.Text = n1.IdentificationCode;
        }

        private void CreateInterchangeHeaderItem(string headerItem, string headerValue)
        {
            DataRow row = interchangeHeaderTable.NewRow();
            row["Header Item"] = headerItem;
            row["Header Value"] = headerValue;

            interchangeHeaderTable.Rows.Add(row);
        }

        private void CreateFunctionalHeaderItem(string headerItem, string headerValue)
        {
            DataRow row = functionalGroupTable.NewRow();
            row["Header Item"] = headerItem;
            row["Header Value"] = headerValue;

            functionalGroupTable.Rows.Add(row);
        }

        private void DisplayMemberLevels(List<DocumentLoop> memberLevels)
        {
            foreach (DocumentLoop memberLoop in memberLevels)
            {
                INS ins = memberLoop.GetSegment<INS>();
                REF subscriberID = memberLoop.GetSegment<REF>();

                DocumentLoop memberNameLoop = memberLoop.GetLoop("2100A");
                NM1 memberName = memberNameLoop.GetSegment<NM1>();

                DocumentLoop healthCoverageLoop = memberLoop.GetLoop("2300");
                HD healthCoverage = healthCoverageLoop.GetSegment<HD>();

                List<DocumentSegment> dates = healthCoverageLoop.GetSegments("DTP");

                DTP policyBeginDate = dates[0] as DTP;
                DTP policyEndDate = dates[1] as DTP;

                DataRow subInfoRow = memberTable.NewRow();
                subInfoRow[0] = memberName.NameLastOrOrganizationName + " " + memberName.FirstName;
                subInfoRow[1] = subscriberID.ReferenceIdentification;
                subInfoRow[2] = ins.YesNoConditionOrResponseCode1;
                subInfoRow[3] = ins.IndividualRelationshipCode;
                subInfoRow[4] = healthCoverage.InsuranceLineCode;
                subInfoRow[5] = policyBeginDate.DateTimePeriod;
                subInfoRow[6] = policyEndDate.DateTimePeriod;

                memberTable.Rows.Add(subInfoRow);
            }
        }

        private void Validation_ProgressChanged(int progress)
        {
            BeginInvoke(new Action(() =>
            {
                if ((pbValidator.Value < pbValidator.Maximum))
                    pbValidator.Value = progress;
            }));
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            pbValidator.Value = 0;

            memberTable.Rows.Clear();

            InitializeEDI();

            LoadEDI();
        }
    }
}