using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Windows.Forms;
using RDPCrystalEDILibrary;
using RDPCrystalEDILibrary.Docs.x5010;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to parse and load a 5010 835 EDI document using the EDIValidator component.
    /// EDI data will then be dislayed in the UI
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Load_5010_835_File : UserControl
    {
        private DataSet mainDataSet = new DataSet("Claims Information");
        private DataTable claimPaymentInfoTable = new DataTable("Claim Payment Information");
        private DataTable servicePaymentInfoTable = new DataTable("Service Payment Information");
        private DataTable interchangeHeaderTable = new DataTable("Interchange Header");
        private DataTable functionalGroupTable = new DataTable("Functional Group");

        private EDIValidator ediValidator = new EDIValidator();

        public Load_5010_835_File()
        {
            InitializeComponent();

            InitializeTables();

            txtFile.Text = File.ReadAllText("EDIFiles\\Sample_5010_835.txt");
        }

        private void InitializeTables()
        {
            interchangeHeaderTable.Columns.Add("Header Item", typeof(string));
            interchangeHeaderTable.Columns.Add("Header Value", typeof(string));

            functionalGroupTable.Columns.Add("Header Item", typeof(string));
            functionalGroupTable.Columns.Add("Header Value", typeof(string));

            DataColumn rowNum = claimPaymentInfoTable.Columns.Add("Claim #", typeof(int));
            claimPaymentInfoTable.Columns.Add("Patient Name");
            claimPaymentInfoTable.Columns.Add("Insured Name");

            claimPaymentInfoTable.PrimaryKey = new DataColumn[] { rowNum };

            DataColumn rowNum1 = servicePaymentInfoTable.Columns.Add("RowNum", typeof(int));
            rowNum1.ColumnMapping = MappingType.Hidden;
            servicePaymentInfoTable.Columns.Add("Product/Service ID");
            servicePaymentInfoTable.Columns.Add("Amount");
            servicePaymentInfoTable.Columns.Add("Quantity");

            mainDataSet.Tables.Add(claimPaymentInfoTable);
            mainDataSet.Tables.Add(servicePaymentInfoTable);

            mainDataSet.Relations.Add("Claim Payment Information", rowNum, rowNum1);

            dataGrid.DataSource = mainDataSet;
        }

        private void InitializeEDI()
        {
            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = "ValidationRules\\Rules_5010_835_005010X221A1.Rules";

            //Set the EDI data to load

            ediValidator.EDIFile = "EDIFiles\\Sample_5010_835.txt";
        }

        private void LoadEDI()
        {
            try
            {
                ediValidator.Validate();

                //The EDI Document will contain all the data of the 835 file
                if (ediValidator.EDILightWeightDocument != null)
                {
                    Typed5010Document doc = new Typed5010Document(ediValidator.EDILightWeightDocument);

                    DocumentLoop interchange = doc.MainSection.GetLoop("INTERCHANGE HEADER");

                    DisplayInterchangeHeader(interchange.GetSegment<ISA>());
                    DisplayFunctionalHeader(interchange.GetSegment<GS>("FUNCTIONAL GROUP/GS"));

                    //Get the ST loops.  The sample EDI file contains 2 ST-SE groups however we will only display one
                    List<DocumentLoop> stHeaderLoops = interchange.GetLoops("FUNCTIONAL GROUP/ST HEADER");

                    foreach (DocumentLoop stHeaderLoop in stHeaderLoops)
                    {
                        //Get the payer identification loop. In an 835 file the payer identification loop is 1000A
                        DocumentLoop payerIdentificationLoop = stHeaderLoop.GetLoop("1000A");
                        DisplayPayerIdenfificationInformation(payerIdentificationLoop);

                        // Get the receiver loop. In an 837 file the receiver loop is 1000B
                        DocumentLoop payeeIdentificationLoop = stHeaderLoop.GetLoop("1000B");
                        DisplayPayeeIdenfificationInformation(payeeIdentificationLoop);

                        // Get the 2000A BILLING/PAY-TO PROVIDER HIERARCHICAL LEVEL
                        //In an 837 file this loop is 2000A
                        DocumentLoop headerNumberLoop = stHeaderLoop.GetLoop("2000");
                        List<DocumentLoop> claimPaymentInformationLoops = headerNumberLoop.GetLoops("2100");

                        DisplayAllClaimPayments(claimPaymentInformationLoops);

                        //In this demo we are only displaying data in the first ST-SE group
                        //However you can display as many ST-SE groups as desired
                        break;
                    }

                    ediDocumentViewer.LoadDocument(ediValidator.EDILightWeightDocument);
                }
            }
            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        private void DisplayInterchangeHeader(ISA isa)
        {
            CreateInterchangeHeaderItem("Authorization Information Qualifier", isa.AuthorizationInformationQualifier);
            CreateInterchangeHeaderItem("Authorization Information", isa.AuthorizationInformation);
            CreateInterchangeHeaderItem("Security Information Qualifier", isa.SecurityInformationQualifier);
            CreateInterchangeHeaderItem("Security Information", isa.SecurityInformation);
            CreateInterchangeHeaderItem("Interchange ID Qualifier1", isa.InterchangeIDQualifier1);
            CreateInterchangeHeaderItem("Interchange Sender ID", isa.InterchangeSenderID);
            CreateInterchangeHeaderItem("Interchange ID Qualifier2", isa.InterchangeIDQualifier2);
            CreateInterchangeHeaderItem("Interchange Receiver ID", isa.InterchangeReceiverID);
            CreateInterchangeHeaderItem("Interchange Date", isa.InterchangeDate);
            CreateInterchangeHeaderItem("Interchange Time", isa.InterchangeTime);
            CreateInterchangeHeaderItem("Repetition Separator", isa.RepetitionSeparator);
            CreateInterchangeHeaderItem("Interchange Control Version Number", isa.InterchangeControlVersionNumber);
            CreateInterchangeHeaderItem("Interchange Control Number", isa.InterchangeControlNumber);
            CreateInterchangeHeaderItem("Acknowledgment Requested", isa.AcknowledgmentRequested);
            CreateInterchangeHeaderItem("Interchange Usage Indicator", isa.InterchangeUsageIndicator);
            CreateInterchangeHeaderItem("Component Element Separator", isa.ComponentElementSeparator);

            dgInterchangeHeader.DataSource = interchangeHeaderTable;
        }

        private void DisplayFunctionalHeader(GS gs)
        {
            CreateFunctionalHeaderItem("Functional Identifier Code", gs.FunctionalIdentifierCode);
            CreateFunctionalHeaderItem("Application Senders Code", gs.ApplicationSenderCode);
            CreateFunctionalHeaderItem("Application Receivers Code", gs.ApplicationReceiverCode);
            CreateFunctionalHeaderItem("Date", gs.Date);
            CreateFunctionalHeaderItem("Time", gs.Time);
            CreateFunctionalHeaderItem("Group Control Number", gs.GroupControlNumber);
            CreateFunctionalHeaderItem("Responsible Agency Code", gs.ResponsibleAgencyCode);
            CreateFunctionalHeaderItem("Version / Release / Industry Identifier Code", gs.VersionReleaseIndustryIdentifierCode);

            dgFunctionalHeader.DataSource = functionalGroupTable;
        }

        private void CreateInterchangeHeaderItem(string headerItem, string headerValue)
        {
            DataRow row = interchangeHeaderTable.NewRow();
            row["Header Item"] = headerItem;
            row["Header Value"] = headerValue;

            interchangeHeaderTable.Rows.Add(row);
        }

        private void CreateFunctionalHeaderItem(string headerItem, string headerValue)
        {
            DataRow row = functionalGroupTable.NewRow();
            row["Header Item"] = headerItem;
            row["Header Value"] = headerValue;

            functionalGroupTable.Rows.Add(row);
        }

        private void DisplayAllClaimPayments(List<DocumentLoop> claimPayments)
        {
            int rowNumber = 1;
            foreach (DocumentLoop loop in claimPayments)
            {
                List<NM1> parties = loop.GetSegments<NM1>("NM1");

                string patientName = parties[0].FirstName + " " + parties[0].NameLastOrOrganizationName;
                string insuredName = parties[1].FirstName + " " + parties[1].NameLastOrOrganizationName;

                DataRow subInfoRow = claimPaymentInfoTable.NewRow();
                subInfoRow[0] = rowNumber;
                subInfoRow[1] = patientName;
                subInfoRow[2] = insuredName;

                claimPaymentInfoTable.Rows.Add(subInfoRow);

                foreach (DocumentLoop paymentInformation in loop.GetLoops("2110"))
                {
                    SVC svc = paymentInformation.GetSegment<SVC>("SVC");

                    DataRow claimInfoRow = servicePaymentInfoTable.NewRow();
                    claimInfoRow[0] = rowNumber;
                    claimInfoRow[1] = svc.CompositeMedicalProcedureIdentifier1.ProductServiceID1;
                    claimInfoRow[2] = svc.MonetaryAmount1;
                    claimInfoRow[3] = svc.Quantity1;

                    servicePaymentInfoTable.Rows.Add(claimInfoRow);
                }

                rowNumber++;
            }

            dataGrid.NavigateTo(0, "Claim Payment Information");
        }

        private void DisplayPayerIdenfificationInformation(DocumentLoop loop)
        {
            //Get the name
            N1 n1 = loop.GetSegment<N1>();
            txtPayerName.Text = n1.Name1;

            //Get the address street
            N3 n3 = loop.GetSegment<N3>();
            if (n3 != null)
            {
                txtPayerAdd1.Text = n3.AddressInformation1;
            }

            //Get the address 2
            N4 n4 = loop.GetSegment<N4>();
            if (n4 != null)
            {
                txtPayerAdd2.Text = n4.CityName + " " + n4.StateOrProvinceCode+ " " + n4.PostalCode;
            }
        }

        private void DisplayPayeeIdenfificationInformation(DocumentLoop loop)
        {
            //Get the name
            N1 n1 = loop.GetSegment<N1>();
            txtPayeeName.Text = n1.Name1;

            //Get the address street
            N3 n3 = loop.GetSegment<N3>();
            if (n3 != null)
            {
                txtPayeeAdd1.Text = n3.AddressInformation1;
            }

            //Get the address 2
            N4 n4 = loop.GetSegment<N4>();
            if (n4 != null)
            {
                txtPayeeAdd2.Text = n4.CityName + " " + n4.StateOrProvinceCode + " " + n4.PostalCode;
            }
        }

        private void Validation_ProgressChanged(int progress)
        {
            BeginInvoke(new Action(() =>
            {
                if ((pbValidator.Value < pbValidator.Maximum))
                    pbValidator.Value = progress;
            }));
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            //Reset the progress bar & tables
            pbValidator.Value = 0;

            servicePaymentInfoTable.Rows.Clear();
            claimPaymentInfoTable.Rows.Clear();

            InitializeEDI();

            LoadEDI();
        }
    }
}