using System;
using System.Collections.Generic;
using System.Data;
using System.Windows.Forms;
using RDPCrystalEDILibrary;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to parse and load a 5010 837 EDI document using the EDIValidator component.
    /// EDI data will then be dislayed in the UI
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Load_5010_837P_File : UserControl
    {
        private DataSet mainDataSet = new DataSet("Claims Information");
        private DataTable subscriberTable = new DataTable("Subcriber Information");
        private DataTable claimsTable = new DataTable("Claims");
        private DataTable serviceLineTable = new DataTable("Service Lines");
        private DataTable interchangeHeaderTable = new DataTable("Interchange Header");
        private DataTable functionalGroupTable = new DataTable("Functional Group");

        private EDIValidator ediValidator = new EDIValidator();

        public Load_5010_837P_File()
        {
            InitializeComponent();

            ediValidator.ProgressChanged += Validation_ProgressChanged;

            InitializeTables();
        }

        private void InitializeTables()
        {
            interchangeHeaderTable.Columns.Add("Header Item", typeof(string));
            interchangeHeaderTable.Columns.Add("Header Value", typeof(string));

            functionalGroupTable.Columns.Add("Header Item", typeof(string));
            functionalGroupTable.Columns.Add("Header Value", typeof(string));

            DataColumn rowNum = subscriberTable.Columns.Add("Claim #", typeof(int));
            subscriberTable.Columns.Add("Subscriber Name");
            subscriberTable.Columns.Add("Insurance Company");
            subscriberTable.Columns.Add("Relation Type");
            subscriberTable.Columns.Add("Insurance Type");

            subscriberTable.PrimaryKey = new DataColumn[] { rowNum };

            DataColumn rowNum1 = claimsTable.Columns.Add("RowNum", typeof(int));
            rowNum1.ColumnMapping = MappingType.Hidden;
            claimsTable.Columns.Add("Claim ID");
            claimsTable.Columns.Add("Amount");
            claimsTable.Columns.Add("Diagnosis 1");

            DataColumn rowNum2 = serviceLineTable.Columns.Add("RowNum", typeof(int));
            rowNum2.ColumnMapping = MappingType.Hidden;
            serviceLineTable.Columns.Add("Procedure");
            serviceLineTable.Columns.Add("Charge");
            serviceLineTable.Columns.Add("Quantity");
            serviceLineTable.Columns.Add("Diagnostic Code");

            mainDataSet.Tables.Add(subscriberTable);
            mainDataSet.Tables.Add(claimsTable);
            mainDataSet.Tables.Add(serviceLineTable);

            mainDataSet.Relations.Add("Claim Information", rowNum, rowNum1);
            mainDataSet.Relations.Add("Service Line Info", rowNum1, serviceLineTable.Columns[0]);

            dataGrid.DataSource = mainDataSet;
        }

        private void InitializeEDI()
        {
            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = "ValidationRules\\Rules_5010_837P_005010X222A1.Rules";

            //Set the EDI data to load
            ediValidator.EDIFile = "EDIFiles\\Sample_5010_837P_For_Loading.txt";
        }

        private void LoadEDI()
        {
            try
            {
                ediValidator.Validate();

                //The EDI Document will contain all the data of the 837 file
                if (ediValidator.EDILightWeightDocument != null)
                {
                    EDILightWeightDocument file837 = ediValidator.EDILightWeightDocument;

                    LightWeightLoop interchangeHeaderLoop = file837.Loops[0].GetLoop("INTERCHANGE HEADER");

                    DisplayInterchangeHeader(interchangeHeaderLoop.GetSegment("ISA"));

                    LightWeightLoop functionalHeaderLoop = interchangeHeaderLoop.GetLoop("FUNCTIONAL GROUP");

                    DisplayFunctionalHeader(functionalHeaderLoop.GetSegment("GS"));

                    LightWeightLoop stHeaderLoop = functionalHeaderLoop.GetLoop("ST HEADER");

                    //Get the submitter loop. In an 837 file the submitter loop is 1000A
                    LightWeightLoop submitterLoop = stHeaderLoop.GetLoop("1000A");
                    DisplaySubmitterInformation(submitterLoop);

                    //Get the receiver loop. In an 837 file the receiver loop is 1000B
                    LightWeightLoop receiverLoop = stHeaderLoop.GetLoop("1000B");
                    DisplayReceiverInformation(receiverLoop);

                    //Get the 2000A BILLING/PAY-TO PROVIDER HIERARCHICAL LEVEL
                    //In an 837 file this loop is 2000A
                    LightWeightLoop bpHierLevel = stHeaderLoop.GetLoop("2000A");

                    LightWeightLoop billingProviderLoop = bpHierLevel.GetLoop("2010AA");
                    DisplayBillingProviderInformation(billingProviderLoop);

                    //Get all the subscribers and claims information
                    List<LightWeightLoop> subscriberAndClaimsLoops = bpHierLevel.GetLoops("2000B");
                    DisplayAllClaims(subscriberAndClaimsLoops);

                    //Show the document on the screen
                    ediDocumentViewer.LoadDocument(ediValidator.EDILightWeightDocument);
                }
            }
            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        private void DisplayInterchangeHeader(LightWeightSegment isa)
        {
            CreateInterchangeHeaderItem("Authorization Information Qualifier", isa.Elements[0].DataValue);
            CreateInterchangeHeaderItem("Authorization Information", isa.Elements[1].DataValue);
            CreateInterchangeHeaderItem("Security Information Qualifier", isa.Elements[2].DataValue);
            CreateInterchangeHeaderItem("Security Information", isa.Elements[3].DataValue);
            CreateInterchangeHeaderItem("Interchange ID Qualifier1", isa.Elements[4].DataValue);
            CreateInterchangeHeaderItem("Interchange Sender ID", isa.Elements[5].DataValue);
            CreateInterchangeHeaderItem("Interchange ID Qualifier2", isa.Elements[6].DataValue);
            CreateInterchangeHeaderItem("Interchange Receiver ID", isa.Elements[7].DataValue);
            CreateInterchangeHeaderItem("Interchange Date", isa.Elements[8].DataValue);
            CreateInterchangeHeaderItem("Interchange Time", isa.Elements[9].DataValue);
            CreateInterchangeHeaderItem("Repetition Separator", isa.Elements[10].DataValue);
            CreateInterchangeHeaderItem("Interchange Control Version Number", isa.Elements[11].DataValue);
            CreateInterchangeHeaderItem("Interchange Control Number", isa.Elements[12].DataValue);
            CreateInterchangeHeaderItem("Acknowledgment Requested", isa.Elements[13].DataValue);
            CreateInterchangeHeaderItem("Interchange Usage Indicator", isa.Elements[14].DataValue);
            CreateInterchangeHeaderItem("Component Element Separator", isa.Elements[15].DataValue);

            dgInterchangeHeader.DataSource = interchangeHeaderTable;
        }

        private void DisplayFunctionalHeader(LightWeightSegment gs)
        {
            CreateFunctionalHeaderItem("Functional Identifier Code", gs.Elements[0].DataValue);
            CreateFunctionalHeaderItem("Application Senders Code", gs.Elements[1].DataValue);
            CreateFunctionalHeaderItem("Application Receivers Code", gs.Elements[2].DataValue);
            CreateFunctionalHeaderItem("Date", gs.Elements[3].DataValue);
            CreateFunctionalHeaderItem("Time", gs.Elements[4].DataValue);
            CreateFunctionalHeaderItem("Group Control Number", gs.Elements[5].DataValue);
            CreateFunctionalHeaderItem("Responsible Agency Code", gs.Elements[6].DataValue);
            CreateFunctionalHeaderItem("Version / Release / Industry Identifier Code", gs.Elements[7].DataValue);

            dgFunctionalHeader.DataSource = functionalGroupTable;
        }

        private void CreateInterchangeHeaderItem(string headerItem, string headerValue)
        {
            DataRow row = interchangeHeaderTable.NewRow();
            row["Header Item"] = headerItem;
            row["Header Value"] = headerValue;

            interchangeHeaderTable.Rows.Add(row);
        }

        private void CreateFunctionalHeaderItem(string headerItem, string headerValue)
        {
            DataRow row = functionalGroupTable.NewRow();
            row["Header Item"] = headerItem;
            row["Header Value"] = headerValue;

            functionalGroupTable.Rows.Add(row);
        }

        private void DisplayAllClaims(List<LightWeightLoop> claims)
        {
            int rowNumber = 1;
            foreach (LightWeightLoop loop in claims)
            {
                LightWeightSegment subscriber = loop.GetSegment("SBR");
                LightWeightLoop subscriberInfo = loop.GetLoop("2010BA");
                LightWeightSegment subName = subscriberInfo.GetSegment("NM1");

                LightWeightLoop insuranceInfo = loop.GetLoop("2010BB");
                LightWeightSegment insuranceName = insuranceInfo.GetSegment("NM1");

                DataRow subInfoRow = subscriberTable.NewRow();
                subInfoRow[0] = rowNumber;
                subInfoRow[1] = subName.Elements[2].DataValue + " " + subName.Elements[3].DataValue + " " + subName.Elements[4].DataValue;
                subInfoRow[2] = insuranceName.Elements[2].DataValue;
                subInfoRow[3] = subscriber.Elements[1].DataValue;
                subInfoRow[4] = subscriber.Elements[8].DataValue;

                subscriberTable.Rows.Add(subInfoRow);

                List<LightWeightLoop> claimLoop = loop.GetLoops("2300");

                foreach (LightWeightLoop cl in claimLoop)
                {
                    LightWeightSegment clm = cl.GetSegment("CLM");
                    LightWeightSegment hi = cl.GetSegment("HI");

                    DataRow claimInfoRow = claimsTable.NewRow();
                    claimInfoRow[0] = rowNumber;
                    claimInfoRow[1] = clm.Elements[0].DataValue;
                    claimInfoRow[2] = clm.Elements[1].DataValue;
                    claimInfoRow[3] = hi.Elements[0].Elements[1].DataValue;

                    claimsTable.Rows.Add(claimInfoRow);

                    List<LightWeightLoop> serviceLineLoops = cl.GetLoops("2400");
                    foreach (LightWeightLoop sl in serviceLineLoops)
                    {
                        //Get the service line segment
                        LightWeightSegment sv1 = sl.GetSegment("SV1");

                        DataRow serviceLineInfoRow = serviceLineTable.NewRow();
                        serviceLineInfoRow[0] = rowNumber;
                        serviceLineInfoRow[1] = sv1.Elements[0].Elements[1].DataValue;
                        serviceLineInfoRow[2] = sv1.Elements[1].DataValue;
                        serviceLineInfoRow[3] = sv1.Elements[3].DataValue;
                        serviceLineInfoRow[4] = sv1.Elements[6].Elements[0].DataValue;

                        serviceLineTable.Rows.Add(serviceLineInfoRow);
                    }
                }

                //process situation where the subscriber and the patient are different
                List<LightWeightLoop> patHL = loop.GetLoops("2000C");
                if (patHL != null && patHL.Count > 0)
                {
                    List<LightWeightLoop> patClaimLoop = patHL[0].GetLoops("2300");

                    foreach (LightWeightLoop cl in patClaimLoop)
                    {
                        LightWeightSegment clm = cl.GetSegment("CLM");
                        LightWeightSegment hi = cl.GetSegment("HI");

                        DataRow claimInfoRow = claimsTable.NewRow();
                        claimInfoRow[0] = rowNumber;
                        claimInfoRow[1] = clm.Elements[0].DataValue;
                        claimInfoRow[2] = clm.Elements[1].DataValue;
                        claimInfoRow[3] = hi.Elements[0].Elements[1].DataValue;

                        claimsTable.Rows.Add(claimInfoRow);

                        List<LightWeightLoop> serviceLineLoops = cl.GetLoops("2400");
                        foreach (LightWeightLoop sl in serviceLineLoops)
                        {
                            //Get the service line segment
                            LightWeightSegment sv1 = sl.GetSegment("SV1");

                            DataRow serviceLineInfoRow = serviceLineTable.NewRow();
                            serviceLineInfoRow[0] = rowNumber;
                            serviceLineInfoRow[1] = sv1.Elements[0].Elements[1].DataValue;
                            serviceLineInfoRow[2] = sv1.Elements[1].DataValue;

                            serviceLineTable.Rows.Add(serviceLineInfoRow);
                        }
                    }
                }

                rowNumber++;
            }

            dataGrid.NavigateTo(0, "Subcriber Information");
        }

        private void DisplayBillingProviderInformation(LightWeightLoop billingProviderNameLoop)
        {
            //Get the Billing Provider's name
            LightWeightSegment nm1 = billingProviderNameLoop.GetSegment("NM1");
            txtBillingProviderName.Text = nm1.Elements[2].DataValue + nm1.Elements[3].DataValue;

            //Get the street address
            LightWeightSegment n3 = billingProviderNameLoop.GetSegment("N3");
            txtProviderStreet.Text = n3.Elements[0].DataValue;

            //Get the city state and zip code
            LightWeightSegment n4 = billingProviderNameLoop.GetSegment("N4");
            txtProviderAddress.Text = n4.Elements[0].DataValue + "," + n4.Elements[1].DataValue + "," + n4.Elements[2].DataValue;
        }

        private void DisplaySubmitterInformation(LightWeightLoop submitterLoop)
        {
            //Get the submitter's name
            LightWeightSegment nm1 = submitterLoop.GetSegment("NM1");
            txtSubmitterName.Text = nm1.Elements[2].DataValue + nm1.Elements[3].DataValue;

            //Get the submitter's contact information
            LightWeightSegment per = submitterLoop.GetSegment("PER");
            if (per != null)
            {
                txtSubmitterPhone.Text = per.Elements[3].DataValue;
            }
        }

        private void DisplayReceiverInformation(LightWeightLoop receiverLoop)
        {
            //Get the receiver's name
            LightWeightSegment nm1 = receiverLoop.GetSegment("NM1");
            txtReceiverName.Text = nm1.Elements[2].DataValue + nm1.Elements[3].DataValue;
        }

        private void Validation_ProgressChanged(int progress)
        {
            BeginInvoke(new Action(() =>
            {
                if ((pbValidator.Value < pbValidator.Maximum))
                    pbValidator.Value = progress;
            }));
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            //Reset the progress bar & tables
            pbValidator.Value = 0;

            claimsTable.Rows.Clear();
            serviceLineTable.Rows.Clear();
            subscriberTable.Rows.Clear();

            InitializeEDI();

            LoadEDI();
        }
    }
}