using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Data;
using System.Data.SQLite;
using System.IO;
using System.Windows.Forms;
using RDPCrystalEDILibrary;
using RDPCrystalEDILibrary.Docs.x5010;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to load and save a 5010 837 DENTAL EDI document into a database
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Save_5010_837D_To_Database : UserControl
    {
        private SQLiteConnection dbConnection;

        private EDIValidator ediValidator = new EDIValidator();

        public Save_5010_837D_To_Database()
        {
            InitializeComponent();
        }

        private void CreateDatabaseTables()
        {
            if (dbConnection != null)
            {
                dbConnection.Close();
            }

            File.Delete("MyDatabase.sqlite1");

            //We are going to use SQLLite to demonstrate how to simple load EDI data and save it in a simple database
            SQLiteConnection.CreateFile("MyDatabase.sqlite1");

            dbConnection = new SQLiteConnection("Data Source=MyDatabase.sqlite1;Version=3;");
            dbConnection.Open();

            //Creates the patient's table

            string patientTable = "CREATE TABLE Patients (Id INT PRIMARY KEY, First TEXT, Last TEXT, Insurance TEXT, RelationShip TEXT, InsuranceType TEXT, DOB TEXT, Gender TEXT)";
            string claimTable = "CREATE TABLE Claims (Id INT, PatientID INT, ClaimID TEXT, Amount REAL, Diagnosis1 TEXT, OfcName TEXT, OfcTaxID TEXT)";
            string servicesTable = "CREATE TABLE ServiceLines (Id INT, ClaimID INT, Procedure TEXT, Charge REAL, ToothNo TEXT, ServiceDate TEXT)";

            SQLiteCommand command = new SQLiteCommand(patientTable, dbConnection);
            command.ExecuteNonQuery();

            command = new SQLiteCommand(claimTable, dbConnection);
            command.ExecuteNonQuery();

            command = new SQLiteCommand(servicesTable, dbConnection);
            command.ExecuteNonQuery();
        }

        private void InitializeEDI()
        {
            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = "ValidationRules\\Rules_5010_837D_005010X224A2.Rules";

            txtEDIData.Text = File.ReadAllText("EDIFiles\\Sample_5010_837D_For_Database.txt");

            //Set EDI file to validate
            ediValidator.EDIFile = "EDIFiles\\Sample_5010_837D_For_Database.txt";
        }

        private void LoadEDI()
        {
            try
            {
                ediValidator.Validate();

                //The EDI Document will contain all the data of the 837 file
                if (ediValidator.EDILightWeightDocument != null)
                {
                    //Convert to Typed Document
                    Typed5010Document doc = new Typed5010Document(ediValidator.EDILightWeightDocument);

                    //Get all the transaction
                    List<DocumentLoop> transactions = doc.MainSection.GetLoops("INTERCHANGE HEADER/FUNCTIONAL GROUP/ST HEADER");

                    foreach (DocumentLoop transaction in transactions)
                    {
                        //Get all billing providers
                        List<DocumentLoop> billingProviders = transaction.GetLoops("2000A");

                        SaveAllClaims(billingProviders);
                    }

                    //Load the patient data in the grid
                    using (SQLiteCommand cmd = new SQLiteCommand("Select * from Patients", dbConnection))
                    {
                        DataTable dt = new DataTable();

                        dt.Load(cmd.ExecuteReader());

                        dgPatients.DataSource = dt;
                    }

                    //display a tree of the EDI data
                    ediDocumentViewer.LoadDocument(ediValidator.EDILightWeightDocument);
                }
            }
            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        private void SaveAllClaims(List<DocumentLoop> billingProviders)
        {
            int patientRecordID = 1;
            int claimRecordID = 1;
            int serviceLineRecordID = 1;

            foreach (DocumentLoop billingProviderLoop in billingProviders)
            {
                DocumentLoop subscriberLoop = billingProviderLoop.GetLoop("2000B");
                SBR subscriber = subscriberLoop.GetSegment<SBR>();

                DocumentLoop subscriberNameLoop = subscriberLoop.GetLoop("2010BA");
                NM1 subName = subscriberNameLoop.GetSegment<NM1>();

                DocumentLoop payerNameLoop = subscriberLoop.GetLoop("2010BB");
                NM1 insuranceName = payerNameLoop.GetSegment<NM1>();

                string subscriberID = subName.IdentificationCode;

                string relationshipToInsuredCode = subscriber.IndividualRelationshipCode;

                if (relationshipToInsuredCode == null)
                {
                    relationshipToInsuredCode = " ";
                }

                string dateOfBirth = null;
                string gender = null;
                List<DocumentLoop> claimLoops = null;

                //Claim section 2300
                if (relationshipToInsuredCode == "18")
                {
                    //Patient == Subscriber

                    DMG demographics = subscriberNameLoop.GetSegment<DMG>();

                    dateOfBirth = demographics.DateTimePeriod.ToString();

                    gender = demographics.GenderCode;

                    claimLoops = subscriberLoop.GetLoops("2300");
                }
                else
                {
                    //Patient != Subscriber

                    DocumentLoop patientLoop = subscriberLoop.GetLoop("2000C");

                    if (patientLoop != null)
                    {
                        claimLoops = patientLoop.GetLoops("2300");

                        PAT patientInfo = patientLoop.GetSegment<PAT>();

                        DocumentLoop patientNameLoop = patientLoop.GetLoop("2010CA");

                        NM1 patientName = patientNameLoop.GetSegment<NM1>();

                        DMG patientDemographics = patientNameLoop.GetSegment<DMG>();

                        dateOfBirth = patientDemographics.DateTimePeriod.ToString();

                        gender = patientDemographics.GenderCode;

                        relationshipToInsuredCode = patientInfo.IndividualRelationshipCode;
                    }
                }


                //Save Patient into database table
                string insert = String.Format("INSERT INTO Patients (Id, First, Last, Insurance, InsuranceType, RelationShip, DOB, Gender) VALUES ({0},'{1}','{2}','{3}','{4}','{5}','{6}','{7}')", patientRecordID, subName.FirstName, subName.NameLastOrOrganizationName, insuranceName.NameLastOrOrganizationName, subscriber.ClaimFilingIndicatorCode, relationshipToInsuredCode, dateOfBirth, gender);
                SQLiteCommand command = new SQLiteCommand(insert, dbConnection);
                command.ExecuteNonQuery();

                DocumentLoop billingProviderNameLoop = billingProviderLoop.GetLoop("2010AA");

                NM1 billingProviderName = billingProviderNameLoop.GetSegment<NM1>();

                string officialName = billingProviderName.NameLastOrOrganizationName;
                string npi = billingProviderName.IdentificationCode;

                string OfcTaxID = "";
                string OfcLicense = " ";

                foreach (REF id in billingProviderNameLoop.GetSegments<REF>("REF"))
                {
                    if (id.ReferenceIdentificationQualifier == "EI")
                    {
                        OfcTaxID = id.ReferenceIdentification;
                    }

                    if (id.ReferenceIdentificationQualifier == "0B")
                    {
                        OfcLicense = id.ReferenceIdentification;
                    }
                }

                foreach (DocumentLoop claimLoop in claimLoops)
                {
                    CLM claim = claimLoop.GetSegment<CLM>();

                    string diagnosisCode1 = "";

                    //Get the diagnosis code
                    HI hiInfo = claimLoop.GetSegment<HI>();
                    if (hiInfo != null)
                    {
                        diagnosisCode1 = hiInfo.HealthCareCodeInformation1.IndustryCode1;
                    }

                    //Get IDs
                    string claimNumber = "";
                    string priorAuthorizationNumber = "";

                    foreach (REF id in claimLoop.GetSegments<REF>("REF"))
                    {
                        if (id.ReferenceIdentificationQualifier == "D9")
                        {
                            claimNumber = id.ReferenceIdentification;
                        }

                        if (id.ReferenceIdentificationQualifier == "G3")
                        {
                            priorAuthorizationNumber = id.ReferenceIdentification;
                        }
                    }

                    string missingTooth = "";
                    foreach (DN2 toothStatus in claimLoop.GetSegments<DN2>("DN2"))
                    {
                        missingTooth = toothStatus.ToothNumber;
                        break;//Just get the first status for this demo
                    }

                    string claimNote = "";

                    NTE Note = claimLoop.GetSegment<NTE>();
                    if (Note != null)
                    {
                        claimNote = Note.Description;
                    }
                  
                    //Save the claim to the database
                    string claimTable = String.Format("INSERT INTO Claims (Id, PatientID, ClaimID, Amount, Diagnosis1, OfcName, OfcTaxID) VALUES ({0},{1},'{2}',{3},'{4}','{5}','{6}')", claimRecordID, patientRecordID, claim.ClaimSubmitterIdentifier, claim.MonetaryAmount, diagnosisCode1, officialName, OfcTaxID);
                    command = new SQLiteCommand(claimTable, dbConnection);
                    command.ExecuteNonQuery();

                    //Process the service lines
                    foreach (DocumentLoop serviceLineLoop in claimLoop.GetLoops("2400"))
                    {
                        //Get the service line segment
                        SV3 dentalService = serviceLineLoop.GetSegment<SV3>();
                        TOO toothInformation = serviceLineLoop.GetSegment<TOO>();

                        string toothNumber = "";
                        string toothSurface1 = "";

                        if (toothInformation != null)
                        {
                            toothNumber = toothInformation.ToothCode;

                            if (toothInformation.ToothSurface.ToothSurfaceCode1 != null)
                            {
                                toothSurface1 = toothInformation.ToothSurface.ToothSurfaceCode1;
                            }
                        }
                      
                        string oralCavity1 = "";
                        if (dentalService.OralCavityDesign.OralCavityDesignCode1 != null)
                        {
                            oralCavity1 = dentalService.OralCavityDesign.OralCavityDesignCode1;
                        }

                        //Get the service date
                        string serviceDate = " ";
                        foreach (DTP date in serviceLineLoop.GetSegments<DTP>("DTP"))
                        {
                            if (date.DateTimeQualifier == "472")
                            {
                                serviceDate = date.DateTimePeriod;
                                break;
                            }
                        }

                        //Save the service line to the database
                        string serviceLine = String.Format("INSERT INTO ServiceLines (Id, ClaimID, Procedure, Charge, toothno, servicedate) VALUES ({0},{1},'{2}',{3},'{4}','{5}')", System.Math.Max(System.Threading.Interlocked.Increment(ref serviceLineRecordID), serviceLineRecordID - 1), claimRecordID, dentalService.CompositeMedicalProcedureIdentifier.ProductServiceID1, dentalService.MonetaryAmount, toothNumber, serviceDate);
                        command = new SQLiteCommand(serviceLine, dbConnection);

                        command.ExecuteNonQuery();
                    }

                    claimRecordID += 1;
                }

                patientRecordID += 1;
            }
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            CreateDatabaseTables();

            InitializeEDI();

            LoadEDI();
        }

        private void dgPatients_SelectionChanged(object sender, EventArgs e)
        {
            if (dgPatients.SelectedRows.Count > 0)
            {
                var patientID = dgPatients.SelectedRows[0].Cells[0];

                using (SQLiteCommand cmd = new SQLiteCommand("Select * from Claims where patientID=" + patientID.Value, dbConnection))
                {
                    DataTable dt = new DataTable();
                    dt.Load(cmd.ExecuteReader());
                    dgClaims.DataSource = dt;
                }

                dgClaims.ClearSelection();
                dgServiceLines.ClearSelection();
            }
        }

        private void dgClaims_SelectionChanged(object sender, EventArgs e)
        {
            if (dgClaims.SelectedRows.Count > 0)
            {
                var claimID = dgClaims.SelectedRows[0].Cells[0];

                using (SQLiteCommand cmd = new SQLiteCommand("Select * from ServiceLines where claimID=" + claimID.Value, dbConnection))
                {
                    DataTable dt = new DataTable();
                    dt.Load(cmd.ExecuteReader());
                    dgServiceLines.DataSource = dt;
                }
            }
        }

        private void lnk_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://www.rdpcrystal.com/loading-edi-data-into-a-database/");
        }
    }
}