using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Data;
using System.Data.SQLite;
using System.IO;
using System.Windows.Forms;
using RDPCrystalEDILibrary;
using RDPCrystalEDILibrary.Docs.x5010;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to load and save a 5010 837 Institutional EDI document into a database
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Save_5010_837I_To_Database : UserControl
    {
        private SQLiteConnection dbConnection;
        private EDIValidator ediValidator = new EDIValidator();

        public Save_5010_837I_To_Database()
        {
            InitializeComponent();
        }

        private void CreateDatabaseTables()
        {
            if (dbConnection != null)
            {
                dbConnection.Close();
            }

            File.Delete("MyDatabase.sqlite2");

            //We are going to use SQLLite to demonstrate how to simple load EDI data and save it in a simple database
            SQLiteConnection.CreateFile("MyDatabase.sqlite2");

            dbConnection = new SQLiteConnection("Data Source=MyDatabase.sqlite2;Version=3;");
            dbConnection.Open();

            //Creates the patient's table
            string patientTable = "CREATE TABLE Patients (Id INT PRIMARY KEY, First TEXT, Last TEXT, Insurance TEXT, RelationShip TEXT, InsuranceType TEXT, DOB TEXT, Gender TEXT)";
            string claimTable = "CREATE TABLE Claims (Id INT, PatientID INT, ClaimID TEXT, Amount REAL, Diagnosis1 TEXT)";
            string servicesTable = "CREATE TABLE ServiceLines (Id INT, ClaimID INT, Procedure TEXT, Charge REAL, ServiceDate TEXT)";

            SQLiteCommand command = new SQLiteCommand(patientTable, dbConnection);
            command.ExecuteNonQuery();

            command = new SQLiteCommand(claimTable, dbConnection);
            command.ExecuteNonQuery();

            command = new SQLiteCommand(servicesTable, dbConnection);
            command.ExecuteNonQuery();
        }

        private void InitializeEDI()
        {
            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = "ValidationRules\\Rules_5010_837I_005010X223A2.Rules";

            txtEDIData.Text = File.ReadAllText("EDIFiles\\Sample_5010_837I_For_Database.txt");

            //Set EDI file to validate
            ediValidator.EDIFile = "EDIFiles\\Sample_5010_837I_For_Database.txt";
        }

        private void LoadEDI()
        {
            try
            {
                ediValidator.Validate();

                //The EDI Document will contain all the data of the 837 file
                if (ediValidator.EDILightWeightDocument != null)
                {
                    Typed5010Document doc = new Typed5010Document(ediValidator.EDILightWeightDocument);

                    DocumentLoop stHeaderLoop = doc.MainSection.GetLoop("INTERCHANGE HEADER/FUNCTIONAL GROUP/ST HEADER");

                    //Get the 2000A BILLING/PAY-TO PROVIDER HIERARCHICAL LEVEL.  In an 837 file this loop is 2000A
                    //Then get all the subscribers and claims information.  This is loop 2000B
                    List<DocumentLoop> subscriberAndClaimsSections = stHeaderLoop.GetLoops("2000A/2000B");
               
                    SaveAllClaims(subscriberAndClaimsSections);

                    ////Load the patient data in the grid
                    using (SQLiteCommand cmd = new SQLiteCommand("Select * from Patients", dbConnection))
                    {
                        DataTable dt = new DataTable();
                        dt.Load(cmd.ExecuteReader());
                        dgPatients.DataSource = dt;
                    }

                    //display a tree of the EDI data
                    ediDocumentViewer.LoadDocument(ediValidator.EDILightWeightDocument);
                }
            }
            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        private void SaveAllClaims(List<DocumentLoop> claims)
        {
            int patientRecordID = 1;
            int claimRecordID = 1;
            int serviceLineRecordID = 1;

            foreach (DocumentLoop claimSection in claims)
            {
                SBR subscriber = claimSection.GetSegment<SBR>();
                DocumentLoop subscriberInfo = claimSection.GetLoop("2010BA");
                NM1 subName = subscriberInfo.GetSegment<NM1>();

                DMG demographics = subscriberInfo.GetSegment<DMG>();

                DocumentLoop insuranceInfoSection = claimSection.GetLoop("2010BB");
                NM1 insuranceName = insuranceInfoSection.GetSegment<NM1>();

                string insert = String.Format("INSERT INTO Patients (Id, First, Last, Insurance, Relationship, InsuranceType, DOB, Gender) VALUES ({0},'{1}','{2}','{3}','{4}','{5}','{6}','{7}')",
                patientRecordID, subName.FirstName, subName.NameLastOrOrganizationName, insuranceName.NameLastOrOrganizationName, subscriber.IndividualRelationshipCode, subscriber.ClaimFilingIndicatorCode, demographics.DateTimePeriod, demographics.GenderCode);

                SQLiteCommand command = new SQLiteCommand(insert, dbConnection);
                command.ExecuteNonQuery();

                List<DocumentLoop> claimLoopSections = claimSection.GetLoops("2300");

                foreach (DocumentLoop cl in claimLoopSections)
                {
                    CLM clm = cl.GetSegment<CLM>();
                    HI hi = cl.GetSegment<HI>();

                    string diagnosisCode = hi.HealthCareCodeInformation1.IndustryCode1;

                    string claimTable = String.Format("INSERT INTO Claims (Id, PatientID, ClaimID, Amount, Diagnosis1) VALUES ({0},{1},'{2}',{3},'{4}')",
                     claimRecordID, patientRecordID, clm.ClaimSubmitterIdentifier, clm.MonetaryAmount, diagnosisCode);

                    command = new SQLiteCommand(claimTable, dbConnection);
                    command.ExecuteNonQuery();

                    List<DocumentLoop> serviceLineSections = cl.GetLoops("2400");
                    foreach (DocumentLoop sl in serviceLineSections)
                    {
                        //Get the service line segment
                        SV2 sv1 = sl.GetSegment<SV2>();
                        DTP serviceDate = sl.GetSegment<DTP>();

                        string serviceLine = String.Format("INSERT INTO ServiceLines (Id, ClaimID, Procedure, Charge, ServiceDate) VALUES ({0},{1},'{2}',{3},'{4}')",
                            serviceLineRecordID++, claimRecordID, sv1.CompositeMedicalProcedureIdentifier.ProductServiceID1, sv1.MonetaryAmount1, serviceDate.DateTimePeriod);

                        command = new SQLiteCommand(serviceLine, dbConnection);
                        command.ExecuteNonQuery();
                    }

                    claimRecordID++;
                }

                patientRecordID++;
            }
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            CreateDatabaseTables();
         
            InitializeEDI();

            LoadEDI();
        }

        private void dgPatients_SelectionChanged(object sender, EventArgs e)
        {
            if (dgPatients.SelectedRows.Count > 0)
            {
                var patientID = dgPatients.SelectedRows[0].Cells[0];

                using (SQLiteCommand cmd = new SQLiteCommand("Select * from Claims where patientID=" + patientID.Value, dbConnection))
                {
                    DataTable dt = new DataTable();
                    dt.Load(cmd.ExecuteReader());
                    dgClaims.DataSource = dt;
                }

                dgClaims.ClearSelection();
                dgServiceLines.ClearSelection();
            }
        }

        private void dgClaims_SelectionChanged(object sender, EventArgs e)
        {
            if (dgClaims.SelectedRows.Count > 0)
            {
                var claimID = dgClaims.SelectedRows[0].Cells[0];

                using (SQLiteCommand cmd = new SQLiteCommand("Select * from ServiceLines where claimID=" + claimID.Value, dbConnection))
                {
                    DataTable dt = new DataTable();
                    dt.Load(cmd.ExecuteReader());
                    dgServiceLines.DataSource = dt;
                }
            }
        }

        private void lnk_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://www.rdpcrystal.com/loading-edi-data-into-a-database/");
        }
    }
}