using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using RDPCrystalEDILibrary;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to load and split an 5010 835 document by service provider's NPI number
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class Split_5010_835_By_NPI : UserControl
    {
        private delegate void SetTextDelegate(RichTextBox txtBox, string text);

        private EDIValidator ediValidator = new EDIValidator();

        public Split_5010_835_By_NPI()
        {
            InitializeComponent();

            ediValidator.CopyElementNumber = true;
            ediValidator.ProgressChanged += Validation_ProgressChanged;

            txtFile.Text = File.ReadAllText("EDIFiles\\Sample_5010_835_For_Split.txt");
        }

        private void btnValidate_Click(object sender, EventArgs e)
        {
            if (txtFile.Text.Trim() == String.Empty)
            {
                MessageBox.Show("Please enter EDI data", "RDPCrystal EDI Library");
                return;
            }

            Cursor = Cursors.WaitCursor;

            pbValidator.Value = 0;

            lv.Items.Clear();

            Cursor = Cursors.Default;

            StartEDIValidator();
        }

        private void Validation_ProgressChanged(int progress)
        {
            BeginInvoke(new Action(() =>
            {
                if ((pbValidator.Value < pbValidator.Maximum))
                    pbValidator.Value = progress;
            }));
        }

        private void StartEDIValidator()
        {
            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = "ValidationRules\\Rules_5010_835_005010X221A1.Rules";

            ediValidator.EDIDataString = txtFile.Text;

            Thread validateThread = new Thread(new ThreadStart(ValidateEDI));
            validateThread.Start();
        }

        private void ValidateEDI()
        {
            try
            {
                ediValidator.Validate();

                lv.Items.Clear();

                //Display the error and warnings
                List<ListViewItem> errors = new List<ListViewItem>();
                List<ListViewItem> warnings = new List<ListViewItem>();

                int i = 1;
                foreach (EDIError error in ediValidator.Errors)
                {
                    ListViewItem e = new ListViewItem(new string[] {
                        i.ToString(),
                        error.LineNumber.ToString(),
                        error.SnipLevel.ToString(),
                        error.SegmentPositionInTransaction.ToString(),
                        error.Message.ToString(),
                        error.Loop,
                        error.Segment + (!String.IsNullOrWhiteSpace(error.SegmentDescription) ? (" [" + error.SegmentDescription + "]") : ""),
                        error.ElementOrdinal == 0 ? "" : (error.Segment + error.ElementOrdinal.ToString("D2") + (!String.IsNullOrWhiteSpace(error.ElementDescription) ? " [" + error.ElementDescription + "]" : "")),
                        error.CompositeElementOrdinal == 0 ? "" : (error.CompositeElementOrdinal.ToString("D2") + (!String.IsNullOrWhiteSpace(error.CompositeElementDescription) ? " [" + error.CompositeElementDescription + "]" : "")),
                        error.Description });

                    errors.Add(e);

                    i++;
                }

                foreach (EDIWarning warning in ediValidator.Warnings)
                {
                    ListViewItem w = new ListViewItem(new string[] {
                        i.ToString(),
                        warning.LineNumber.ToString(),
                        warning.SnipLevel.ToString(),
                        warning.SegmentPositionInTransaction.ToString(),
                        warning.Message.ToString(),
                        warning.Loop, warning.Segment,
                       warning.Segment + (!String.IsNullOrWhiteSpace(warning.SegmentDescription) ? (" [" + warning.SegmentDescription + "]") : ""),
                        warning.ElementOrdinal == 0 ? "" : (warning.Segment + warning.ElementOrdinal.ToString("D2") + (!String.IsNullOrWhiteSpace(warning.ElementDescription) ? " [" + warning.ElementDescription + "]" : "")),
                        warning.CompositeElementOrdinal == 0 ? "" : (warning.CompositeElementOrdinal.ToString("D2") + (!String.IsNullOrWhiteSpace(warning.CompositeElementDescription) ? " [" + warning.CompositeElementDescription + "]" : "")),
                        "(WARNING)" + warning.Description });

                    warnings.Add(w);
                    i++;
                }

                //Display the document visually
                this.BeginInvoke(new Action(() =>
                {
                    errors.ForEach(e => lv.Items.Add(e));
                    warnings.ForEach(e => lv.Items.Add(e));

                    ediDocumentViewer1.LoadDocument(ediValidator.EDILightWeightDocument);
                }));

                //Split the data by NPI
                SplitAndSave();
            }
            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        private void SplitAndSave()
        {
            Dictionary<string, EDILightWeightDocument> newFilesToCreate = new Dictionary<string, EDILightWeightDocument>();

            Dictionary<string, LightWeightLoop> documents2000AMap = new Dictionary<string, LightWeightLoop>();
            Dictionary<string, LightWeightLoop> documentsISALoopMap = new Dictionary<string, LightWeightLoop>();
            Dictionary<string, LightWeightLoop> documentsGSLoopMap = new Dictionary<string, LightWeightLoop>();
            Dictionary<string, LightWeightLoop> documentsSTLoopMap = new Dictionary<string, LightWeightLoop>();

            EDILightWeightDocument file835 = ediValidator.EDILightWeightDocument;

            LightWeightLoop interchangeHeaderLoop = file835.Loops[0].GetLoop("INTERCHANGE HEADER");
            //Keep a reference to the end loops so that we can add them to the newly created files
            LightWeightLoop endISALoop = interchangeHeaderLoop.GetLoop("END ISA LOOP");

            LightWeightSegment isa = interchangeHeaderLoop.GetSegment("ISA");

            LightWeightLoop functionalHeaderLoop = interchangeHeaderLoop.GetLoop("FUNCTIONAL GROUP");
            //Keep a reference to the end loops so that we can add them to the newly created files
            LightWeightLoop endGSLoop = functionalHeaderLoop.GetLoop("END GS LOOP");

            LightWeightSegment gs = functionalHeaderLoop.GetSegment("GS");

            LightWeightLoop stHeaderLoop = functionalHeaderLoop.GetLoop("ST HEADER");
            //Keep a reference to the end loops so that we can add them to the newly created files
            LightWeightLoop endSTHeaderLoop = stHeaderLoop.GetLoop("END ST HEADER");

            LightWeightSegment st = stHeaderLoop.GetSegment("ST");

            //Get the Payer ID loop. In an 835 file the submitter loop is 1000A
            LightWeightLoop submitterLoop = stHeaderLoop.GetLoop("1000A");

            //Get the Payee ID loop. In an 835 file the receiver loop is 1000B
            LightWeightLoop receiverLoop = stHeaderLoop.GetLoop("1000B");

            //Get the 2000A Header Loop
            LightWeightLoop headerNumberLoop = stHeaderLoop.GetLoop("2000");

            List<LightWeightLoop> clpLoops = headerNumberLoop.GetLoops("2100");

            //Go through each loop and check the NM1 Service providers
            foreach (LightWeightLoop loop2100 in clpLoops)
            {
                //Get the segment and element we want to split by- Service Provider
                List<LightWeightSegment> serviceProvider = loop2100.GetSegments("NM1", 0, "82");

                string npi = serviceProvider[0].Elements[8].DataValue;

                if (!newFilesToCreate.ContainsKey(npi))
                {
                    EDILightWeightDocument doc = new EDILightWeightDocument();

                    //Set the delimiters
                    doc.Delimiters = ediValidator.Delimiters;

                    //Create the Interchange Envelope
                    LightWeightLoop isaLoop = new LightWeightLoop("INTERCHANGE HEADER");
                    documentsISALoopMap[npi] = isaLoop;
                    isaLoop.Segments.Add(isa);

                    doc.Loops = new List<LightWeightLoop>();
                    doc.Loops.Add(isaLoop);

                    //Create the Functional Header
                    LightWeightLoop gsLoop = isaLoop.CreateLoop("FUNCTIONAL GROUP");
                    documentsGSLoopMap[npi] = gsLoop;
                    gsLoop.Segments.Add(gs);

                    //Create a transaction header loop
                    LightWeightLoop stLoop = gsLoop.CreateLoop("ST HEADER");
                    documentsSTLoopMap[npi] = stLoop;

                    //Copy the segments from the original ST header loops
                    foreach (LightWeightSegment seg in stHeaderLoop.Segments)
                    {
                        stLoop.Segments.Add(seg);
                    }

                    //Just add the Payer and Payee Identification loops
                    stLoop.Loops = new List<LightWeightLoop>();
                    stLoop.Loops.Add(submitterLoop);
                    stLoop.Loops.Add(receiverLoop);

                    LightWeightLoop headerNumLoop = stLoop.CreateLoop("2000");

                    //Copy the segments from the original 2000 (Header Number) Loop
                    foreach (LightWeightSegment seg in headerNumberLoop.Segments)
                    {
                        headerNumLoop.Segments.Add(seg);
                    }

                    headerNumLoop.Loops = new List<LightWeightLoop>();
                    headerNumLoop.Loops.Add(loop2100);

                    newFilesToCreate.Add(npi, doc);

                    documents2000AMap.Add(npi, headerNumLoop);  //keep a reference to the 2000A loop so that we can add more 2100 loops if the NPI matches later
                }
                else
                {
                    documents2000AMap[npi].Loops.Add(loop2100);
                }
            }

            //Ok now let's add the trailer segments for each document

            foreach (var stLoop in documentsSTLoopMap)
            {
                stLoop.Value.Loops.Add(endSTHeaderLoop);
            }

            foreach (var gsLoop in documentsGSLoopMap)
            {
                gsLoop.Value.Loops.Add(endGSLoop);
            }

            foreach (var isaLoop in documentsISALoopMap)
            {
                isaLoop.Value.Loops.Add(endISALoop);
            }

            try
            {
                //Serialize To file system
                foreach (var newEDIDocument in newFilesToCreate)
                {
                    newEDIDocument.Value.NewLineAfterSegment = true;
                    newEDIDocument.Value.GenerateEDIFile(newEDIDocument.Key + ".txt");
                }

                if (this.InvokeRequired)
                {
                    this.BeginInvoke(new Action(DisplayFiles));
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error File Saving File\n Do you have write permission," + ex.Message);
            }
        }

        private void DisplayFiles()
        {
            //Read the new file and display them
            txtNPI1.Text = File.ReadAllText("NPI1.txt");
            txtNPI2.Text = File.ReadAllText("NPI2.txt");
            txtNPI3.Text = File.ReadAllText("NPI3.txt");
        }
    }
}