using System;
using System.Diagnostics;
using System.Drawing;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using RDPCrystalEDILibrary;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how validate an EDI document, display errors and generate a 997/999 Acknowledgement
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class ValidateUserControl : UserControl
    {
        private bool is5010 = true;

        private EDIValidator ediValidator = new EDIValidator();

        public ValidateUserControl(string sampleFilePath, string ediRulesPath, bool is5010 = true, bool highlightAck = false)
        {
            InitializeComponent();

            ediValidator.ProgressChanged += Validation_ProgressChanged;

            this.is5010 = is5010;

            if (highlightAck)
            {
                txtAck.BackColor = Color.LightBlue;
            }

            txtFile.Text = File.ReadAllText(sampleFilePath);

            //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
            //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
            //You can view/edit them using the EDI Rules Creator Studio 
            ediValidator.EDIRulesFile = ediRulesPath;
            ediValidator.OnCodeCondition += new EDIValidator.CodeConditionEvent(ediValidator_CustomCondition);
        }

        private void btnValidate_Click(object sender, EventArgs e)
        {
            if (txtFile.Text.Trim() == String.Empty)
            {
                MessageBox.Show("Please enter EDI data", "RDPCrystal EDI Library");
                return;
            }

            pbValidator.Value = 0;

            lv.Items.Clear();

            StartEDIValidator();
        }

        private void StartEDIValidator()
        {
            ediValidator.EDIDataString = txtFile.Text;

            Thread validateThread = new Thread(new ThreadStart(ValidateEDI));
            validateThread.Start();
        }

        private void ValidateEDI()
        {
            try
            {
                ediValidator.Validate();

                lv.Items.Clear();

                //Display the error and warnings
                List<ListViewItem> errors = new List<ListViewItem>();
                List<ListViewItem> warnings = new List<ListViewItem>();

                int i = 1;
                foreach (EDIError error in ediValidator.Errors)
                {
                    ListViewItem e = new ListViewItem(new string[] {
                        i.ToString(),
                        error.LineNumber.ToString(),
                        error.SnipLevel.ToString(),
                        error.SegmentPositionInTransaction.ToString(),
                        error.Message.ToString(),
                        error.Loop,
                        error.Segment + (!String.IsNullOrWhiteSpace(error.SegmentDescription) ? (" [" + error.SegmentDescription + "]") : ""),
                        error.ElementOrdinal == 0 ? "" : (error.Segment + error.ElementOrdinal.ToString("D2") + (!String.IsNullOrWhiteSpace(error.ElementDescription) ? " [" + error.ElementDescription + "]" : "")),
                        error.CompositeElementOrdinal == 0 ? "" : (error.CompositeElementOrdinal.ToString("D2") + (!String.IsNullOrWhiteSpace(error.CompositeElementDescription) ? " [" + error.CompositeElementDescription + "]" : "")),
                        error.Description });

                    errors.Add(e);

                    i++;
                }

                foreach (EDIWarning warning in ediValidator.Warnings)
                {
                    ListViewItem w = new ListViewItem(new string[] {
                        i.ToString(),
                        warning.LineNumber.ToString(),
                        warning.SnipLevel.ToString(),
                        warning.SegmentPositionInTransaction.ToString(),
                        warning.Message.ToString(),
                        warning.Loop, warning.Segment,
                       warning.Segment + (!String.IsNullOrWhiteSpace(warning.SegmentDescription) ? (" [" + warning.SegmentDescription + "]") : ""),
                        warning.ElementOrdinal == 0 ? "" : (warning.Segment + warning.ElementOrdinal.ToString("D2") + (!String.IsNullOrWhiteSpace(warning.ElementDescription) ? " [" + warning.ElementDescription + "]" : "")),
                        warning.CompositeElementOrdinal == 0 ? "" : (warning.CompositeElementOrdinal.ToString("D2") + (!String.IsNullOrWhiteSpace(warning.CompositeElementDescription) ? " [" + warning.CompositeElementDescription + "]" : "")),
                        "(WARNING)" + warning.Description });

                    warnings.Add(w);

                    i++;
                }

                EDIDocument acknowlegdementDoc = null;
                if (is5010)
                {
                    Ack999Generator ack = new Ack999Generator();
                    acknowlegdementDoc = ack.Generate(ediValidator);
                }
                else
                {
                    Ack997Generator ack = new Ack997Generator();
                    acknowlegdementDoc = ack.Generate(ediValidator);
                }

                this.BeginInvoke(new Action(() =>
                {
                    errors.ForEach(e => lv.Items.Add(e));
                    warnings.ForEach(e => lv.Items.Add(e));

                    txtAck.Text = acknowlegdementDoc.GenerateEDIData();
                    ediDocumentViewer1.LoadDocument(ediValidator.EDILightWeightDocument);

                    if (is5010)
                    {
                        lblAck.Text = "Generated 999 data";
                    }
                    else
                    {
                        lblAck.Text = "Generated 997 data";
                    }
                }));
            }
            catch (ValidationException e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "RDPCrystal EDI Library");
            }
        }

        private void Validation_ProgressChanged(int progress)
        {
            BeginInvoke(new Action(() =>
            {
                if ((pbValidator.Value < pbValidator.Maximum))
                    pbValidator.Value = progress;
            }));
        }

        private void ediValidator_CustomCondition(object sender, RDPCrystalEDILibrary.Rules.CodeConditionEventArgs e)
        {
            //If in our rules file we have a rule like the following
            //SegPos[277:2]=if ( CodeCondition["test"] ) then Error[ElementHasWrongValue,"Code Condition Works"] end
            //When our validator see this rule with it's CodeCondition this event gets called.
            //It is now up to us to check is the condition is valid

            if (e.CodeCondition.Id == "test")
            {
                if (e.CurrentSegment.Elements[0].DataValue == "1")
                {
                    e.ConditionValid = true;
                }
            }
        }

        private void lnk_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://www.rdpcrystal.com/products/edi-library/edi-validator/");
        }
    }
}