using System;
using System.Windows.Forms;
using RDPCrystalEDILibrary;

namespace RDPCrystalEDILibrary_Demo.UserControls
{
    /// <summary>
    /// #PROJECT DESCRIPTION
    /// Demonstrates how to load and display validation rules using the EDIRulesReader component
    /// 
    /// #TECHNICAL SUPPORT
    /// Please contact us at support@rdpcrystal.com. Our support team can help answer your questions
    /// </summary>
    public partial class View_EDI_Rules : UserControl
    {
        private TreeNode treeNode = new TreeNode("Root");

        public View_EDI_Rules()
        {
            InitializeComponent();
        }

        private void btnLoadRules_Click(object sender, EventArgs e)
        {
            try
            {
                EDIRulesReader rulesReader = new EDIRulesReader();

                //Subscribe to the ProgressChanged event
                rulesReader.ProgressChanged += RulesReader_ProgressChanged;

                //Set the EDI rules file
                //EDI Rules files are used by the EDIValidator component to validate and load EDI files.
                //Validation rules are located at \RDPCrystalInstallation\Validation Rules in your installation directory
                //You can view/edit them using the EDI Rules Creator Studio 
                rulesReader.EDIRulesFilePath = "ValidationRules\\Rules_5010_834_005010X220A1.Rules";

                rulesReader.Parse();

                //Copy the rules from the EDI rule file to a treeview
                CreateRuleTreeStructure(rulesReader.Schema, treeNode);

                tvRules.Nodes.Add(treeNode);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "RDPCrystal EDI Library");
            }
        }

        private void RulesReader_ProgressChanged(int progress)
        {
            lblProgress.Text = progress.ToString() + "%";
        }

        private void CreateRuleTreeStructure(Loop main, TreeNode parentNode)
        {
            //Set the loop requirements
            if (main.RepetitionIndefinite)
                parentNode.Text = main.Name + " Repeat>1" + " [" + main.Description + "]";
            else
                parentNode.Text = main.Name + " Repeat=" + main.MaximumOccurrences.ToString() + " [" + main.Description + "]";

            //Go through and add a treenode for each segment
            foreach (DataSegment seg in main.Segments)
            {
                TreeNode childNode = new TreeNode();
                parentNode.Nodes.Add(childNode);

                if (seg.RepeatIndefinite)
                    childNode.Text = seg.Name + "[" + seg.OrdinalNumber.ToString() + "] Repeat=>1 Usage=" + seg.Usage.ToString() + " [" + seg.Description + "]";
                else
                    childNode.Text = seg.Name + "[" + seg.OrdinalNumber.ToString() + "] Repeat=" + seg.MaximumOccurrences + " Usage=" + seg.Usage.ToString() + " [" + seg.Description + "]";

                int i = 1;

                string fs = "{0,5}  {1}  {2}  {3}  {4}  {5}";

                foreach (DataElement element in seg.Elements)
                {
                    TreeNode elemNode = new TreeNode();

                    childNode.Nodes.Add(elemNode);
                    elemNode.Text = String.Format(fs, "[" + seg.Name + i.ToString("D2") + "]", element.ElementNumber.ToString(), "Min:" + element.MinimumLength.ToString(), "Max: " + element.MaximumLength.ToString(), "Data Type=" + element.DataType.ToString(), "Usage=" + element.Usage.ToString());

                    if (element.Composite)
                    {
                        elemNode.Text += " Composite";
                        int j = 1;
                        foreach (DataElement c in element.Elements)
                        {
                            TreeNode comNode = new TreeNode();

                            elemNode.Nodes.Add(comNode);
                            comNode.Text = String.Format(fs, "[" + j.ToString() + "]", c.ElementNumber.ToString(), "Min:" + c.MinimumLength.ToString(), "Max: " + c.MaximumLength.ToString(), "Data Type=" + c.DataType.ToString(), "Usage=" + c.Usage.ToString());

                            if (c.HasAcceptedValues())
                            {
                                TreeNode cvaluesNode = new TreeNode();

                                comNode.Nodes.Add(cvaluesNode);
                                cvaluesNode.Text = "Accepted Values";

                                foreach (string value in c.AcceptedValues)
                                {
                                    TreeNode codeNode = new TreeNode();
                                    cvaluesNode.Nodes.Add(codeNode);

                                    string value1 = value;

                                    if (c.Codes != null && c.Codes.ContainsKey(value1))
                                        value1 = value1 + " : " + c.Codes[value1];

                                    codeNode.Text = value1;
                                }
                            }

                            j++;
                        }
                    }

                    if (element.HasAcceptedValues())
                    {
                        TreeNode valuesNode = new TreeNode();
                        elemNode.Nodes.Add(valuesNode);
                        valuesNode.Text = "Accepted Values";

                        foreach (string value in element.AcceptedValues)
                        {
                            TreeNode codeNode = new TreeNode();
                            valuesNode.Nodes.Add(codeNode);

                            string value1 = value;

                            if (element.Codes != null && element.Codes.ContainsKey(value1))
                                value1 = value1 + " : " + element.Codes[value1];
                            else
                            {
                                if (element.CodeList != null && element.CodeList.Codes.ContainsKey(value1))
                                {
                                    value1 = value1 + " : " + element.CodeList.Codes[value1].Description;
                                }
                            }

                            codeNode.Text = value1;
                        }
                    }

                    i++;
                }

                //Check if the segment has any element pair constraints
                if (seg.HasElementPairConstraints())
                {
                    TreeNode constNode = new TreeNode();

                    childNode.Nodes.Add(constNode);
                    constNode.Text = "Constraints";

                    foreach (ElementPairConstraint p in seg.ElementPairConstraints)
                    {
                        TreeNode elemNode = new TreeNode();

                        constNode.Nodes.Add(elemNode);
                        elemNode.Text = "Pair Contraints:";

                        foreach (int g in p.ElementOrdinals)
                        {
                            elemNode.Text += g.ToString() + ",";
                        }
                    }

                    foreach (ElementConditionalConstraint c in seg.ElementConditionalConstraints)
                    {
                        TreeNode elemNode = new TreeNode();

                        constNode.Nodes.Add(elemNode);
                        elemNode.Text = "Conditional Contraints:";

                        foreach (int g in c.ElementOrdinals)
                        {
                            elemNode.Text += g.ToString() + ",";
                        }
                    }

                    foreach (ElementRequiredConstraint c in seg.ElementRequiredConstraint)
                    {
                        TreeNode elemNode = new TreeNode();

                        constNode.Nodes.Add(elemNode);
                        elemNode.Text = "Required Contraints: At least one element required:";

                        foreach (int g in c.ElementOrdinals)
                        {
                            elemNode.Text += g.ToString() + ",";
                        }
                    }

                    foreach (ElementListConditionalConstraint c in seg.ElementListConditionConstraints)
                    {
                        TreeNode elemNode = new TreeNode();

                        constNode.Nodes.Add(elemNode);
                        elemNode.Text = "Required List Contraints: ";

                        foreach (int g in c.ElementOrdinals)
                        {
                            elemNode.Text += g.ToString() + ",";
                        }
                    }

                    foreach (ElementExclusionConstraint c in seg.ElementExclusionConstraints)
                    {
                        TreeNode elemNode = new TreeNode();

                        constNode.Nodes.Add(elemNode);
                        elemNode.Text = "Exclusion Contraints: ";

                        foreach (int g in c.ElementOrdinals)
                        {
                            elemNode.Text += g.ToString() + ",";
                        }
                    }
                }

                //Check to see if the segment has any rules
                foreach (SemanticRule rule in seg.Rules)
                {
                    TreeNode rulesNode = new TreeNode();
                    childNode.Nodes.Add(rulesNode);
                    rulesNode.Text = "Rule=" + rule.ToString();
                }
            }

            //Process all child loops of the main loop
            foreach (Loop ls in main.Loops)
            {
                TreeNode childNode = new TreeNode();

                parentNode.Nodes.Add(childNode);

                CreateRuleTreeStructure(ls, childNode);
            }
        }
    }
}